@@ -3,7 +3,7 @@
 (* This module declares a data type which represents an Adobe PDF document,
 and defines various simple operations on it. *)
 open Utility
-open Io
+open Pdfio
 
 (* \section{Data Type for Representing PDF Documents} *)
 
@@ -39,6 +39,7 @@ type pdfobject =
   | Stream of (pdfobject * stream) ref
   | Indirect of int
 
+(*IF-OCAML*)
 (* Pdf objects are stored in an efficient map structure. *)
 module PdfObjMap =
   Map.Make
@@ -47,6 +48,13 @@ module PdfObjMap =
        let compare = compare
     end)
 
+let pdfobjmap_find = PdfObjMap.find
+let pdfobjmap_mapi = PdfObjMap.mapi
+let pdfobjmap_iter = PdfObjMap.iter
+let pdfobjmap_remove = PdfObjMap.remove
+let pdfobjmap_add = PdfObjMap.add
+let pdfobjmap_empty = PdfObjMap.empty
+
 (* An object is either lexed, or needs to be lexed from a position in the
 input. *)
 type objectdata =
@@ -56,9 +64,34 @@ type objectdata =
 (* We hold the maximum object number in use, [maxobjnum] to allow easy
 production of new keys for the map. *)
 type pdfobjects =
-  {maxobjnum : int;
-   parse : (PdfObjMap.key -> pdfobject) option;
-   pdfobjects : (objectdata ref * int) PdfObjMap.t} (*r int is generation *)
+  {mutable maxobjnum : int;
+   mutable parse : (PdfObjMap.key -> pdfobject) option;
+   mutable pdfobjects : (objectdata ref * int) PdfObjMap.t} (*r int is generation *)
+(*ENDIF-OCAML*)
+
+(*i*)(*F#
+
+(* An object is either lexed, or needs to be lexed from a position in the
+input. *)
+type objectdata =
+  | Parsed of pdfobject
+  | ToParse
+
+let PdfObjMap : (int, objectdata ref * int) Map.Provider = Map.Make compare
+
+let pdfobjmap_find = PdfObjMap.find
+let pdfobjmap_mapi = PdfObjMap.mapi
+let pdfobjmap_iter = PdfObjMap.iter
+let pdfobjmap_remove = PdfObjMap.remove
+let pdfobjmap_add = PdfObjMap.add
+let pdfobjmap_empty = PdfObjMap.empty
+
+type pdfobjects =
+  {mutable maxobjnum : int;
+   mutable parse : (int -> pdfobject) option;
+   mutable pdfobjects : Tagged.Map<int, (objectdata ref * int), System.Collections.Generic.IComparer<int>>} (*r int is generation *)
+
+F#*)(*i*)
 
 (* \intf PDF Document. The major and minor version numbers, the root object number,
 the list of objects and the trailer dictionary.
@@ -73,11 +106,11 @@ type pdfdoc =
    mutable trailerdict : pdfobject} 
 
 (* \intf The null PDF document. *)
-let empty =
+let empty () =
   {major = 1;
    minor = 0;
    root = 0;
-   objects = {maxobjnum = 0; parse = None; pdfobjects = PdfObjMap.empty};
+   objects = {maxobjnum = 0; parse = None; pdfobjects = pdfobjmap_empty};
    trailerdict = Dictionary []}
 
 (* \intf General exception for low-level errors. *)
@@ -96,23 +129,26 @@ let getstream = function
       if l = 0L then stream := (d, Got (mkstream 0)) else
         let s = mkstream (i64toi l) in
           begin try
-            i.seek_in o;
+            (*IF-OCAML*)i.seek_in o; (*ENDIF-OCAML*)
+            (*i*)(*F#i.seek_in (i64toi o); F#*)(*i*)
             for c = 0 to i64toi l - 1 do
-              s.{c} <- i.input_byte ()
+              match i.input_byte () with
+              | b when b = Pdfio.no_more -> dpr "H"; raise End_of_file
+              | b -> sset s c b
             done;
             stream := (d, Got s)
           with
-            EndOfInput ->
+            End_of_file ->
               raise (PDFError "Pdf.getstream: can't read stream.")
           end
   | Stream _ -> ()
   | _ -> raise (PDFError "Pdf.getstream: not a stream")
 
-let recurse_array f elts =
+let recurse_array (f : pdfobject -> pdfobject) elts =
   Array (map f elts)
 
 (* \intf Similarly for dictionaries. *)
-let recurse_dict f elts =
+let recurse_dict (f : pdfobject -> pdfobject) elts =
   let names, objects = split elts in
     let objects' = map f objects in
       Dictionary (combine names objects')
@@ -137,7 +173,7 @@ let parse_rectangle = function
   | _ -> raise (PDFError "Pdf.parse_rectangle: not a rectangle")
 
 let change_obj doc i obj =
-  match fst (PdfObjMap.find i doc.objects.pdfobjects) with
+  match fst (pdfobjmap_find i doc.objects.pdfobjects) with
   | {contents = Parsed _} -> assert false
   | {contents = ToParse} as r -> r := Parsed obj
 
@@ -145,7 +181,7 @@ let change_obj doc i obj =
 it is ready-parsed should it be required again. *)
 let parse_lazy pdf n =
   match pdf.objects.parse with
-  | None -> assert false
+  | None -> raise (Assert_failure ("Pdf.parse_lazy", 0, 0))
   | Some f ->
       let obj = f n in
         change_obj pdf n obj;
@@ -154,11 +190,11 @@ let parse_lazy pdf n =
 (* \intf Look up an object. On an error return [Pdf.Null] *)
 let lookup_obj doc i =
   try
-    match fst (PdfObjMap.find i doc.objects.pdfobjects) with
+    match fst (pdfobjmap_find i doc.objects.pdfobjects) with
     | {contents = Parsed obj} -> obj
     | {contents = ToParse} -> parse_lazy doc i
   with
-    Not_found -> Null
+    Not_found -> dpr "2H"; Null
 
 let catalog_of_pdf pdf =
   try lookup_obj pdf pdf.root with
@@ -169,11 +205,11 @@ direct object. A hanging indirect is defined as [Null]. *)
 let rec direct pdf = function
   | Indirect i ->
       begin try
-        match fst (PdfObjMap.find i pdf.objects.pdfobjects) with
+        match fst (pdfobjmap_find i pdf.objects.pdfobjects) with
         | {contents = Parsed pdfobject} -> direct pdf pdfobject
         | {contents = ToParse} -> parse_lazy pdf i
       with
-        Not_found -> Null
+        Not_found -> dpr "2I"; Null
       end
   | obj -> obj
 
@@ -188,7 +224,7 @@ let map_stream f pdf =
   in
     let objects' =
       {pdf.objects with
-         pdfobjects = PdfObjMap.mapi (map_stream_inner f) pdf.objects.pdfobjects}
+         pdfobjects = pdfobjmap_mapi (map_stream_inner f) pdf.objects.pdfobjects}
     in
       {pdf with objects = objects'}
 
@@ -201,7 +237,7 @@ let iter_stream f pdf =
         iter_stream_inner f i (r, g)
     | _ -> ()
   in
-    PdfObjMap.iter (iter_stream_inner f) pdf.objects.pdfobjects
+    pdfobjmap_iter (iter_stream_inner f) pdf.objects.pdfobjects
 
 (* \intf Lookup a key in a dictionary, following indirect references,  returning
 [None] on any failure. This works on both plain dictionaries and streams. *)
@@ -223,14 +259,14 @@ let lookup_direct_orelse pdf k k' d =
 (* \intf Look something up in a dictionary, failing with given exception if not
 found. We make direct both the dictionary and the result of the lookup. This
 also allows us to look things up in a stream dictionary transparently. *)
-let lookup_exception exp pdf key dict =
+let lookup_exception (exp : exn) pdf key dict =
   let dict' =
     match direct pdf dict with
     | Dictionary d | Stream {contents = Dictionary d, _} -> d
     | o -> raise (PDFError "not a dictionary")
   in
     match lookup key dict' with
-    | None -> raise exp
+    | None -> dpr "G"; raise exp
     | Some v -> direct pdf v
 
 (* \intf A specialised one raising [PDFError]. *)
@@ -262,7 +298,7 @@ let objiter f doc =
     | {contents = Parsed obj}, _ -> f k obj
     | {contents = ToParse}, _ -> f k (parse_lazy doc k)
   in
-    PdfObjMap.iter f' doc.objects.pdfobjects
+    pdfobjmap_iter f' doc.objects.pdfobjects
 
 (* \intf Same, but also pass generation number. *)
 let objiter_gen f doc =
@@ -271,7 +307,7 @@ let objiter_gen f doc =
     | {contents = Parsed obj}, g -> f k g obj
     | {contents = ToParse}, g -> f k g (parse_lazy doc k)
   in
-    PdfObjMap.iter f' doc.objects.pdfobjects
+    pdfobjmap_iter f' doc.objects.pdfobjects
 
 (* \intf Map on objects. *)
 let objmap f doc =
@@ -281,7 +317,7 @@ let objmap f doc =
   in
     {doc with objects =
        {doc.objects with
-          pdfobjects = PdfObjMap.mapi f' doc.objects.pdfobjects}}
+          pdfobjects = pdfobjmap_mapi f' doc.objects.pdfobjects}}
 
 let maxobjnum pdf =
   pdf.objects.maxobjnum
@@ -289,7 +325,7 @@ let maxobjnum pdf =
 (* Return a list of object numbers. *)
 let objnumbers pdf =
   let keys = ref [] in
-    objiter (fun k _ -> keys =:: k) pdf;
+    objiter (fun k _ -> keys =| k) pdf;
     rev !keys
 
 (* \intf Cardinality of object set. O(n). *)
@@ -301,55 +337,46 @@ let objcard pdf =
 (* Remove an object. *)
 let removeobj doc o =
   {doc with objects =
-    {doc.objects with pdfobjects = PdfObjMap.remove o doc.objects.pdfobjects}}
+    {doc.objects with pdfobjects = pdfobjmap_remove o doc.objects.pdfobjects}}
 
 (* Return a list of (k, v) pairs. *)
 let list_of_objs doc =
   let objs = ref [] in
-    objiter (fun k v -> objs =:: (k, Parsed v)) doc;
+    objiter (fun k v -> objs =| (k, Parsed v)) doc;
     !objs
 
+(* \intf Add an object, given an object number. *)
+let addobj_given_num doc (num, obj) =
+  doc.objects.maxobjnum <- max doc.objects.maxobjnum num;
+  doc.objects.pdfobjects <- pdfobjmap_add num (ref (Parsed obj), 0) doc.objects.pdfobjects
+
 (* \intf Add an object. We use the first number larger than the maxobjnum, and update that. *)
 let addobj doc obj =
-  let new_objnum = doc.objects.maxobjnum + 1 in
-    let new_doc = 
-      {doc with objects =
-        {doc.objects with
-           maxobjnum =
-             new_objnum;
-           pdfobjects =
-             PdfObjMap.add new_objnum (ref (Parsed obj), 0) doc.objects.pdfobjects}}
-    in
-      new_doc, new_objnum
-
-(* \intf The same, but use a given object number. *)
-let addobj_given_num ?(gen=0) doc (num, obj) =
-  {doc with objects =
-     {doc.objects with
-        maxobjnum =
-          max doc.objects.maxobjnum num;
-        pdfobjects =
-          PdfObjMap.add num (ref (Parsed obj), gen) doc.objects.pdfobjects}}
+  let num = doc.objects.maxobjnum + 1 in
+    addobj_given_num doc (num, obj);
+    num
 
 (* Make a objects entry from a list of (number, object) pairs. *)
 let objects_of_list parse l =
   let maxobj = ref 0
-  and map = ref PdfObjMap.empty in
+  and map = ref pdfobjmap_empty in
     iter
       (fun (k, v) ->
+         (*i Printf.printf "Adding object %i to PDF\n" k; i*)
          maxobj := max !maxobj k;
-         map := PdfObjMap.add k v !map)
+         map := pdfobjmap_add k v !map)
       l;
     {parse = parse; pdfobjects = !map; maxobjnum = !maxobj}
 
 (* Renumber an object given a change table (A hash table mapping old to new
 numbers). *)
-let rec renumber_object_parsed pdf changes obj =
+let rec renumber_object_parsed (pdf : pdfdoc) changes obj =
   match obj with
   | Indirect i ->
       let i' =
-        try Hashtbl.find changes i with
-          Not_found -> i (*r A dangling indirect is valid. *)
+        match tryfind changes i with
+        | Some x -> x
+        | None -> i (*r A dangling indirect is valid. *)
       in
         Indirect i'
   | Array a ->
@@ -369,7 +396,7 @@ let renumber_object pdf changes objnum = function
 (* Perform all renumberings given by a change table. *)
 let renumber change_table pdf =
   let root' =
-    try Hashtbl.find change_table pdf.root with Not_found -> pdf.root
+    match tryfind change_table pdf.root with Some x -> x | None -> pdf.root
   and trailerdict' =
     renumber_object pdf change_table 0 (Parsed pdf.trailerdict)
   and objects' =
@@ -377,7 +404,7 @@ let renumber change_table pdf =
       let objs' =
         map2 (renumber_object pdf change_table) nums objs
       and nums' =
-        map (fun k -> try Hashtbl.find change_table k with _ -> k) nums
+        map (function k -> match tryfind change_table k with Some x -> x | None -> k) nums
       in
         objects_of_list
           pdf.objects.parse
@@ -395,19 +422,21 @@ be used on PDFs where the generation numbers still matter (i.e before
 decryption). *)
 let renumber_pdfs pdfs =
   let keylists = map objnumbers pdfs
-  and base = ref 1
+  and bse = ref 1
   and tables = ref [] in
     iter
       (fun k ->
          let length = length k in
            let table = Hashtbl.create length in
-             List.iter2 (Hashtbl.add table) k (ilist !base (!base + length - 1));
-             tables =:: table;
-             base += length)
+             List.iter2 (Hashtbl.add table) k (ilist !bse (!bse + length - 1));
+             tables =| table;
+             bse += length)
       keylists;
     map2 renumber (rev !tables) pdfs
 
 (* Used for sets of object numbers. *)
+
+(*IF-OCAML*)
 module RefSet =
   Set.Make
     (struct
@@ -415,90 +444,85 @@ module RefSet =
        let compare = compare
     end)
 
+let refset_add = RefSet.add
+let refset_empty = RefSet.empty
+let refset_elements = RefSet.elements
+(*ENDIF-OCAML*)
+
+(*i*)(*F#
+let RefSet : int Set.Provider = Set.Make compare
+
+let refset_add = RefSet.add
+let refset_empty = RefSet.empty
+let refset_elements = RefSet.elements
+F#*)(*i*)
+
 (* Give a list of object numbers referenced in a given [pdfobject] *)
-let rec
-  referenced ?(don't_follow_dict_entries = []) ?(don't_follow_if_dict_contains = []) pdf found i
-= function
+let rec referenced no_follow_entries no_follow_contains pdf found i = function
   | Parsed (Indirect i) ->
       if not (RefSet.mem i !found) then
         begin
           let obj = 
             try lookup_obj pdf i with
-              Not_found -> Null
+              Not_found -> dpr "2M"; Null
           in
             match obj with
             | Dictionary d ->
-                if not (member true (map (member' don't_follow_if_dict_contains) d)) then
+                if not (mem true (map (mem' no_follow_contains) d)) then
                   begin
                   found := RefSet.add i !found;
-                  referenced
-                    ~don't_follow_dict_entries ~don't_follow_if_dict_contains pdf found i (Parsed obj)
+                  referenced no_follow_entries no_follow_contains pdf found i (Parsed obj)
                   end
             | _ ->
               found := RefSet.add i !found;
-              referenced
-                ~don't_follow_dict_entries ~don't_follow_if_dict_contains pdf found i (Parsed obj)
+              referenced no_follow_entries no_follow_contains pdf found i (Parsed obj)
         end
   | Parsed (Array a) ->
       iter
-        (referenced
-          ~don't_follow_dict_entries ~don't_follow_if_dict_contains pdf found i)
+        (referenced no_follow_entries no_follow_contains pdf found i)
         (map (fun x -> Parsed x) a)
   | Parsed (Dictionary d) ->
       iter
-        (referenced
-           ~don't_follow_dict_entries ~don't_follow_if_dict_contains pdf found i)
+        (referenced no_follow_entries no_follow_contains pdf found i)
         (map
           (fun x -> Parsed (snd x))
-          (lose (fun (k, _) -> member k don't_follow_dict_entries) d))
+          (lose (fun (k, _) -> mem k no_follow_entries) d))
   | Parsed (Stream s) ->
-      referenced
-        ~don't_follow_dict_entries ~don't_follow_if_dict_contains pdf found i (Parsed (fst !s))
+      referenced no_follow_entries no_follow_contains pdf found i (Parsed (fst !s))
   | Parsed _ ->
       ()
   | ToParse ->
-      referenced
-        ~don't_follow_dict_entries ~don't_follow_if_dict_contains pdf found i
-        (Parsed (parse_lazy pdf i))
+      referenced no_follow_entries no_follow_contains pdf found i (Parsed (parse_lazy pdf i))
 
 (* \intf Remove any unreferenced objects. *)
 let remove_unreferenced pdf =
   let found = ref RefSet.empty in
-    referenced pdf found pdf.root (Parsed (lookup_obj pdf pdf.root));
-    referenced pdf found 0 (Parsed pdf.trailerdict);
+    referenced [] [] pdf found pdf.root (Parsed (lookup_obj pdf pdf.root));
+    referenced [] [] pdf found 0 (Parsed pdf.trailerdict);
     found := RefSet.add pdf.root !found;
     let eltnumbers = RefSet.elements !found in
       (* If not found, just ignore. *)
       let elements =
         map
-          (fun n -> try lookup_obj pdf n with Not_found -> Null)
+          (fun n -> try lookup_obj pdf n with Not_found -> dpr "2N"; Null)
           eltnumbers
       in
-        fold_left
-          addobj_given_num
-          {pdf with 
-             objects =
-               {maxobjnum = 0;
-                parse = pdf.objects.parse;
-                pdfobjects = PdfObjMap.empty}}
-          (combine eltnumbers elements)
+        pdf.objects <-
+          {maxobjnum = 0;
+           parse = pdf.objects.parse;
+           pdfobjects = pdfobjmap_empty};
+        iter (addobj_given_num pdf) (combine eltnumbers elements)
 
 (* \intf Objects referenced from a given one. *)
-let objects_referenced
-  ?(don't_follow_dict_entries = []) ?(don't_follow_if_dict_contains = []) pdf pdfobject
-=
+let objects_referenced no_follow_entries no_follow_contains pdf pdfobject =
   let set = ref RefSet.empty in
-    referenced
-      ~don't_follow_dict_entries ~don't_follow_if_dict_contains pdf set 0 (Parsed pdfobject);
+    referenced no_follow_entries no_follow_contains pdf set 0 (Parsed pdfobject);
     RefSet.elements !set
 
 (* \intf The same, but return the objects too. *)
-let objects_referenced_and_objects
-  ?(don't_follow_dict_entries = []) ?(don't_follow_if_dict_contains = []) pdf pdfobject
-=
+let objects_referenced_and_objects no_follow_entries no_follow_contains pdf pdfobject =
   let nums =
-    objects_referenced
-    ~don't_follow_dict_entries ~don't_follow_if_dict_contains pdf pdfobject
+    objects_referenced no_follow_entries no_follow_contains pdf pdfobject
   in
     combine nums (map (lookup_obj pdf) nums)
 
@@ -548,16 +572,22 @@ let unique_key prefix obj =
     let names = fst (split elts)
     and name_of_num n = "/" ^ prefix ^ string_of_int n
     and num = ref 0 in
-      while member (name_of_num !num) names do incr num done;
+      while mem (name_of_num !num) names do incr num done;
       name_of_num !num
 
 (* \intf Given a PDF and potential filename, calculate an MD5 string and build a
 suitable /ID entry from it. *)
-let generate_id pdf path =
-  let digest =
-    Digest.string (path ^ string_of_float (Unix.gettimeofday ()))
+let generate_id (pdf : pdfdoc) (path : string) =
+  (*IF-OCAML*)
+  let gettime () = Unix.gettimeofday () in
+  (*ENDIF-OCAML*)
+  (*F#
+  let gettime () = Sys.time () in
+  F#*)
+  let d =
+    digest (path ^ string_of_float (gettime ()))
   in
-    Array [String digest; String digest]
+    Array [String d; String d]
 
 (* Find the page reference numbers, given the top level node of the page tree *)
 let rec page_reference_numbers_inner pdf pages_node node_number =
@@ -593,7 +623,7 @@ let reference_numbers_of_dict_entry pdf dict entry =
   match dict with
   | Dictionary d ->
       begin match lookup entry d with
-      | Some x -> objects_referenced pdf x
+      | Some x -> objects_referenced [] [] pdf x
       | None ->
           raise (PDFError "reference_numbers_of_dict_entry: no entry")
       end
