@@ -4,7 +4,7 @@
 [Io.input] *)
 type stream =
   | Got of Utility.bytestream
-  | ToGet of Io.input * int64 * int64
+  | ToGet of Pdfio.input * int64 * int64
 
 (** PDF objects. *)
 type pdfobject =
@@ -31,9 +31,9 @@ type pdfdoc =
    mutable objects : pdfobjects;
    mutable trailerdict : pdfobject}
 
-(** The empty document (PDF 1.0, no objects, no root, empty trailer dictionary.
+(** The empty document (PDF 1.0, no objects, no root, empty trailer dictionary).
 Note this is not a well-formed PDF. *)
-val empty : pdfdoc
+val empty : unit -> pdfdoc
 
 (** Any function may return this. *)
 exception PDFError of string
@@ -63,7 +63,7 @@ val lookup_obj : pdfdoc -> int -> pdfobject
 (** [lookup_fail errtext doc key dict] looks up a key in a PDF dictionary or the
 dictionary of a PDF stream. Fails with [PDFError errtext] if the key is not
 found. Follows indirect object links. *)
-val lookup_fail : string -> pdfdoc -> string -> pdfobject -> pdfobject
+val lookup_fail : string -> (pdfdoc -> string -> pdfobject -> pdfobject)
 
 (** Same, but with customised exception. *)
 val lookup_exception : exn -> pdfdoc -> string -> pdfobject -> pdfobject
@@ -101,11 +101,11 @@ val objmap : (pdfobject -> pdfobject) -> pdfdoc -> pdfdoc
 (** Return the cardinality of the object map. *)
 val objcard : pdfdoc -> int
 
-(** Add an object. Returns the ammended document, and the number chosen. *)
-val addobj : pdfdoc -> pdfobject -> pdfdoc * int
+(** Add an object. Returns the number chosen. *)
+val addobj : pdfdoc -> pdfobject -> int
 
 (** Same, but pick a number ourselves. *)
-val addobj_given_num : ?gen:int -> pdfdoc -> (int * pdfobject) -> pdfdoc
+val addobj_given_num : pdfdoc -> (int * pdfobject) -> unit
 
 (** Map over just the stream objects in a document. *)
 val map_stream : (pdfobject -> pdfobject) -> pdfdoc -> pdfdoc
@@ -118,7 +118,7 @@ then be merged etc. without clashes. *)
 val renumber_pdfs : pdfdoc list -> pdfdoc list
 
 (** Garbage-collect a pdf document. *)
-val remove_unreferenced : pdfdoc -> pdfdoc
+val remove_unreferenced : pdfdoc -> unit
 
 (** Given a dictionary and a prefix (e.g gs), return a name, starting with the
 prefix, which is not already in the dictionary (e.g /gs0). *)
@@ -148,10 +148,7 @@ val maxobjnum : pdfdoc -> int
 val objects_of_list :
   (int -> pdfobject) option -> (int * (objectdata ref * int)) list -> pdfobjects
 
-val objects_referenced :
-  ?don't_follow_dict_entries:string list ->
-  ?don't_follow_if_dict_contains:(string * pdfobject) list ->
-  pdfdoc -> pdfobject -> int list
+val objects_referenced : string list -> (string * pdfobject) list -> pdfdoc -> pdfobject -> int list
 
 (** Generate and ID for a PDF document given its prospective file name (and using
 the current date and time) . If the file name is blank, the ID is still likely to
@@ -165,9 +162,7 @@ val page_reference_numbers : pdfdoc -> int list
 val reference_numbers_of_dict_entry : pdfdoc -> pdfobject -> string -> int list
 
 val objects_referenced_and_objects :
-  ?don't_follow_dict_entries:string list ->
-  ?don't_follow_if_dict_contains:(string * pdfobject) list ->
-  pdfdoc -> pdfobject -> (int * pdfobject) list
+  string list -> (string * pdfobject) list -> pdfdoc -> pdfobject -> (int * pdfobject) list
 
 val catalog_of_pdf : pdfdoc -> pdfobject
 
