@@ -1,25 +1,28 @@
 (* \chaptertitle{PDFCodec}{PDF compression and decompression} *)
 open Utility
-open Io
+open Pdfio
 open Pdf
 
 (* \section{Preliminaries} *)
 
 (* Get the next non-whitespace character in a stream. *)
 let rec get_streamchar skipped s =
-  let chr = s.input_char () in
-    if is_whitespace chr then
-      begin
-        incr skipped;
-        get_streamchar skipped s
-      end
-    else chr
+  match s.input_byte () with
+  | x when x = Pdfio.no_more -> dpr "I"; raise End_of_file
+  | x ->
+    let chr = char_of_int x in
+      if is_whitespace chr then
+        begin
+          incr skipped;
+          get_streamchar skipped s
+        end
+      else chr
 
 (* Same, but return an option type instead of raising an exception at end of
 input. *)
 let get_streamchar_option skipped s =
   try Some (get_streamchar skipped s) with
-    EndOfInput -> None
+    End_of_file -> dpr "2O"; None
 
 (* \intf Raised if there was bad data. *)
 exception Couldn'tDecodeStream of string
@@ -34,15 +37,20 @@ convert this to the output stream. *)
 let encode_ASCIIHex stream =
   let size = stream_size stream in
     let stream' = mkstream (size * 2 + 1) in
-      stream'.{size * 2} <- int_of_char '>'; (*r ['>'] is end-of-data *)
+      sset stream' (size * 2) (int_of_char '>'); (*r ['>'] is end-of-data *)
       for p = 0 to size - 1 do
-        let chars = explode (Printf.sprintf "%02X" stream.{p}) in
-          stream'.{p * 2} <- int_of_char (hd chars);
-          stream'.{p * 2 + 1} <- int_of_char (hd (tl chars))
+        let chars = explode (Printf.sprintf "%02X" (sget stream p)) in
+          sset stream' (p * 2) (int_of_char (hd chars));
+          sset stream' (p * 2 + 1) (int_of_char (hd (tl chars)))
       done;
       stream'
 
 (* Decode ASCIIHex *)
+
+(* Calulate a character from two hex digits a and b *)
+let char_of_hex a b =
+  char_of_int (int_of_string ("0x" ^ string_of_char a ^ string_of_char b))
+
 let decode_ASCIIHex i =
   let output = ref []
   and enddata = ref false in
@@ -53,21 +61,20 @@ let decode_ASCIIHex i =
             match b, b' with
             | '>', _ -> set enddata
             | ('0'..'9' | 'a'..'f' | 'A'..'F') as c, '>' ->
-                output =::
-                  char_of_int (Scanf.sscanf (implode [c; '0']) "%x" ident);
+                output =| char_of_hex c '0';
                 set enddata
             | ('0'..'9' | 'a'..'f' | 'A'..'F' as c),
               ('0'..'9' | 'a'..'f' | 'A'..'F' as c') ->
-                output =::
-                  char_of_int (Scanf.sscanf (implode [c; c']) "%x" ident)
+                output =| char_of_hex c c'
             | _ -> raise Not_found (*r Bad data. *)
       done;
       bytestream_of_charlist (rev !output)
     with
-      | EndOfInput ->
+      | End_of_file ->
           (* We ran out of data. This is a normal exit. *)
+          dpr "J";
           bytestream_of_charlist (rev !output)
-      | Not_found | Scanf.Scan_failure _  ->
+      | Not_found ->
           raise (Couldn'tDecodeStream "ASCIIHex")
 
 (* \section{ASCII85}*)
@@ -85,7 +92,7 @@ let decode_5bytes (c1, c2, c3, c4, c5) =
       in
         extract 24, extract 16, extract 8, extract 0
 
-(* Main function *)
+(* Main function *)        
 let decode_ASCII85 i =
   let output = ref []
   and enddata = ref false
@@ -101,33 +108,34 @@ let decode_ASCII85 i =
         let c3 = get_streamchar_option skipped i in
         let c4 = get_streamchar_option skipped i in
         let c5 = get_streamchar_option skipped i in
-          match c1, c2, c3, c4, c5 with
-          | Some 'z', _, _, _, _ ->
-              i.seek_in (i64sub (i.pos_in ()) (i64add 4L (i64ofi !skipped)));
-              output := '\000'::'\000'::'\000'::'\000'::!output
-          | Some ('!'..'u' as c1), Some ('!'..'u' as c2), Some ('!'..'u' as c3),
-            Some ('!'..'u' as c4), Some ('!'..'u' as c5) ->
-              let b1, b2, b3, b4 = decode_5bytes (c1, c2, c3, c4, c5) in
-                output := b4::b3::b2::b1::!output
-          | Some '~', Some '>', _, _, _ ->
-              set enddata
-          | Some ('!'..'u' as c1), Some ('!'..'u' as c2),
-            Some '~', Some '>', _ ->
-              let b1, b2, b3, b4 = decode_5bytes (c1, c2, '~', '>', '!') in
-                set enddata; output := b1::!output
-          | Some ('!'..'u' as c1), Some ('!'..'u' as c2), Some ('!'..'u' as c3),
-            Some '~', Some '>' ->
-              let b1, b2, _, _ = decode_5bytes (c1, c2, c3, '~', '>') in
-                set enddata; output := b2::b1::!output
-          | Some ('!'..'u' as c1), Some ('!'..'u' as c2), Some ('!'..'u' as c3),
-            Some ('!'..'u' as c4), Some '~' ->
-              let b1, b2, b3, _ = decode_5bytes (c1, c2, c3, c4, '~') in
-                set enddata; output := b3::b2::b1::!output
-          | _ -> raise EndOfInput
+          let ischar c = c >= '!' && c <= 'u' in
+            match c1, c2, c3, c4, c5 with
+            | Some 'z', _, _, _, _ ->
+                i.seek_in (possub (i.pos_in ()) (posadd (posofi 4) (posofi !skipped)));
+                output := '\000'::'\000'::'\000'::'\000'::!output
+            | Some c1, Some c2, Some c3, Some c4, Some c5 when
+                ischar c1 && ischar c2 && ischar c3 && ischar c4 && ischar c5 ->
+                  let b1, b2, b3, b4 = decode_5bytes (c1, c2, c3, c4, c5) in
+                    output := b4::b3::b2::b1::!output
+            | Some '~', Some '>', _, _, _ ->
+                set enddata
+            | Some c1, Some c2, Some '~', Some '>', _
+                when ischar c1 && ischar c2 ->
+                  let b1, b2, b3, b4 = decode_5bytes (c1, c2, '~', '>', '!') in
+                    set enddata; output := b1::!output
+            | Some c1, Some c2, Some c3, Some '~', Some '>'
+                when ischar c1 && ischar c2 && ischar c3 ->
+                  let b1, b2, _, _ = decode_5bytes (c1, c2, c3, '~', '>') in
+                    set enddata; output := b2::b1::!output
+            | Some c1, Some c2, Some c3, Some c4, Some '~'
+                when ischar c1 && ischar c2 && ischar c3 && ischar c4 ->
+                  let b1, b2, b3, _ = decode_5bytes (c1, c2, c3, c4, '~') in
+                    set enddata; output := b3::b2::b1::!output
+            | _ -> raise End_of_file
       done;
       bytestream_of_charlist (rev !output)
     with
-      EndOfInput -> raise (Couldn'tDecodeStream "ASCII85")
+      End_of_file -> raise (Couldn'tDecodeStream "ASCII85")
 
 (* Encode a single symbol set. *)
 let encode_4bytes = function
@@ -151,7 +159,7 @@ let encode_4bytes = function
                 let t, c3 = t - rem t three85, rem t three85 / two85 in
                   let t, c2 = t - rem t four85, rem t four85 / three85 in
                     i64toi (t / four85), i64toi c2, i64toi c3, i64toi c4, i64toi c5
-  | _ -> assert false
+  | _ -> raise (Assert_failure ("encode_4bytes", 0, 0))
 
 (* Encode a stream. *)
 let encode_ASCII85 stream =
@@ -159,10 +167,10 @@ let encode_ASCII85 stream =
   and enddata = ref false
   and istream = input_of_bytestream stream in
     while not !enddata do
-      let b1 = try Some (istream.input_char ()) with EndOfInput -> None in
-      let b2 = try Some (istream.input_char ()) with EndOfInput -> None in
-      let b3 = try Some (istream.input_char ()) with EndOfInput -> None in
-      let b4 = try Some (istream.input_char ()) with EndOfInput -> None in
+      let b1 = istream.input_char () in
+      let b2 = istream.input_char () in
+      let b3 = istream.input_char () in
+      let b4 = istream.input_char () in
         match b1, b2, b3, b4 with
         | Some b1, Some b2, Some b3, Some b4 ->
             output := [b1; b2; b3; b4]::!output
@@ -207,11 +215,13 @@ let bytestream_of_strings strings =
       iter
         (fun str ->
            for x = 0 to String.length str - 1 do
-             s.{!pos} <- int_of_char str.[x]; incr pos
+             sset s !pos (int_of_char str.[x]); incr pos
            done)
         strings;
       s
 
+
+(*IF-OCAML*)
 let flate_process f data =
   let strings = ref []
   and pos = ref 0
@@ -221,29 +231,32 @@ let flate_process f data =
          let s = String.length buf in
            let towrite = min (inlength - !pos) s in
              for x = 0 to towrite - 1 do
-               buf.[x] <- char_of_int data.{!pos}; incr pos
+               buf.[x] <- char_of_int (sget data !pos); incr pos
              done;
              towrite)
     and output =
       (fun buf length ->
-         if length > 0 then strings =:: String.sub buf 0 length)
+         if length > 0 then strings =| String.sub buf 0 length)
     in
       f input output;
       bytestream_of_strings (rev !strings)
 
-(* When decoding from an input (for an inline image), we proceed one byte at a
-time so as to leave the input pointer at the correct place. *)
 let decode_flate_input i =
   let strings = ref [] in
     let input =
       (fun buf ->
          let s = String.length buf in
            if s > 0 then
-             begin buf.[0] <- i.input_char (); 1 end
+             begin
+               match i.input_byte () with
+               | x when x = Pdfio.no_more -> raise End_of_file
+               | x ->
+                   buf.[0] <- char_of_int x; 1
+             end
            else 0)
     and output =
       (fun buf length ->
-         if length > 0 then strings =:: String.sub buf 0 length)
+         if length > 0 then strings =| String.sub buf 0 length)
     in
       Zlib.uncompress input output;
       bytestream_of_strings (rev !strings)
@@ -254,6 +267,78 @@ let encode_flate stream =
 let decode_flate stream =
   try flate_process Zlib.uncompress stream with
     Zlib.Error (a, b) -> raise (Couldn'tDecodeStream "Flate")
+(*ENDIF-OCAML*)
+
+(*i*)(*F#
+(* When decoding from an input (for an inline image), we proceed one byte at a
+time so as to leave the input pointer at the correct place. *)
+let io_of_pdfio i =
+  let read () =
+    match i.input_byte () with
+    | x when x = Pdfio.no_more -> raise Io.No_more_input
+    | x -> char_of_int x
+  and input s i j =
+    0
+  and close () =
+    ()
+  in
+    Io.create_in read input close
+
+(* Encode a bigarray using the ZSharpLib library, returning another bigarray. *)
+let encode_flate s =
+  let arr = Array.make (stream_size s) (Byte.of_int 0) in
+    for x = 0 to Array.length arr - 1 do
+      arr.(x) <- Byte.of_int (sget s x)
+    done;
+    let memstream = new System.IO.MemoryStream() in
+      let deflater =
+        new ICSharpCode.SharpZipLib.Zip.Compression.Streams.DeflaterOutputStream(memstream)
+      in
+        deflater.IsStreamOwner <- false;
+        deflater.Write(arr, 0, stream_size s);
+        deflater.Flush();
+        deflater.Finish();
+        ignore (memstream.Seek(0L, System.IO.SeekOrigin.Begin));
+        let bytes = ref []
+        and finished = ref false in
+          while not !finished do
+            let x = memstream.ReadByte() in
+              if x = ~-1 then set finished else bytes =| x
+          done;
+          bytestream_of_list (rev !bytes)
+
+(* Encode a bigarray using the ZSharpLib library, returning another bigarray. *)
+let decode_flate s =
+  let arr = Array.make (stream_size s) (Byte.of_int 0) in
+    for x = 0 to Array.length arr - 1 do
+      arr.(x) <- Byte.of_int (sget s x)
+    done;
+    let memstream = new System.IO.MemoryStream(arr) in
+      let inflater =
+        new ICSharpCode.SharpZipLib.Zip.Compression.Streams.InflaterInputStream(memstream)
+      in
+        let bytes = ref []
+        and finished = ref false in
+          while not !finished do
+            let x = inflater.ReadByte() in
+              if x = ~-1 then set finished else bytes =| x
+          done;
+          bytestream_of_list (rev !bytes)
+
+(* The same, but we don't know where the data ends. Need to deflate until the
+end of the stream, and leave the input pointer pointing to the byte following
+the compressed data. *)
+let decode_flate_input i =
+  let startpos = i.pos_in () in
+    try
+      stream_of_int_array (Io.read_all (Unzip.inflate true (io_of_pdfio i)))
+    with
+      _ ->
+        dpr "2P";
+        i.seek_in startpos;
+        stream_of_int_array (Io.read_all (Unzip.inflate false (io_of_pdfio i)))
+
+F#*)(*i*)
 
 (* \section{LZW} *)
 
@@ -277,9 +362,10 @@ let decode_lzw early i =
     and input_code stream =
       while !bit_count <= 24 do
         let streambyte =
-          try stream.input_byte () with
-            EndOfInput ->
-              if !endflush = 0 then raise EndOfInput else (decr endflush; 0)
+          match stream.input_byte () with
+          | b when b = Pdfio.no_more ->
+              if !endflush = 0 then raise End_of_file else (decr endflush; 0)
+          | b -> b
         in
           bit_buffer := lor32 !bit_buffer (lsl32 (i32ofi streambyte) (24 - !bit_count));
           bit_count += 8
@@ -297,6 +383,7 @@ let decode_lzw early i =
       code_length := 9;
       old_code := 256
     in
+      (* FIXME: How could it ever be 257? It's a byte... *)
       match peek_byte i with 257 -> mkstream 0 | _ ->
         bit_count := 0; bit_buffer := 0l;
         endflush := 4; reset_table ();
@@ -338,7 +425,7 @@ let decode_lzw early i =
                 done;
                 let out = mkstream outstream_data.pos in
                   for x = 0 to stream_size out - 1 do
-                    out.{x} <- outstream_data.data.{x};
+                    sset out x (sget outstream_data.data x);
                   done;
                   out
 
@@ -712,26 +799,46 @@ let decode_CCITTFax k eol eba c r eob bone dra input =
         let output_span l v =
           if l < 0 then raise (Failure "Bad CCITT stream") else
             begin
-              for x = !column to !column + l - 1 do !currline.(x) <- v; done;
+              for x = !column to !column + l - 1 do
+                let r = !currline in r.(x) <- v
+              done;
               column += l
             end
         and find_b1 () =
           let pos = ref !column
           and curr, opp = if !white then whiteval, blackval else blackval, whiteval in
+            (* Altered to get rid of exception - test *)
             let find v =
-              while !refline.(!pos) <> v do incr pos done; !pos
+              while
+                let r = !refline in
+                  if !pos >= 0 && !pos < Array.length r then
+                    r.(!pos) <> v
+                  else
+                    false
+              do
+                incr pos
+              done; !pos
             in
               try
                 (* Careful to skip imaginary black at beginning *)
                 ignore (if !column = 0 && !white then 0 else find curr);
                 find opp
               with
-                _ -> c
+                _ -> dpr "2Q"; c
         and find_b2 () =
           let pos = ref !column
           and curr, opp = if !white then whiteval, blackval else blackval, whiteval in
+            (* Altered to get rid of exception - test *)
             let find v =
-              while !refline.(!pos) <> v do incr pos done; !pos
+              while
+                let r = !refline in
+                  if !pos >=0 && !pos < Array.length r then
+                    r.(!pos) <> v
+                  else
+                    false
+              do
+                incr pos
+              done; !pos
             in
               try
                 (* Careful to skip imaginary black at beginning *)
@@ -739,7 +846,7 @@ let decode_CCITTFax k eol eba c r eob bone dra input =
                 ignore (find opp);
                 find curr
               with
-                _ -> c
+                _ -> dpr "2R"; c
         in
           try
             while true do
@@ -751,7 +858,7 @@ let decode_CCITTFax k eol eba c r eob bone dra input =
                   set white;
                   if eba then align b;
                   incr row;
-                  if !row >= r && r > 0 then raise EndOfInput
+                  if !row >= r && r > 0 then raise End_of_file
                 end
               else
                begin
@@ -774,7 +881,7 @@ let decode_CCITTFax k eol eba c r eob bone dra input =
                     | Vertical n ->
                         output_span (find_b1 () - !column - n) (if !white then whiteval else blackval);
                         flip white
-                    | EOFB -> raise EndOfInput
+                    | EOFB -> raise End_of_file
                     | Uncompressed -> raise DecodeNotSupported
                   else if k = 0 then
                     (* Group 3 *)
@@ -794,7 +901,8 @@ let decode_CCITTFax k eol eba c r eob bone dra input =
             done;
             mkstream 0
           with
-            EndOfInput -> bytestream_of_write_bitstream output
+            | End_of_file -> dpr "K"; bytestream_of_write_bitstream output
+            | _ -> raise (Failure "Bad CCITT Stream") 
 
 (* \section{PNG and TIFF Predictors} *)
 
@@ -811,8 +919,8 @@ let decode_tiff_predictor colors bpc columns stream =
         for line = 0 to stream_size stream / scanline_width - 1 do
           let linestart = line * scanline_width in
             for p = 1 to scanline_width - 1 do
-              stream.{linestart + p} <-
-                (stream.{linestart + p - 1} + stream.{linestart + p}) mod 256
+              sset stream (linestart + p)
+                ((sget stream (linestart + p - 1) + sget stream (linestart + p)) mod 256)
             done
         done;
         stream
@@ -873,17 +981,21 @@ let decode_predictor pred colors bpc columns stream =
         and pred = ref 0
         and got_predictor = ref false in
           while not !finished do
-            begin try
-              clear got_predictor;
-              pred := i.input_byte ();
-              set got_predictor;
-              prev := !curr;
-              for x = 0 to scanline_width - 1 do
-                !curr.(x) <- i.input_byte ()
-              done
-            with
-              EndOfInput -> set finished
+            clear got_predictor;
+            begin match i.input_byte () with
+            | x when x = Pdfio.no_more -> set finished
+            | x -> pred := x
             end;
+            if !finished then () else
+              begin
+                set got_predictor;
+                prev := !curr;
+                for x = 0 to scanline_width - 1 do
+                  match i.input_byte () with
+                  | x when x = Pdfio.no_more -> set finished
+                  | b -> (!curr).(x) <- b
+                done
+              end;
             (* We allow an unfinished final line only if we managed to get a
             predictor byte *)
             if !got_predictor then
@@ -891,19 +1003,25 @@ let decode_predictor pred colors bpc columns stream =
                 prior_decoded :=
                 decode_scanline_pair
                   !prev !prior_decoded !curr !pred bpc ((bpc * colors + 7) / 8);
-                outputlines =:: !prior_decoded
+                outputlines =| !prior_decoded
               end
           done;
           bytestream_of_arraylist (rev !outputlines)
 
 (* \section{Run Length Encoding} *)
 let encode_runlength stream =
-  let i = Io.input_of_bytestream stream in
+  let i = input_of_bytestream stream in
     let data_in = ref [] in
       begin try
-        while true do data_in =:: i.input_byte () done
+        while true do
+          data_in =|
+            begin match i.input_byte () with
+            | x when x = Pdfio.no_more -> raise End_of_file
+            | x -> x
+            end
+        done
       with
-        EndOfInput -> data_in := rev !data_in
+        End_of_file -> dpr "M"; data_in := rev !data_in
       end;
       let rec runs_of_data prev = function
         | [] -> rev prev
@@ -918,8 +1036,8 @@ let encode_runlength stream =
           let writechunk () =
             if !chunksize > 0 then
               begin
-                outbytes =:: !chunksize - 1; 
-                iter (( =:: ) outbytes) (rev !chunkdata);
+                outbytes =| !chunksize - 1; 
+                iter (( =| ) outbytes) (rev !chunkdata);
                 chunkdata := [];
                 chunksize := 0;
               end
@@ -936,44 +1054,57 @@ let encode_runlength stream =
                     writechunk ();
                     let l = ref l in
                       while !l > 0 do
-                        outbytes =:: 257 - min !l 128;
-                        outbytes =:: x;
+                        outbytes =| 257 - min !l 128;
+                        outbytes =| x;
                         l -= 128
                       done
                 end;
                 runs := tl !runs
               done;
               writechunk ();
-              outbytes =:: 128; (*r End-of-data *)
+              outbytes =| 128; (*r End-of-data *)
               bytestream_of_list (rev !outbytes)
 
 let decode_runlength i =
   let s =  {pos = 0; data = mkstream 4096} in
-    let o = Io.output_of_stream s in
+    let o = output_of_stream s in
       let eod = ref false in
         try
           while not !eod do
-            let l = i.input_byte () in
+            let l =
+              match i.input_byte () with
+              | x when x = Pdfio.no_more -> raise End_of_file
+              | x -> x
+            in
               if l < 128 then
                 for x = 1 to l + 1 do
-                  o.output_byte (i.input_byte ())
+                  o.output_byte
+                    begin match i.input_byte () with
+                      | x when x = Pdfio.no_more -> raise End_of_file
+                      | x -> x
+                    end
                 done
               else if l > 128 then
-                let towrite = i.input_byte () in
+                let towrite =
+                  begin match i.input_byte () with
+                  | x when x = Pdfio.no_more -> raise End_of_file
+                  | x -> x
+                  end;
+                in
                   for x = 1 to 257 - l do
                     o.output_byte towrite
                   done
               else
                 set eod
           done;
-          let osize = i64toi (o.out_channel_length ()) in
+          let osize = postoi (o.out_channel_length ()) in
             let output = mkstream osize in
               for x = 0 to osize - 1 do
-                output.{x} <- s.data.{x}
+                sset output x (sget s.data x)
               done;
               output
         with
-          EndOfInput -> raise (Couldn'tDecodeStream "RunLength")
+          End_of_file -> raise (Couldn'tDecodeStream "RunLength")
 
 (* \section{Decoding PDF streams} *)
 type source =
@@ -1148,28 +1279,30 @@ let rec decode_pdfstream pdf = function
             end
       | _ -> raise (PDFError "Pdf.remove_decoder: malformed /Filter")
       end
+  | Indirect i ->
+      decode_pdfstream pdf (Pdf.direct pdf (Indirect i))
   | _ -> raise (PDFError "Pdf.decode_pdfstream: malformed Stream")
 
 (* \intf Decode a stream until a decoding isn't supported. *)
 let decode_pdfstream_until_unknown pdf s =
   try decode_pdfstream pdf s with
-    DecodeNotSupported -> ()
+    DecodeNotSupported -> dpr "2T"; ()
 
 (* Decode the first decoder from an input. Any further ones can be done in the
 usual fashion. Fails if no decoder (you should have dealt with this already). *)
 let decode_from_input i dict =
-  match lookup_direct_orelse empty "/F" "/Filter" dict with
+  match lookup_direct_orelse (Pdf.empty ()) "/F" "/Filter" dict with
   | Some (Name n) ->
-      Some (decode_one Pdf.empty dict (InputSource i))
+      Some (decode_one (Pdf.empty ()) dict (InputSource i))
   | Some (Array (h::t)) ->
-      let stream = decode_one Pdf.empty dict (InputSource i) in
+      let stream = decode_one (Pdf.empty ()) dict (InputSource i) in
         let rec decode_rest stream = function
           | [] -> stream
           | Name n::more ->
               let dict' = remove_dict_entry dict "/Filter" in
                 let dict'' = remove_dict_entry dict' "/F" in
                   let stream' =
-                    decode_one Pdf.empty dict'' (StreamSource stream)
+                    decode_one (Pdf.empty ()) dict'' (StreamSource stream)
                   in
                     decode_rest stream' more
           | _ -> raise (PDFError "Malformed filter array")
