@@ -8,7 +8,6 @@ LZWDecode, RunLengthDecode.
 - Predictors: PNG (all), TIFF (8-bit only).
 
 *)
-
 (** There was bad data. *)
 exception Couldn'tDecodeStream of string
 
@@ -44,5 +43,5 @@ stream dictionary, decode the first decoder and its predictor. Return the data,
 or [None] if this decoder isn't supported but the data pointer has been left in
 the right place. The exceptions above can both be raised, in the case of bad
 data or a completely unknown encoding. *)
-val decode_from_input : Io.input -> Pdf.pdfobject -> Utility.bytestream option
+val decode_from_input : Pdfio.input -> Pdf.pdfobject -> Utility.bytestream option
 
