@@ -17,10 +17,9 @@ let find_hash r obj gen key keylength =
   and extra =
     if r = 4 then [| 0x73; 0x41; 0x6C; 0x54 |] else [| |]
   in
-    let digest_input = [key; from_obj; from_gen; extra] in
-      let digest = Digest.string (string_of_int_arrays digest_input) in
-        int_array_of_string
-          (String.sub digest 0 (min 16 (keylength / 8 + 5)))
+    let digest_input = string_of_int_arrays [key; from_obj; from_gen; extra] in
+      int_array_of_string
+        (String.sub (digest digest_input) 0 (min 16 (keylength / 8 + 5)))
 
 (* Find a key, given a password, O entry, P entry, id entry, and key length in
 bits. *)
@@ -40,7 +39,7 @@ let pad_password password =
         done;
   pw
 
-let find_key ?(no_encrypt_metadata=false) password r o p id keylength =
+let find_key no_encrypt_metadata password r o p id keylength =
   let password = int_array_of_string password
   and o = int_array_of_string o
   and id = int_array_of_string id in
@@ -55,12 +54,12 @@ let find_key ?(no_encrypt_metadata=false) password r o p id keylength =
       in
         let todigest = [pw; o; from_p; id; rev4_no_metadata] in
           let hash_input = string_of_int_arrays todigest in
-            let hashed = Digest.string hash_input in
+            let hashed = digest hash_input in
               let hashed' =
                 if r >= 3 then
                   let h = ref hashed in
                     for x = 1 to 50 do
-                      let hashed = Digest.string !h in
+                      let hashed = digest !h in
                         h :=
                           string_of_int_array
                             (Array.sub (int_array_of_string hashed) 0 (keylength / 8))
@@ -96,7 +95,7 @@ let crypt key data =
   and out = mkstream (stream_size data) in
     ksa s key;
     for x = 0 to stream_size data - 1 do
-      out.{x} <- data.{x} lxor prga s pi pj
+      sset out x (sget data x lxor prga s pi pj)
     done;
     out
 
@@ -104,7 +103,7 @@ let crypt key data =
 
 (* The state, an array of four length 4 arrays. state.(row).(column) *)
 let st = 
-  Array.make_matrix 4 4 0
+  Array.create_matrix 4 4 0
 
 (* Finite field addition *)
 let ( ++ ) = ( lxor )
@@ -371,12 +370,24 @@ let inv_cipher data_in =
   add_round_key 0;
   output_from_state ()
 
+(*i*)(*F#
+let seed = System.Random ()
+F#*)(*i*)
+
 (* Pad the input data (RFC2898, PKCS \#5), then encrypt using a 16 byte AES
 cipher in cipher block chaining mode, with a random initialisation vector, which
 is stored as the first 16 bytes of the result. *)
-let mkiv () =
+let ran255 () =
+  (*IF-OCAML*)
   Random.self_init ();
-  let r () = Random.int 255 in
+  Random.int 255
+  (*ENDIF-OCAML*)
+  (*i*)(*F#
+  seed.Next (0, 255)
+  F#*)(*i*)
+
+let mkiv () =
+  let r = ran255 in
     [| r (); r (); r (); r ();
        r (); r (); r (); r ();
        r (); r (); r (); r ();
@@ -384,7 +395,7 @@ let mkiv () =
 
 (* Debug function to print a block as characters. *)
 let print_block arr =
-  Array.iter (fun i -> Printf.printf "%C" (char_of_int i)) arr;
+  Array.iter (fun i -> Printf.printf "%c" (char_of_int i)) arr;
   flprint "\n\n"
 
 (* Build blocks for encryption, including padding. *)
@@ -394,9 +405,9 @@ let get_blocks data =
       if l < 16 then [] else
         let blocks = ref [] in
           for x = 0 to l / 16 - 1 do
-            blocks =::
+            blocks =|
               Array.of_list
-                (map (fun x -> data.{x}) (ilist (x * 16) (x * 16 + 15)))
+                (map (sget data) (ilist (x * 16) (x * 16 + 15)))
           done;
           rev !blocks
     and lastblock =
@@ -404,7 +415,7 @@ let get_blocks data =
         if n = 0 then [] else
           let bytes = ref [] in
             for x = 0 to n - 1 do
-              bytes =:: data.{l - 1 - x}
+              bytes =| sget data (l - 1 - x)
             done;
             !bytes
       and pad n =
@@ -422,7 +433,7 @@ let stream_of_blocks blocks =
     and p = ref 0 in
       iter
         (fun a ->
-          Array.iter (fun v -> s.{!p} <- v; incr p) a)
+          Array.iter (fun v -> sset s !p v; incr p) a)
         blocks;
       s
 
@@ -430,11 +441,11 @@ let stream_of_blocks blocks =
 let get_padding s =
   let l = stream_size s in
     assert (l >= 16);
-    let potential = s.{l - 1} in
+    let potential = sget s (l - 1) in
       if potential > 0x10 || potential < 0x01 then None else
         let rec elts_equal p f t =
-          if f = t then p = s.{t} else
-            p = s.{f} && elts_equal p (f + 1) t
+          if f = t then p = sget s t else
+            p = sget s f && elts_equal p (f + 1) t
         in
           if elts_equal potential (l - potential - 1) (l - 1)
             then Some potential
@@ -448,7 +459,7 @@ let cutshort s =
       | Some padding ->
           let s' = mkstream (stream_size s - padding) in
             for x = 0 to stream_size s' - 1 do
-              s'.{x} <- s.{x}
+              sset s' x (sget s x)
             done;
             s'
 
@@ -456,7 +467,7 @@ let cutshort s =
 let get_plain_blocks d =
   if stream_size d mod 16 <> 0 then raise (Pdf.PDFError "Bad AES data") else
     map
-      (fun n -> Array.init 16 (fun x -> d.{16 * n + x}))
+      (fun n -> Array.init 16 (fun x -> sget d (16 * n + x)))
       (ilist 0 (stream_size d / 16 - 1))
 
 (* Decrypt data *)
@@ -473,7 +484,7 @@ let aes_decrypt_data key data =
                (array_map2 (lxor)) (inv_cipher block) !prev_ciphertext
              in 
                prev_ciphertext := block;
-               outblocks =:: plaintext)
+               outblocks =| plaintext)
           codeblocks;
           cutshort (stream_of_blocks (rev !outblocks))
 
@@ -489,7 +500,7 @@ let aes_encrypt_data key data =
             cipher ((array_map2 (lxor)) block !prev_ciphertext)
           in
             prev_ciphertext := ciphertext;
-            outblocks =:: ciphertext)
+            outblocks =| ciphertext)
         (get_blocks data);
         stream_of_blocks (firstblock::rev !outblocks)
 
@@ -497,14 +508,14 @@ let aes_encrypt_data key data =
 
 (* Authenticate the user password, given the password string and U, O, P, id
 and key length entry. *)
-let authenticate_user ?(no_encrypt_metadata=false) password r u o p id keylength =
+let authenticate_user no_encrypt_metadata password r u o p id keylength =
   let u = int_array_of_string u in
-    let key = find_key ~no_encrypt_metadata password r o p id keylength in
+    let key = find_key no_encrypt_metadata password r o p id keylength in
       if r >= 3 then
         let id = int_array_of_string id in
           let todigest = [padding; id] in
             let hash_input = string_of_int_arrays todigest in
-              let hashed = Digest.string hash_input in
+              let hashed = digest hash_input in
                 let encrypted_hashed =
                   int_array_of_stream (crypt key (bytestream_of_string hashed))
                 in
@@ -524,9 +535,7 @@ let authenticate_user ?(no_encrypt_metadata=false) password r u o p id keylength
         u = int_array_of_stream (crypt key (stream_of_int_array padding))
 
 (* Decrypt a PDF file, given the user password. *)
-let rec
-  decrypt pdf ?(no_encrypt_metadata = false) ?(encrypt = false) obj gen key keylength r
-= function
+let rec decrypt pdf no_encrypt_metadata encrypt obj gen key keylength r = function
   | Pdf.String s ->
       let f =
         (if r = 4 then
@@ -536,23 +545,21 @@ let rec
       in
         let s_ints = bytestream_of_string s in
           let hash = find_hash r (i32ofi obj) (i32ofi gen) key keylength in
-            Pdf.String (string_of_stream (f hash s_ints))
+            Pdf.String (string_of_bytestream (f hash s_ints))
   | (Pdf.Stream _) as stream ->
-      decrypt_stream pdf ~no_encrypt_metadata ~encrypt obj gen key keylength r stream
+      decrypt_stream pdf no_encrypt_metadata encrypt obj gen key keylength r stream
   | Pdf.Array a ->
-      Pdf.recurse_array (decrypt pdf ~no_encrypt_metadata ~encrypt obj gen key keylength r) a
+      Pdf.recurse_array (decrypt pdf no_encrypt_metadata encrypt obj gen key keylength r) a
   | Pdf.Dictionary d ->
-      Pdf.recurse_dict (decrypt pdf ~no_encrypt_metadata ~encrypt obj gen key keylength r) d
+      Pdf.recurse_dict (decrypt pdf no_encrypt_metadata encrypt obj gen key keylength r) d
   | x -> x
 
-and decrypt_stream
-  pdf ?(no_encrypt_metadata = false) ?(encrypt = false) obj gen key keylength r stream
-=
+and decrypt_stream pdf no_encrypt_metadata encrypt obj gen key keylength r stream =
   Pdf.getstream stream;
   begin match stream with
   | (Pdf.Stream {contents = (Pdf.Dictionary dict as d, Pdf.Got data)}) as stream ->
       if
-        let identity_crypt_filter_present =
+        begin let identity_crypt_filter_present =
           match Pdf.lookup_direct pdf "/Filter" d with
           | Some (Pdf.Name "/Crypt")
           | Some (Pdf.Array (Pdf.Name "/Crypt"::_)) ->
@@ -572,6 +579,7 @@ and decrypt_stream
           (no_encrypt_metadata &&
              Pdf.lookup_direct pdf "/Type" d = Some (Pdf.Name "/Metadata"))
           || identity_crypt_filter_present
+        end
       then
         stream
       else
@@ -586,7 +594,7 @@ and decrypt_stream
               f hash data
         and dict' =
           Pdf.recurse_dict
-            (decrypt pdf ~no_encrypt_metadata ~encrypt obj gen key keylength r) dict
+            (decrypt pdf no_encrypt_metadata encrypt obj gen key keylength r) dict
         in
           let dict'' =
             if stream_size data <> stream_size data' then
@@ -596,30 +604,24 @@ and decrypt_stream
               dict'
           in
             Pdf.Stream {contents = (dict'', Pdf.Got data')}
-  | _ -> assert false 
+  | _ -> raise (Assert_failure ("decrypt_stream", 0, 0))
   end
 
-let process_cryption
-  ?(no_encrypt_metadata = false) ?(encrypt = false) pdf crypt_type user_pw r u o p id keylength
-=
-  if authenticate_user ~no_encrypt_metadata user_pw r u o p id keylength then
+let process_cryption no_encrypt_metadata encrypt pdf crypt_type user_pw r u o p id keylength =
+  if authenticate_user no_encrypt_metadata user_pw r u o p id keylength then
     begin
-      let key = find_key ~no_encrypt_metadata user_pw r o p id keylength
-      and newdoc = ref Pdf.empty in
+      let key = find_key no_encrypt_metadata user_pw r o p id keylength in
         Pdf.objiter_gen
           (fun objnum gennum obj ->
-             newdoc :=
-               Pdf.addobj_given_num
-                 !newdoc
-                 (objnum,
-                    decrypt pdf ~no_encrypt_metadata ~encrypt objnum gennum key keylength r obj)
-          )
+             ignore
+               (Pdf.addobj_given_num
+               pdf
+               (objnum,
+                  decrypt pdf no_encrypt_metadata encrypt objnum gennum key keylength r obj)))
           pdf;
         let trailerdict' = Pdf.remove_dict_entry pdf.Pdf.trailerdict "/Encrypt" in
-          Some
-            {pdf with
-                 Pdf.objects = !newdoc.Pdf.objects;
-                 Pdf.trailerdict = trailerdict'}
+          pdf.Pdf.trailerdict <- trailerdict';
+          Some pdf
     end
   else
     None
@@ -633,7 +635,7 @@ type encryption =
 
 let get_encryption_values pdf =
   match Pdf.lookup_direct pdf "/Encrypt" pdf.Pdf.trailerdict with
-  | None -> assert false (* Never called on an unencrypted PDF *)
+  | None -> raise (Assert_failure ("get_encryption_values", 0, 0)) (* Never called on an unencrypted PDF *)
   | Some encryptdict ->
       let crypt_type =
         match
@@ -723,7 +725,7 @@ let p_of_banlist toban =
     let setbit n b =
       if b then p := Int32.logor !p (Int32.shift_left 1l (n - 1))
     and notin =
-      notpred (member' toban)
+      notpred (mem' toban)
     in
       setbit 3 (notin NoPrint);
       setbit 4 (notin NoEdit);
@@ -743,14 +745,14 @@ let banlist_of_p p =
   and bitset n =
     Int32.logand (Int32.shift_right p (n - 1)) 1l = 0l
   in
-    if bitset 3 then l =:: NoPrint;
-    if bitset 4 then l =:: NoEdit;
-    if bitset 5 then l =:: NoCopy;
-    if bitset 6 then l =:: NoAnnot;
-    if bitset 9 then l =:: NoForms;
-    if bitset 10 then l =:: NoExtract;
-    if bitset 11 then l =:: NoAssemble;
-    if bitset 12 then l =:: NoHqPrint;
+    if bitset 3 then l =| NoPrint;
+    if bitset 4 then l =| NoEdit;
+    if bitset 5 then l =| NoCopy;
+    if bitset 6 then l =| NoAnnot;
+    if bitset 9 then l =| NoForms;
+    if bitset 10 then l =| NoExtract;
+    if bitset 11 then l =| NoAssemble;
+    if bitset 12 then l =| NoHqPrint;
     !l
 
 (* Main function for decryption. *)
@@ -768,7 +770,7 @@ let decrypt_pdf user_pw pdf =
          | Some (Pdf.Boolean false) -> false
          | _ -> true
        in
-         process_cryption ~no_encrypt_metadata:(not encrypt_metadata)
+         process_cryption (not encrypt_metadata) false
          pdf crypt_type user_pw r u o p id keylength,
          banlist_of_p p
 
@@ -790,28 +792,28 @@ let decrypt_single_stream user_pw pdf obj gen stream =
          | _ -> false
        in
          if
-           authenticate_user ~no_encrypt_metadata user_pw r u o p id keylength
+           authenticate_user no_encrypt_metadata user_pw r u o p id keylength
          then
-           let key = find_key ~no_encrypt_metadata user_pw r o p id keylength in
-             decrypt_stream pdf ~no_encrypt_metadata obj gen key keylength r stream
+           let key = find_key no_encrypt_metadata user_pw r o p id keylength in
+             decrypt_stream pdf no_encrypt_metadata false obj gen key keylength r stream
          else
            raise (Pdf.PDFError "Bad password when decrypting stream")
 
 (* Calculate the owner key from the padded owner password (as calculated by
 [pad_password] *)
 let owner_key padded_owner keylength r =
-  let digest = Digest.string (string_of_int_array padded_owner) in
-    let digest' =
+  let digest1 = digest (string_of_int_array padded_owner) in
+    let digest2 =
       if r >= 3 then
-        let d = ref digest in
+        let d = ref digest1 in
           for x = 1 to 50 do
-            d := Digest.string !d
+            d := digest !d
           done;
           !d
         else
-          digest 
+          digest1
     in
-      int_array_of_string (String.sub digest' 0 (keylength / 8))
+      int_array_of_string (String.sub digest2 0 (keylength / 8))
 
 (* Calculate XOR keys *)
 let mkkey key x =
@@ -836,14 +838,14 @@ let decrypt_pdf_owner owner_pw pdf =
           let user_pw =
             let key = owner_key padded_owner keylength r in
               if r = 2 then
-                string_of_stream (crypt key (bytestream_of_string o))
+                string_of_bytestream (crypt key (bytestream_of_string o))
               else (* r >= 3 *)
                 begin
                   let acc = ref (bytestream_of_string o) in
                     for x = 19 downto 0 do
                       acc := crypt (mkkey key x) !acc
                     done;
-                    string_of_stream !acc 
+                    string_of_bytestream !acc 
                 end
           in
             fst (decrypt_pdf user_pw pdf)
@@ -859,27 +861,27 @@ let mk_owner r owner_pw user_pw keylength =
     let key = owner_key padded_owner keylength r in
       let padded_user = pad_password (int_array_of_string user_pw) in
         if r = 2 then
-          string_of_stream (crypt key (stream_of_int_array padded_user))
+          string_of_bytestream (crypt key (stream_of_int_array padded_user))
         else (* r >= 3 *)
           let acc = ref (crypt key (stream_of_int_array padded_user)) in
             for x = 1 to 19 do
               acc := crypt (mkkey key x) !acc
             done;
-            string_of_stream !acc 
+            string_of_bytestream !acc
             
 (* Make a user password *)
-let mk_user ?(no_encrypt_metadata = false) user_pw o p id r keylength =
-  let key = find_key ~no_encrypt_metadata user_pw r o p id keylength in
+let mk_user no_encrypt_metadata user_pw o p id r keylength =
+  let key = find_key no_encrypt_metadata user_pw r o p id keylength in
     if r = 2 then
-      string_of_stream (crypt key (stream_of_int_array padding))
+      string_of_bytestream (crypt key (stream_of_int_array padding))
     else (* r >= 3 *)
       let digest_input = [padding; int_array_of_string id] in
-        let digest = Digest.string (string_of_int_arrays digest_input) in
-          let acc = ref (crypt key (bytestream_of_string digest)) in
+        let d = digest (string_of_int_arrays digest_input) in
+          let acc = ref (crypt key (bytestream_of_string d)) in
             for x = 1 to 19 do
               acc := crypt (mkkey key x) !acc
             done;
-            string_of_stream !acc ^ (implode (many '\000' 16))
+            string_of_bytestream !acc ^ (implode (many '\000' 16))
 
 (* Get the ID, or add one if there's not one there. Return the updated pdf and
 the ID *)
@@ -896,14 +898,14 @@ let get_or_add_id pdf =
         in
           match idobj with
           | Pdf.Array [Pdf.String s; _] -> s, pdf'
-          | _ -> assert false
+          | _ -> raise (Assert_failure ("get_or_add_id", 0, 0))
 
 (* 40bit encryption *)
 let encrypt_pdf_40bit user_pw owner_pw banlist pdf =
   let p = p_of_banlist banlist
   and owner = mk_owner 2 owner_pw user_pw 40
   and id, pdf = get_or_add_id pdf in
-    let user = mk_user user_pw owner p id 2 40 in
+    let user = mk_user false user_pw owner p id 2 40 in
       let crypt_dict =
         Pdf.Dictionary
           ["/Filter", Pdf.Name "/Standard";
@@ -913,11 +915,12 @@ let encrypt_pdf_40bit user_pw owner_pw banlist pdf =
            "/U", Pdf.String user;
            "/P", Pdf.Integer (i32toi p)]
       in
-        match process_cryption pdf (ARC4 (40, 2)) user_pw 2 user owner p id 40 with
+        match process_cryption false false pdf (ARC4 (40, 2)) user_pw 2 user owner p id 40 with
         | Some pdf ->
             {pdf with
               Pdf.trailerdict =
-                Pdf.add_dict_entry pdf.Pdf.trailerdict "/Encrypt" crypt_dict}
+                Pdf.add_dict_entry
+                  pdf.Pdf.trailerdict "/Encrypt" (Pdf.Indirect (Pdf.addobj pdf crypt_dict))}
         | None -> raise (Pdf.PDFError "Encryption failed")
 
 (* 128bit encryption *)
@@ -925,7 +928,7 @@ let encrypt_pdf_128bit user_pw owner_pw banlist pdf =
   let p = p_of_banlist banlist
   and owner = mk_owner 3 owner_pw user_pw 128
   and id, pdf = get_or_add_id pdf in
-    let user = mk_user user_pw owner p id 3 128 in
+    let user = mk_user false user_pw owner p id 3 128 in
       let crypt_dict =
         Pdf.Dictionary
           ["/Filter", Pdf.Name "/Standard";
@@ -936,7 +939,7 @@ let encrypt_pdf_128bit user_pw owner_pw banlist pdf =
            "/Length", Pdf.Integer 128;
            "/P", Pdf.Integer (i32toi p)]
       in
-        match process_cryption pdf (ARC4 (128, 3)) user_pw 3 user owner p id 128 with
+        match process_cryption false false pdf (ARC4 (128, 3)) user_pw 3 user owner p id 128 with
         | Some pdf ->
             {pdf with
               Pdf.trailerdict =
@@ -948,9 +951,7 @@ let encrypt_pdf_AES encrypt_metadata user_pw owner_pw banlist pdf =
   let p = p_of_banlist banlist
   and owner = mk_owner 4 owner_pw user_pw 128
   and id, pdf = get_or_add_id pdf in
-    let user =
-      mk_user ~no_encrypt_metadata:(not encrypt_metadata) user_pw owner p id 4 128
-    in
+    let user = mk_user (not encrypt_metadata) user_pw owner p id 4 128 in
       let crypt_dict =
         Pdf.Dictionary
           ["/Filter", Pdf.Name "/Standard";
@@ -972,8 +973,8 @@ let encrypt_pdf_AES encrypt_metadata user_pw owner_pw banlist pdf =
            "/StmF", Pdf.Name "/StdCF"]
       in
         match 
-          process_cryption ~no_encrypt_metadata:(not encrypt_metadata) ~encrypt:true
-          pdf AESV2 user_pw 4 user owner p id 128
+          process_cryption
+            (not encrypt_metadata) true pdf AESV2 user_pw 4 user owner p id 128
         with
         | Some pdf ->
             {pdf with
@@ -989,3 +990,4 @@ let is_encrypted pdf =
   | Some _ -> true
   | None -> false
 
+
