@@ -44,3 +44,11 @@ val is_encrypted : Pdf.pdfdoc -> bool
 val decrypt_single_stream :
   string -> Pdf.pdfdoc -> int -> int -> Pdf.pdfobject -> Pdf.pdfobject
 
+type encryption =
+  | ARC4 of int * int
+  | AESV2
+
+(* Don't call on an unencrypted PDF *)
+val get_encryption_values : Pdf.pdfdoc -> encryption * string * string * int32 * string
+
+val banlist_of_p : int32 -> permission list
