@@ -21,7 +21,11 @@ type page =
    rest : pdfobject} (*r A dictionary of the other records in the page. *)
 
 (* Make a PDF rectangle from a [Paper.papersize]. *)
-let rectangle_of_paper (u, w, h) =
+let rectangle_of_paper paper =
+  let u = Paper.unit paper
+  and w = Paper.width paper
+  and h = Paper.height paper
+  in
   let w', h' =
     let f = Units.convert 100. u Units.PdfPoint in
       f w, f h
@@ -30,13 +34,17 @@ let rectangle_of_paper (u, w, h) =
 
 (* \intf Create a page with empty content, media box from the given paper size,
 empty resources, zero rotation and no extra dictionary entries. *)
-let blankpage papersize =
+
+let custompage rectangle =
   {content = [];
-   mediabox = rectangle_of_paper papersize;
+   mediabox = rectangle;
    resources = Dictionary [];
    rotate = Rotate0;
    rest = Dictionary []}
 
+let blankpage papersize =
+  custompage (rectangle_of_paper papersize)
+
 (* \section {Utilities} *)
   
 (* \intf Utility function to convert from rotation to integers. *)
@@ -170,7 +178,7 @@ let make_changes pdf pages =
         let entries name =
           map2 (entries_of_page name) pagenums pages
         in
-          let entries = flatten & flatten (map entries resource_keys) in
+          let entries = flatten <| flatten (map entries resource_keys) in
             let table = Hashtbl.create 10000 in
               iter
                 (fun (entry, pageseq, k, name) ->
@@ -178,17 +186,25 @@ let make_changes pdf pages =
                 entries;
               table
 
-let change_operator lookup seqnum = function
+let change_operator lookup lookup_option seqnum = function
   | Pdfpages.Op_Tf (f, s) ->
       Pdfpages.Op_Tf (lookup "/Font" seqnum f, s)
   | Pdfpages.Op_gs n ->
       Pdfpages.Op_gs (lookup "/ExtGState" seqnum n)
   | Pdfpages.Op_CS n ->
-      begin try Pdfpages.Op_CS (lookup "/ColorSpace" seqnum n) with
-        _ -> Pdfpages.Op_CS n end
+      begin match lookup_option "/ColorSpace" seqnum n with
+      | Some x -> Pdfpages.Op_CS x
+      | None -> Pdfpages.Op_CS n
+      end
+      (*i begin try Pdfpages.Op_CS (lookup "/ColorSpace" seqnum n) with
+        _ -> dpr "2U"; Pdfpages.Op_CS n end i*)
   | Pdfpages.Op_cs n ->
-      begin try Pdfpages.Op_cs (lookup "/ColorSpace" seqnum n) with
-        _ -> Pdfpages.Op_cs n end
+      begin match lookup_option "/ColorSpace" seqnum n with
+      | Some x -> Pdfpages.Op_cs x
+      | None -> Pdfpages.Op_cs n
+      end
+      (*i begin try Pdfpages.Op_cs (lookup "/ColorSpace" seqnum n) with
+        _ -> dpr "2V"; Pdfpages.Op_cs n end i*)
   | Pdfpages.Op_SCNName (s, ns) ->
       Pdfpages.Op_SCNName (lookup "/Pattern" seqnum s, ns)
   | Pdfpages.Op_scnName (s, ns) ->
@@ -208,14 +224,18 @@ let renumber_pages pdf pages =
   | [] -> []
   | pages ->
       let changes = make_changes pdf pages in
-        let rec lookup dictname page oldkey =
-          try Hashtbl.find changes (dictname, page, oldkey) with
-            Not_found -> raise (Pdf.PDFError "key failure")
+        let rec lookup_option dictname page oldkey =
+          tryfind changes (dictname, page, oldkey)
+        and lookup dictname page oldkey =
+          try
+            Hashtbl.find changes (dictname, page, oldkey)
+          with
+            Not_found -> raise (Failure "Pdfdoc: Bad key")
         in
         let change_content seqnum resources content =
           let operators = Pdfpages.parse_operators pdf resources content in
             let operators' =
-              map (change_operator lookup seqnum) operators
+              map (change_operator lookup lookup_option seqnum) operators
             in
               [Pdfpages.stream_of_ops operators']
         and change_resources seqnum resources =
@@ -304,8 +324,6 @@ let mkpage getobjnum parent page =
 
 (* Build a list of objnum, pdfobject pairs from the ptree. The pages in the
 ptree are just missing their parent entries, so we add those. *)
-(*i let extras = ref [] i*)
-
 let rec objects_of_ptree getobjnum extras = function
   | Lf (pages, parent, this) ->
       let page_objects =
@@ -322,7 +340,7 @@ let rec objects_of_ptree getobjnum extras = function
                 (["/Type", Name "/Pages";
                   "/Kids",
                      Array (
-                       map (fun x -> Pdf.Indirect x) (fst & split page_objects));
+                       map (fun x -> Pdf.Indirect x) (fst <| split page_objects));
                   "/Count", Integer (length pages)]
                  @ parent_entry)
           in
@@ -335,27 +353,27 @@ let rec objects_of_ptree getobjnum extras = function
         let left_num =
           match objs_left with
           | (n, _)::_ -> n
-          | [] -> assert false
+          | [] -> raise (Assert_failure ("", 0, 0))
         and right_num =
           match objs_right with
           | (n, _)::_ -> n
-          | [] -> assert false
+          | [] -> raise (Assert_failure ("", 0, 0))
         and count_left =
           match objs_left with
           | (_, Dictionary d)::_ ->
               begin match lookup "/Count" d with
               | Some (Integer i) -> i 
-              | _ -> assert false
+              | _ -> raise (Assert_failure ("", 0, 0))
               end
-          | _ -> assert false
+          | _ -> raise (Assert_failure ("", 0, 0))
         and count_right =
           match objs_right with
           | (_, Dictionary d)::_ ->
               begin match lookup "/Count" d with
               | Some (Integer i) -> i 
-              | _ -> assert false
+              | _ -> raise (Assert_failure ("", 0, 0))
               end
-          | _ -> assert false
+          | _ -> raise (Assert_failure ("", 0, 0))
         in
           let this_objects =
             let page_objects =
@@ -368,7 +386,7 @@ let rec objects_of_ptree getobjnum extras = function
                   let parent_entry =
                     if parent = 0 then [] else ["/Parent", Indirect parent]
                   in
-                    let kids = fst & split page_objects in
+                    let kids = fst <| split page_objects in
                       Dictionary
                         (["/Type", Name "/Pages";
                           "/Kids",
@@ -394,9 +412,8 @@ let add_pagetree pages pdf =
       let ptree = pagetree getobjnum pages 0 in
         let objects = objects_of_ptree getobjnum extras ptree in
           let topnode = match hd objects with (n, _) -> n in
-            let pdf = fold_left addobj_given_num pdf objects in
-              let pdf = fold_left addobj_given_num pdf !extras in
-                pdf, topnode
+            iter (fun x -> ignore (addobj_given_num pdf x)) (objects @ !extras);
+            pdf, topnode
 
 (* \intf Add a root entry, replacing the Type and Pages entry, and any entries
 in [extras]. Preserves any entries in any existing root (e.g Metadata pointer). 
@@ -408,7 +425,7 @@ let add_root pageroot extras pdf =
       | Dictionary d -> d
       | _ -> []
     with
-    _ -> []
+    _ -> dpr "2V"; []
   in
     let root =
       Pdf.Dictionary
@@ -417,7 +434,7 @@ let add_root pageroot extras pdf =
               ([("/Type", Pdf.Name "/Catalog"); ("/Pages", Pdf.Indirect pageroot)] @ existing_entries)
               extras)
     in
-      let pdf, rootnum = Pdf.addobj pdf root in
+      let rootnum = Pdf.addobj pdf root in
         let trailerdict' =
           match pdf.Pdf.trailerdict with
           | Dictionary d -> Dictionary (add "/Root" (Pdf.Indirect rootnum) d)
@@ -432,11 +449,11 @@ let add_root pageroot extras pdf =
 Other objects (e.g destinations in the document outline) may point to the
 individual page objects, so we must renumber these. We can only do this if the
 number of pages are the same. We do this [if replace_numbers is true]. *)
-let change_pages ?(change_references = true) basepdf pages' =
-  let pdf = ref Pdf.empty in
-    Pdf.objiter (fun k v -> pdf := Pdf.addobj_given_num !pdf (k, v)) basepdf;
+let change_pages change_references basepdf pages' =
+  let pdf = Pdf.empty () in
+    Pdf.objiter (fun k v -> ignore (Pdf.addobj_given_num pdf (k, v))) basepdf;
     let old_page_numbers = Pdf.page_reference_numbers basepdf in
-    let pdf, pagetree_num = add_pagetree pages' !pdf in
+    let pdf, pagetree_num = add_pagetree pages' pdf in
       let pdf =
         {pdf with
            Pdf.major = basepdf.Pdf.major;
@@ -449,7 +466,7 @@ let change_pages ?(change_references = true) basepdf pages' =
             | Dictionary d -> d
             | _ -> []
           with
-          _ -> []
+          _ -> dpr "2W"; []
         in
           let pdf = add_root pagetree_num existing_root_entries pdf in
             let new_page_numbers = Pdf.page_reference_numbers pdf in
@@ -467,7 +484,7 @@ words they are all explicit. This is required before writing a file with
 linearization *)
 let pagetree_make_explicit pdf =
   let pages = pages_of_pagetree pdf in
-    change_pages pdf pages
+    change_pages true pdf pages
 
 let _ =
   Pdfwrite.pagetree_make_explicit := pagetree_make_explicit
