@@ -81,19 +81,6 @@ and string_of_calculator cs =
   in
     "{" ^ ops ^ "}"
 
-(* Print floats, integers or int32 values with spaces between them. *)
-let print_floats fs =
-  iter (fun x -> print_float x; print_string " ") fs;
-  print_newline ()
-
-let print_ints is =
-  iter (fun x -> print_int x; print_string " ") is;
-  print_newline ()
-
-let print_int32s is =
-  iter (fun x -> Printf.printf "%li " x) is;
-  print_newline ()
-
 (* Print a function out for debug. *)
 let rec print_function f =
   print_string "Domain...\n";
@@ -148,61 +135,40 @@ let keyword_of_string = function
   | "copy" -> Copy | "exch" -> Exch | "pop" -> Pop
   | "dup" -> Dup | "index" -> Index | "roll" -> Roll
   | s ->
-      flprint ("Bad keyword " ^ s); assert false
-
-let calculator_lexer =
-  Genlex.make_lexer
-    ["{"; "}"; "if"; "ifelse"; "true"; "false"; "abs"; "add"; "atan";
-     "ceiling"; "cos"; "cvi"; "cvr"; "dvi"; "exp"; "floor"; "idiv";
-     "ln"; "log"; "mod"; "mul"; "neg"; "round"; "sin"; "sqrt"; "sub";
-     "truncate"; "and"; "bitshift"; "eq"; "ge"; "gt"; "le"; "lt"; "ne";
-     "not"; "or"; "xor"; "copy"; "exch"; "pop"; "dup"; "index"; "roll"]
-
-let string_of_lexeme = function
-  | Genlex.Kwd s -> "Kwd: " ^ s
-  | Genlex.Ident s -> "Ident: " ^ s
-  | Genlex.Int i -> "Int: " ^ string_of_int i
-  | Genlex.Float f -> "Float: " ^ string_of_float f
-  | Genlex.String s -> "String: " ^ s
-  | Genlex.Char c -> "Char: " ^ string_of_char c
-
-let print_lexemes =
-  iter (fun x -> print_string (string_of_lexeme x); print_newline ())
+      flprint ("Bad keyword " ^ s); raise (Assert_failure ("", 0, 0))
 
 let parse_calculator s =
   let lexemes =
-    try list_of_stream & calculator_lexer & Stream.of_string s with
-      Stream.Error _ ->
-        raise (PDFError "Syntax error in calculator function.")
+    Cgenlex.lex (Pdfio.input_of_bytestream (bytestream_of_string s))
   in
     let rec strip_outer_braces = function
-      | Genlex.Kwd ("{" | "}")::t ->
+      | Cgenlex.Ident ("{" | "}")::t ->
           rev (strip_outer_braces (rev t))
       | x -> x
     and group_operators = function
       | [] -> []
-      | Genlex.Kwd "{"::t ->
-          let ops, rest = cleavewhile (neq (Genlex.Kwd "}")) t in
+      | Cgenlex.Ident "{"::t ->
+          let ops, rest = cleavewhile (neq (Cgenlex.Ident "}")) t in
             ops::group_operators (tl rest)
       | h::t -> [h]::group_operators t
     and parse = function
       | [] -> []
-      | l::l'::[Genlex.Kwd "ifelse"]::t ->
-          IfElse (process l, process l')::parse t
-      | l::[Genlex.Kwd "if"]::t -> If (process l)::parse t
-      | [Genlex.Kwd "true"]::t -> Bool true::parse t
-      | [Genlex.Kwd "false"]::t -> Bool false::parse t
-      | [Genlex.Float f]::t -> Float f::parse t
-      | [Genlex.Int i]::t-> Int (i32ofi i)::parse t (* FIXME: range *)
-      | [Genlex.Kwd x]::t -> keyword_of_string x::parse t
-      | h::_ -> print_lexemes h; raise (Failure "Bad lexeme")
-    and process lexemes =
+      | l::l'::[Cgenlex.Ident "ifelse"]::t ->
+          IfElse (procss l, procss l')::parse t
+      | l::[Cgenlex.Ident "if"]::t -> If (procss l)::parse t
+      | [Cgenlex.Ident "true"]::t -> Bool true::parse t
+      | [Cgenlex.Ident "false"]::t -> Bool false::parse t
+      | [Cgenlex.Float f]::t -> Float f::parse t
+      | [Cgenlex.Int i]::t-> Int (i32ofi i)::parse t (* FIXME: range *)
+      | [Cgenlex.Ident x]::t -> keyword_of_string x::parse t
+      | h::_ -> (*i Printf.eprintf (Cgenlex.string_of_tokens h); i*) raise (Failure "Bad lexeme")
+    and procss lexemes =
       try
-        parse & group_operators & strip_outer_braces lexemes
+        parse (group_operators (strip_outer_braces lexemes))
       with
-        _ -> raise (Pdf.PDFError "Cannot parse Type 4 function")
+      _ -> raise (Pdf.PDFError "Cannot parse Type 4 function")
     in
-      process lexemes
+      procss lexemes
 
 (* \section{Parsing functions} *)
 let rec parse_function pdf f =
@@ -264,13 +230,13 @@ let rec parse_function pdf f =
                     and bitstream =
                       match f with
                       | Stream {contents = _, Got data} ->
-                          Io.bitstream_of_input (Io.input_of_bytestream data)
-                      | _ -> assert false
+                          Pdfio.bitstream_of_input (Pdfio.input_of_bytestream data)
+                      | _ -> raise (Assert_failure ("", 0, 0))
 
                     in
                       let data = Array.make samples 1l in
                         for i = 0 to Array.length data - 1 do
-                          data.(i) <- Io.getval_32 bitstream bitspersample
+                          data.(i) <- Pdfio.getval_32 bitstream bitspersample
                         done;
                         data
                   in
@@ -374,16 +340,16 @@ let eval_function_sampled f s clamped_inputs =
         in
           (* 3. Read values from table. For now, just linear iterpolation. *)
           let ceilings =
-            map (fun x -> int (ceil x)) clamped_encoded_inputs
+            map (fun x -> toint (ceil x)) clamped_encoded_inputs
           and floors =
-            map (fun x -> int (floor x)) clamped_encoded_inputs
+            map (fun x -> toint (floor x)) clamped_encoded_inputs
           in
             let outputs = 
               let ceiling_results = read_table ceilings
               and floor_results = read_table floors in
                 map2
                   (fun x y ->
-                     Int32.to_float x /. 2. +. Int32.to_float y /. 2._)
+                     Int32.to_float x /. 2. +. Int32.to_float y /. 2.)
                   ceiling_results
                   floor_results
             in
@@ -420,6 +386,8 @@ let eval_function_calculator clamped_inputs ops =
       let x = getint () in x, getint ()
     in
       let rec eval k =
+      (*i flprint (string_of_calculator !s);
+      Printf.printf "Now op %s\n\n" (string_of_calculator [k]); i*)
       match k with
       | If l ->
           begin match !s with
@@ -432,7 +400,7 @@ let eval_function_calculator clamped_inputs ops =
           | _ -> typecheck ()
           end
       | (Bool _ | Float _ | Int _) as immediate ->
-          s =:: immediate
+          s =| immediate
       | Abs ->
           begin match !s with
           | Float f::r -> s := Float (fabs f)::r
@@ -485,8 +453,8 @@ let eval_function_calculator clamped_inputs ops =
           let n, n' = getfloats () in
             s := Float (n /. n')::!s
       | Exp ->
-          let base, exponent = getfloats () in
-            s := Float (base ** exponent)::!s
+          let bse, exponent = getfloats () in
+            s := Float (bse ** exponent)::!s
       | Floor ->
           begin match !s with
           | Int i::r -> ()
@@ -555,7 +523,7 @@ let eval_function_calculator clamped_inputs ops =
       | Truncate ->
           begin match !s with
           | Int _::_ -> ()
-          | Float f::r -> s := Int (i32ofi (int f))::r
+          | Float f::r -> s := Int (i32ofi (toint f))::r
           | _ -> typecheck ()
           end
       | And ->
@@ -754,3 +722,56 @@ let rec eval_function f inputs =
         | None -> outputs
         | Some range -> clampvals outputs range
 
+(*i let eval_function f vals =
+  Printf.printf "eval_function\n";
+  iter (Printf.printf "%f ") vals;
+  flprint "\n";
+  print_function f;
+  flprint "\n";
+  let vals = eval_function f vals in
+    iter (Printf.printf "%f ") vals;
+    flprint "\n";
+    vals i*)
+
+let funtype_of_function f =
+  match f.func with
+  | Interpolation _ -> 2
+  | Stitching _ -> 3
+  | Sampled _ -> 0
+  | Calculator _ -> 4
+
+let mkreal x = Pdf.Real x
+
+let entries_of_interpolation i =
+  ["/C0", Pdf.Array (map mkreal i.c0);
+   "/C1", Pdf.Array (map mkreal i.c1);
+   "/N", Pdf.Real i.n]
+
+let rec entries_of_stitching pdf i =
+  (* Add the functions as objects. *)
+  let numbers =
+    map (fun f -> Pdf.Indirect (Pdf.addobj pdf (pdfobject_of_function pdf f))) i.functions
+  in
+    ["/Functions", Pdf.Array numbers;
+     "/Bounds", Pdf.Array (map mkreal i.bounds);
+     "/Encode", Pdf.Array (map mkreal i.stitch_encode)]
+
+and extra_entries_of_function pdf f =
+  match f.func with
+  | Interpolation i -> entries_of_interpolation i
+  | Stitching s -> entries_of_stitching pdf s
+  | Sampled s -> []
+  | Calculator c -> []
+
+and pdfobject_of_function pdf f =
+  let domain =
+    Pdf.Array (map (function x -> Pdf.Real x) f.domain)
+  and range =
+    match f.range with
+    | None -> []
+    | Some fs -> ["/Range", Pdf.Array (map (function x -> Pdf.Real x) fs)]
+  in
+    Pdf.Dictionary
+      (["/FunctionType", Pdf.Integer (funtype_of_function f); "/Domain", domain]
+      @ range @ extra_entries_of_function pdf f)
+
