@@ -1,6 +1,6 @@
 (* \chaptertitle{PDFPages}{High level PDF operations} *)
 open Utility
-open Io
+open Pdfio
 open Pdf
 open Pdfread
 
@@ -161,7 +161,7 @@ let lexemes_of_op = function
   | Op_TJ pdfobject -> [PdfObj pdfobject; Op "TJ"]
   | Op_' s -> [Obj (LexString s); Op "'"]
   | Op_'' (f, f', s) -> 
-      [Obj (LexReal f); Obj (LexReal f'); Obj (LexString s); Op "''"]
+      [Obj (LexReal f); Obj (LexReal f'); Obj (LexString s); Op "\""]
   | Op_d0 (f, f') -> [Obj (LexReal f); Obj (LexReal f'); Op "d0"]
   | Op_d1 (a, b, c, d, e, f) ->
       [Obj (LexReal a); Obj (LexReal b);
@@ -174,9 +174,9 @@ let lexemes_of_op = function
   | Op_SCN fs -> map (fun f -> Obj (LexReal f)) fs @ [Op "SCN"]
   | Op_scn fs -> map (fun f -> Obj (LexReal f)) fs @ [Op "scn"]
   | Op_SCNName (s, fs) ->
-      [Obj (LexName s)] @ map (fun x -> Obj (LexReal x)) fs @ [Op "SCN"]
+      map (fun x -> Obj (LexReal x)) fs @ [Obj (LexName s); Op "SCN"]
   | Op_scnName (s, fs) ->
-      [Obj (LexName s)] @ map (fun x -> Obj (LexReal x)) fs @ [Op "scn"]
+      map (fun x -> Obj (LexReal x)) fs @ [Obj (LexName s); Op "scn"]
   | Op_G f -> [Obj (LexReal f); Op "G"] 
   | Op_g f -> [Obj (LexReal f); Op "g"] 
   | Op_RG (r, g, b) ->
@@ -210,21 +210,21 @@ let string_of_lexemes lexemes =
         let dict_string = Pdfwrite.string_of_pdf dict in
           let dict_string' =
             (* Remove the dictionary markers. *)
-            implode & rev & (drop' 2) & rev & (drop' 2) & explode dict_string
+            implode (rev (drop' 2 (rev (drop' 2 (explode dict_string)))))
           and data_string =
             string_of_bytestream data
           and space =
             let filters =
-              match lookup_direct_orelse Pdf.empty "/F" "/Filter" dict with
+              match lookup_direct_orelse (Pdf.empty ()) "/F" "/Filter" dict with
               | Some (Array filters) -> filters
               | Some (Name f) -> [Name f]
               | _ -> []
             in
               if
-                (member (Name "/ASCIIHexDecode") filters
-                || member (Name "/ASCII85Decode") filters
-                || member (Name "/AHx") filters
-                || member (Name "/A85") filters)
+                (mem (Name "/ASCIIHexDecode") filters
+                || mem (Name "/ASCII85Decode") filters
+                || mem (Name "/AHx") filters
+                || mem (Name "/A85") filters)
               then ""
               else " "
           in
@@ -289,17 +289,20 @@ let rec components pdf resources t =
   | Array (Name "/DeviceN"::_::alternate::_) ->
       components pdf resources (direct pdf alternate)
   | Array [Name "/Separation"; _; _; _]
-  | Array (Name "/Indexed"::_::_) -> 1
+  | Array (Name ("/Indexed" | "/I")::_::_) -> 1
   | Array [Name "/CalRGB"; _] -> 3
   | Array [Name "/CalCMYK"; _] -> 4
   | Array [Name "/CalGray"; _] -> 1
   | Array [Name "/Pattern"; alternate] ->
       components pdf resources (direct pdf alternate)
-  | _ -> raise (PDFSemanticError "Unknown colourspace")
+  | cs ->
+     Printf.eprintf "%s\n" (Pdfwrite.string_of_pdf cs);
+     raise (PDFSemanticError "Unknown colourspace")
 
 (* Lex an inline image. We read the dictionary, and then the stream. *)
 let lex_inline_image pdf resources i =
   (*i flprint "lex_inline_image\n"; i*)
+  try
   let fail () =
     raise Couldn'tHandleContent
   and dict =
@@ -308,8 +311,8 @@ let lex_inline_image pdf resources i =
   in
     (* Read ID token *)
     dropwhite i;
-    let c = i.input_char () in
-      let c' = i.input_char () in
+    let c = char_of_int (i.input_byte ()) in
+      let c' = char_of_int (i.input_byte ()) in
         match c, c' with
         | 'I', 'D' ->
           (* Skip a byte if not ASCII85 / ASCIIHex as one of the filters. *)
@@ -321,10 +324,10 @@ let lex_inline_image pdf resources i =
               | _ -> []
             in
               not
-                (member (Name "/ASCIIHexDecode") filters
-                || member (Name "/ASCII85Decode") filters
-                || member (Name "/AHx") filters
-                || member (Name "/A85") filters)
+                (mem (Name "/ASCIIHexDecode") filters
+                || mem (Name "/ASCII85Decode") filters
+                || mem (Name "/AHx") filters
+                || mem (Name "/A85") filters)
           in
             if toskip then ignore (i.input_byte ());
             let bytes =
@@ -373,12 +376,12 @@ let lex_inline_image pdf resources i =
                     bytewidth * height
             in
               let data =
-                match lookup_direct_orelse empty "/F" "/Filter" dict with
+                match lookup_direct_orelse (Pdf.empty ()) "/F" "/Filter" dict with
                 | None | Some (Array []) ->
                     begin try let data = mkstream bytes in
                       if bytes > 0 then
                         for x = 0 to stream_size data - 1 do
-                          data.{x} <- i.input_byte ();
+                          sset data x (i.input_byte ());
                         done;
                       data
                     with
@@ -398,8 +401,8 @@ let lex_inline_image pdf resources i =
               in
                 (* Read EI token *)
                 dropwhite i;
-                let c = i.input_char () in
-                  let c' = i.input_char () in
+                let c = char_of_int (i.input_byte ()) in
+                  let c' = char_of_int (i.input_byte ()) in
                     begin match c, c' with
                     | 'E', 'I' ->
                         (* Remove filter, predictor. *)
@@ -415,6 +418,8 @@ let lex_inline_image pdf resources i =
                         fail ()
                     end
         | _ -> fail ()
+  with
+    _ -> raise Couldn'tHandleContent
 
 (* Lex a keyword. *)
 let lex_keyword pdf resources i =
@@ -429,7 +434,7 @@ let lex_keyword pdf resources i =
     | "B*" | "b" | "b*" | "n" | "W" | "W*" | "BT" | "ET"
     | "Tc" | "Tw" | "Tz" | "TL" | "Tf" | "Tr" | "Ts"
     | "Td" | "TD" | "Tm" | "T*" | "Tj" | "TJ" | "\'"
-    | "\'\'" | "d0" | "d1" | "CS" | "cs" | "SC" | "SCN"
+    | "\"" | "d0" | "d1" | "CS" | "cs" | "SC" | "SCN"
     | "sc" | "scn" | "G" | "g" | "RG" | "rg" | "K" | "k"
     | "sh" | "Do" | "MP" | "DP" | "BMC"
     | "BDC" | "EMC" | "BX" | "EX" ) as opstring ->
@@ -452,30 +457,38 @@ let lex_hexstring i =
 let lex_next pdf resources i =
   try
     dropwhite i;
-    match peek_char i with
+    match peek_byte i with
+    | x when x = Pdfio.no_more -> None
+    | chr ->
+    match char_of_int chr with
     | '/' -> lex_name i
     | '0'..'9' | '+' | '-' | '.' -> lex_number i
-    | 'A'..'Z' | 'a'..'z' | '\'' -> lex_keyword pdf resources i
+    | 'A'..'Z' | 'a'..'z' | '\'' | '\"' -> lex_keyword pdf resources i
     | '(' -> lex_string i
     | '[' -> nudge i; Some (Obj (LexLeftSquare))
     | ']' -> nudge i; Some (Obj (LexRightSquare))
     | '<' -> 
-        begin match nudge i; let c = peek_char i in rewind i; c with
+        begin match nudge i; let c = unopt (peek_char i) in rewind i; c with
         | '<' -> get_dictionary i
         | _ -> lex_hexstring i
         end
     | '%' -> ignore (Pdfread.lex_comment i); Some (LexComment)
     | _ ->
-        (*i flprint "\nLexing failure: chars: ";
-        for x = 0 to 60 do
-          print_char (i.input_char ())
-        done; i*)
+        for x = 1 to 10 do rewind i done;
+        Printf.eprintf "\nLexing failure: chars before: ";
+        for x = 1 to 10 do
+          print_char (unopt (i.input_char ()))
+        done;
+        Printf.eprintf "\nChars after\n";
+        for x = 1 to 40 do
+          print_char (unopt (i.input_char ()))
+        done;
         raise (PDFSemanticError "Lexing failure in content stream")
   with
     | PDFReadError r -> 
         raise (PDFReadError ("Pdfpages.lex_next => " ^ r))
-    | EndOfInput -> None 
-    | Couldn'tHandleContent -> None
+    | Failure "unopt" | End_of_file -> dpr "3C"; None 
+    | Couldn'tHandleContent -> dpr "3E"; None
 
 let print_lexeme = function
   | Obj p -> print_lexeme p
@@ -579,7 +592,7 @@ let parse_operator compatibility string = function
   | [Op "EX"] -> decr compatibility; Op_EX
   | [Obj (LexReal m); Op "M"] -> Op_M m
   | [Obj (LexString s); Op "\'"] -> Op_' s
-  | [Obj (LexReal aw); Obj (LexReal ac); Obj (LexString s); Op "\'\'"] ->
+  | [Obj (LexReal aw); Obj (LexReal ac); Obj (LexString s); Op "\""] ->
       Op_'' (aw, ac, s)
   | [Obj (LexReal wx); Obj (LexReal wy); Op "d0"] ->
       Op_d0 (wx, wy)
@@ -591,7 +604,7 @@ let parse_operator compatibility string = function
   | [Obj (LexReal tc); Op "Tc"] -> Op_Tc tc
   | [Obj (LexReal tw); Op "Tw"] -> Op_Tw tw
   | [Obj (LexReal tz); Op "Tz"] -> Op_Tz tz
-  | [Obj (LexReal tr); Op "Tr"] -> Op_Tr (int tr)
+  | [Obj (LexReal tr); Op "Tr"] -> Op_Tr (toint tr)
   | [Obj (LexReal ts); Op "Ts"] -> Op_Ts ts
   | [LexInlineImage d] -> InlineImage d
   | ls ->
@@ -602,7 +615,11 @@ let parse_operator compatibility string = function
           | Obj (LexReal n) -> n
           | _ -> raise (PDFSemanticError errtext)
         in
-          map (real_of_real_lexeme errtext) lexemes
+          (* Adobe Distiller 5.0.5 produces bad [Op_scn] *)
+          try
+            map (real_of_real_lexeme errtext) lexemes
+          with
+            _ -> dpr "3E"; [0.;0.;0.;]
       in
         match rev ls with
         | Op "sc"::nums ->
@@ -646,7 +663,9 @@ let parse_operator compatibility string = function
             | _ -> raise (PDFSemanticError "malformed TJ op")
             end
         | Op _::_ -> Op_Unknown string
-        | _ -> failwith "Empty or malformed graphics operation."
+        | _ ->
+           Printf.eprintf "Empty or malformed graphics operation.";
+           Op_Unknown string
 
 (* Split the lexemes into sections (zero or more operands followed by an
 operator) and parse each. *)
@@ -675,7 +694,6 @@ let rec parse_lexemes compatibility ls ops =
         in
           parse_lexemes compatibility remaining (op::ops)
 
-
 (* \intf Parse, given a list of streams. The contents of a single PDF page can be
 split over several streams, which must be concatenated at the lexeme level.
 *)
@@ -688,10 +706,10 @@ let concat_bytestreams ss =
         iter
           (fun s ->
              for x = 0 to stream_size s - 1 do
-               s'.{!p} <- s.{x};
+               sset s' !p (sget s x);
                incr p
              done;
-             s'.{!p} <- int_of_char ' ';
+             sset s' !p (int_of_char ' ');
              incr p)
           ss;
         s'
@@ -722,7 +740,7 @@ let stream_of_lexemes ls =
     while !ls <> [] do
       let l, l' = split !ls in
         ls := l';
-        oplists =:: l
+        oplists =| l
     done;
     let strings =
       rev_map
@@ -742,7 +760,7 @@ let stream_of_lexemes ls =
               let l = String.length str in
                 if l > 0 then
                   for n = 0 to l - 1 do
-                    s.{!pos} <- int_of_char str.[n];
+                    sset s !pos (int_of_char str.[n]);
                     incr pos
                   done;
                 strings := tl !strings
@@ -752,7 +770,7 @@ let stream_of_lexemes ls =
 let print_stream s =
   if stream_size s > 0 then 
     for x = 0 to stream_size s - 1 do
-      Printf.printf "%c" (char_of_int s.{x})
+      Printf.printf "%c" (char_of_int (sget s x))
     done;
   print_newline ()
     
@@ -762,4 +780,3 @@ let stream_of_ops ops =
     Pdf.Stream
       (ref (Pdf.Dictionary [("/Length", Pdf.Integer (stream_size data))], Pdf.Got data))
 
-
