@@ -8,24 +8,24 @@ two exceptions. *)
 
 (** Read a PDF from an [Io.input], with an optional user password which, if
 absent, is assumed to be the empty string. *)
-val pdf_of_input : ?upw:string -> Io.input -> Pdf.pdfdoc
+val pdf_of_input : string option -> Pdfio.input -> Pdf.pdfdoc
 
 (** Same, but delay loading of streams and parsing of objects. *)
-val pdf_of_input_lazy : ?upw:string -> Io.input -> Pdf.pdfdoc
+val pdf_of_input_lazy : string option-> Pdfio.input -> Pdf.pdfdoc
 
 (** Read a PDF from an OCaml channel. *)
-val pdf_of_channel : ?upw:string -> in_channel -> Pdf.pdfdoc
+val pdf_of_channel : string option -> in_channel -> Pdf.pdfdoc
 
 (** Same, but delay loading of streams and parsing of objects. *)
-val pdf_of_channel_lazy : ?upw:string -> in_channel -> Pdf.pdfdoc
+val pdf_of_channel_lazy : string option -> in_channel -> Pdf.pdfdoc
 
 (** Read a PDF from the given filename. *)
-val pdf_of_file : ?upw:string -> string -> Pdf.pdfdoc
+val pdf_of_file : string option -> string -> Pdf.pdfdoc
 
 (**/**)
 (* For internal use by other parts of the library *)
 
-val read_header : Io.input -> int * int
+val read_header : (Pdfio.input -> int * int)
 
 type lexeme =
   | LexNull
@@ -47,25 +47,33 @@ type lexeme =
   | StopLexing
   | LexNone
 
-val lex_stream_data : Io.input -> int -> bool -> lexeme
+val lex_stream_data : Pdfio.input -> int -> bool -> lexeme
 
-val getuntil_white_or_delimiter : Io.input -> char list
+val getuntil_white_or_delimiter : (Pdfio.input -> char list)
 
-val lex_number : Io.input -> lexeme
+val lex_name : Pdfio.input -> lexeme
 
-val lex_string : Io.input -> lexeme
+val lex_number : Pdfio.input -> lexeme
 
-val lex_hexstring : Io.input -> lexeme
+val lex_string : Pdfio.input -> lexeme
 
-val lex_comment : Io.input -> lexeme
+val lex_hexstring : Pdfio.input -> lexeme
 
-val lex_dictionary : Io.input -> lexeme list
+val lex_comment : Pdfio.input -> lexeme
 
-val parse : ?objnum':int -> lexeme list -> int * Pdf.pdfobject
+val lex_dictionary : Pdfio.input -> lexeme list
 
-val dropwhite : Io.input -> unit
+val parse : lexeme list -> int * Pdf.pdfobject
+
+val dropwhite : Pdfio.input -> unit
 
 val print_lexeme : lexeme -> unit
 
 val string_of_lexeme : lexeme -> string
 
+(** Return encryption method in use *)
+val what_encryption : Pdf.pdfdoc -> Pdfwrite.encryption_method option
+
+(** Return list of permissions *)
+val permissions : Pdf.pdfdoc -> Pdfcrypt.permission list
+
