@@ -12,22 +12,19 @@ let _ =
     | _ -> print_string "Syntax: pdftest <input> <output>\n\n"; exit 1
   in
     try
-      let pdf = Pdfread.pdf_of_file in_name in
+      let pdf = Pdfread.pdf_of_file None in_name in
         let pages = Pdfdoc.pages_of_pagetree pdf in
           let pages' = 
-            map 
+            (*i map 
               (fun page ->
-                {page with Pdfdoc.content =
-                  [Pdfpages.stream_of_ops
-                    (Pdfpages.parse_operators pdf page.Pdfdoc.resources
-                    page.Pdfdoc.content)]})
+                 let graphic = Pdfgraphics.graphic_of_page pdf page in
+                     flprint (Pdfgraphics.string_of_graphic graphic);
+                     Pdfgraphics.page_of_graphic pdf (Pdf.parse_rectangle page.Pdfdoc.mediabox) graphic) i*)
               pages
           in
-            let pdf, pagetree_num = Pdfdoc.add_pagetree pages' pdf in
-              let pdf = Pdfdoc.add_root pagetree_num [] pdf in
-                Pdf.iter_stream (Pdfcodec.decode_pdfstream_until_unknown pdf) pdf;
-                Pdf.iter_stream (Pdfcodec.encode_pdfstream pdf Pdfcodec.Flate) pdf;
-                Pdfwrite.pdf_to_file (Pdf.remove_unreferenced pdf) out_name
+            let pdf = Pdfdoc.change_pages true pdf pages' in
+              Pdf.remove_unreferenced pdf;
+              Pdfwrite.pdf_to_file pdf out_name
     with
       err ->
         Printf.printf "Test failed:\n%s\n\n" (Printexc.to_string err);
