@@ -17,7 +17,12 @@ type simple_fonttype =
   | Type3 of type3_glpyhs
   | Truetype
 
-type fontmetrics = int array (*r widths of glyphs 0..255 *)
+type fontmetrics = float array (*r widths of glyphs 0..255 *)
+
+type fontfile =
+  | FontFile of int
+  | FontFile2 of int
+  | FontFile3 of int
 
 (* The fontfile is an indirect reference into the document, rather than a
 PDFobject itself. This preserves polymorphic equality (a pdfobject can contain
@@ -28,7 +33,7 @@ type fontdescriptor =
    leading : float;
    avgwidth : float;
    maxwidth : float;
-   fontfile : int option}
+   fontfile : fontfile option}
 
 type differences = (string * int) list
 
@@ -64,6 +69,22 @@ type standard_font =
   | Symbol
   | ZapfDingbats
 
+let string_of_standard_font = function
+  | TimesRoman -> "Times-Roman"
+  | TimesBold -> "Times-Bold"
+  | TimesItalic -> "Times-Italic"
+  | TimesBoldItalic -> "Times-BoldItalic"
+  | Helvetica -> "Helvetica"
+  | HelveticaBold -> "Helvetica-Bold"
+  | HelveticaOblique -> "Helvetica-Oblique"
+  | HelveticaBoldOblique -> "Helvetica-BoldOblique"
+  | Courier -> "Courier"
+  | CourierBold -> "Courier-Bold"
+  | CourierOblique -> "Courier-Oblique"
+  | CourierBoldOblique -> "Courier-BoldOblique"
+  | Symbol -> "Symbol"
+  | ZapfDingbats -> "ZapfDingbats"
+  
 type cid_system_info =
   {registry : string;
    ordering : string;
@@ -73,7 +94,7 @@ type composite_CIDfont =
   {cid_system_info : cid_system_info;
    cid_basefont : string;
    cid_fontdescriptor : fontdescriptor;
-   cid_widths : (int * int) list;
+   cid_widths : (int * float) list;
    cid_default_width : int}
 
 type cmap_encoding =
@@ -139,12 +160,14 @@ let read_fontdescriptor pdf font =
         | None -> 0.
       and fontfile =
         match Pdf.find_indirect "/FontFile" fontdescriptor with
-        | Some i -> Some i
+        | Some i -> Some (FontFile i)
         | None ->
             match Pdf.find_indirect "/FontFile2" fontdescriptor with
-            | Some i -> Some i
+            | Some i -> Some (FontFile2 i)
             | None ->
-                Pdf.find_indirect "/FontFile3" fontdescriptor
+                match Pdf.find_indirect "/FontFile3" fontdescriptor with
+                | Some i -> Some (FontFile3 i)
+                | None -> None
       in
         Some
           {ascent = ascent;
@@ -179,8 +202,9 @@ let read_metrics pdf font =
           | _ -> raise (Pdf.PDFError "No /LastChar")
         and missingwidth =
           match Pdf.lookup_direct pdf "/MissingWidth" fontdescriptor with
-          | Some (Pdf.Integer w) -> w
-          | _ -> 0
+          | Some (Pdf.Integer w) -> float w
+          | Some (Pdf.Real w) -> w
+          | _ -> 0.
         in
           let elts =
             match Pdf.lookup_direct pdf "/Widths" font with
@@ -196,7 +220,8 @@ let read_metrics pdf font =
                   map
                     (fun elt ->
                        match Pdf.direct pdf elt with
-                       | Pdf.Integer i -> i
+                       | Pdf.Integer i -> float i
+                       | Pdf.Real f -> f
                        | _ -> raise (Pdf.PDFError "Bad /Width entry"))
                     elts
                 and after =
@@ -210,7 +235,7 @@ let pairs_of_differences pdf differences =
     match elts with
     | [] -> prev
     | Pdf.Integer n::rest ->
-        let stripname = function Pdf.Name n -> n | _ -> assert false in
+        let stripname = function Pdf.Name n -> n | _ -> raise (Assert_failure ("", 0, 0)) in
           let names, more =
             cleavewhile (function Pdf.Name _ -> true | _ -> false) rest
           in
@@ -290,7 +315,7 @@ let is_embedded pdf font =
       with
       | Some _ -> true
       | None ->
-          match Pdf.lookup_direct pdf "/Fontfile3" fontdescriptor with
+          match Pdf.lookup_direct pdf "/FontFile3" fontdescriptor with
           | Some _ -> true
           | None -> false
 
@@ -323,7 +348,7 @@ let read_encoding pdf font =
               | Some (Pdf.Name "/WinAnsiEncoding") -> WinAnsiEncoding
               | None ->
                   if is_embedded pdf font
-                  then (ImplicitInFontFile)
+                  then ImplicitInFontFile
                     else if is_symbolic pdf font
                       then ImplicitInFontFile
                       else StandardEncoding
@@ -406,21 +431,28 @@ let rec read_cid_widths = function
       let nums =
         map
           (function
-           | Pdf.Integer i -> i
+           | Pdf.Integer i -> float i
+           | Pdf.Real r -> r
            | _ -> raise (Pdf.PDFError "Bad /W array"))
         ws
       in
         combine (indxn c nums) nums @ read_cid_widths more
-  | Pdf.Integer c_first::Pdf.Integer c_last::Pdf.Integer w::more ->
-      if c_last <= c_first
-        then raise (Pdf.PDFError "Bad /W array")
-        else
-          let pairs =
-            combine
-              (ilist c_first c_last)
-              (many w (c_last - c_first + 1))
-          in
-            pairs @ read_cid_widths more
+  | Pdf.Integer c_first::Pdf.Integer c_last::w::more ->
+      let w =
+        match w with
+        | Pdf.Integer i -> float i
+        | Pdf.Real r -> r
+        | _ -> raise (Pdf.PDFError "Bad /W array")
+      in
+        if c_last <= c_first
+          then raise (Pdf.PDFError "Bad /W array")
+          else
+            let pairs =
+              combine
+                (ilist c_first c_last)
+                (many w (c_last - c_first + 1))
+            in
+              pairs @ read_cid_widths more
   | [] -> []
   | _ -> raise (Pdf.PDFError "Malformed /W in CIDfont")
 
@@ -479,6 +511,7 @@ let read_cidkeyed_font pdf font =
 
 (* Reads a font *)
 let read_font pdf font =
+  (*i flprint (Pdfwrite.string_of_pdf font); i*)
   if is_standard14font pdf font
     then read_standard14font pdf font
     else if is_simple_font pdf font
@@ -496,7 +529,7 @@ let name_to_standard =
   0o113; "/L", 0o114; "/Lslash", 0o350; "/M", 0o115; "/N", 0o116; "/O", 0o117;
   "/OE", 0o352; "/Oslash", 0o351; "/P", 0o120; "/Q", 0o121; "/R", 0o122; "/S",
   0o123; "/T", 0o124; "/U", 0o125; "/V", 0o126; "/W", 0o127; "/X", 0o130; "/Y",
-  0o131; "/Z", 0o132; "/a", 0o140; "/acute", 0o302; "/ae", 0o361; "/ampersand",
+  0o131; "/Z", 0o132; "/a", 0o141; "/acute", 0o302; "/ae", 0o361; "/ampersand",
   0o046; "/asciicircum", 0o136; "/asciitilde", 0o176; "/asterisk", 0o052; "/at",
   0o100; "/b", 0o142; "/backslash", 0o134; "/bar", 0o174; "/braceleft", 0o173;
   "/braceright", 0o175; "/bracketleft", 0o133; "/bracketright", 0o135; "/breve",
@@ -513,7 +546,7 @@ let name_to_standard =
   "/h", 0o150; "/hungarumlaut", 0o315; "/hyphen", 0o055; "/i", 0o151; "/j",
   0o152; "/k", 0o153; "/l", 0o154; "/less", 0o074; "/lslash", 0o370; "/m",
   0o155; "/macron", 0o305; "/n", 0o156; "/nine", 0o071; "/numbersign", 0o043;
-  "/o", 0o157; "/oe", 0o372; "/ogonek", 0o316; "/one", 0o316; "/ordfeminine",
+  "/o", 0o157; "/oe", 0o372; "/ogonek", 0o316; "/one", 0o061; "/ordfeminine",
   0o343; "/ordmasculine", 0o353; "/oslash", 0o361; "/p", 0o160; "/paragraph",
   0o266; "/parenleft", 0o050; "/parenright", 0o051; "/percent", 0o045;
   "/period", 0o056; "/periodcentered", 0o264; "/perthousand", 0o275; "/plus",
@@ -569,7 +602,7 @@ let name_to_macroman =
   0o045; "/period", 0o056; "/periodcentered", 0o341; "/perthousand", 0o344;
   "/plus", 0o053; "/plusminus", 0o261; "/q", 0o161; "/question", 0o077;
   "/questiondown", 0o300; "/quotedbl", 0o042; "/quotedblbase", 0o343;
-  "/quotedblleft", 0o322; "/quotedblright", 0o325; "/quoteleft", 0o324;
+  "/quotedblleft", 0o322; "/quotedblright", 0o323; "/quoteleft", 0o324;
   "/quoteright", 0o325; "/quotesinglbase", 0o342; "/quotesingle", 0o047; "/r",
   0o162; "/registered", 0o250; "/ring", 0o373; "/s", 0o163; "/section", 0o244;
   "/semicolon", 0o073; "/seven", 0o067; "/six", 0o066; "/slash", 0o057;
@@ -577,7 +610,7 @@ let name_to_macroman =
   0o367; "/trademark", 0o252; "/two", 0o062; "/u", 0o165; "/uacute", 0o234;
   "/ucircumflex", 0o236; "/udieresis", 0o237; "/ugrave", 0o235; "/underscore",
   0o137; "/v", 0o166; "/w", 0o167; "/x", 0o170; "/y", 0o171; "/ydieresis",
-  0o330; "/yen", 0o264; "/z", 0o172; "/zero", 0o060]
+  0o330; "/yen", 0o264; "/z", 0o172; "/zero", 0o060; "/space", 0o312]
 
 (* Win Ansi Encoding *)
 let name_to_win =
@@ -585,7 +618,7 @@ let name_to_win =
   "/Adieresis", 0o304; "/Agrave", 0o300; "/Aring", 0o305; "/Atilde", 0o303;
   "/B", 0o102; "/C", 0o103; "/Ccedilla", 0o307; "/D", 0o104; "/E", 0o105;
   "/Eacute", 0o311; "/Ecircumflex", 0o312; "/Edieresis", 0o313; "/Egrave",
-  0o310; "/Eth", 0o320; "/Euro", 0o240; "/F", 0o106; "/G", 0o107; "/H", 0o110;
+  0o310; "/Eth", 0o320; "/Euro", 0o200; "/F", 0o106; "/G", 0o107; "/H", 0o110;
   "/I", 0o111; "/Iacute", 0o315; "/Icircumflex", 0o316; "/Idieresis", 0o317;
   "/Igrave", 0o314; "/J", 0o112; "/K", 0o113; "/L", 0o114; "/M", 0o115; "/N",
   0o116; "/Ntilde", 0o321; "/O", 0o117; "/OE", 0o214; "/Oacute", 0o323;
@@ -600,12 +633,12 @@ let name_to_win =
   0o176; "/asterisk", 0o052; "/at", 0o100; "/atilde", 0o343; "/b", 0o142;
   "/backslash", 0o134; "/bar", 0o174; "/braceleft", 0o173; "/braceright", 0o175;
   "/bracketleft", 0o133; "/bracketright", 0o135; "/brokenbar", 0o246; "/bullet",
-  0o225; "/c", 0o143; "/ccedilla", 0o347; "/cedilla", 0o270; "/cent", 0o240;
+  0o225; "/c", 0o143; "/ccedilla", 0o347; "/cedilla", 0o270; "/cent", 0o242;
   "/circumflex", 0o210; "/colon", 0o072; "/comma", 0o054; "/copyright", 0o251;
   "/currency", 0o244; "/d", 0o144; "/dagger", 0o206; "/daggerdbl", 0o207;
   "/degree", 0o260; "/dieresis", 0o250; "/divide", 0o367; "/dollar",0o044; "/e",
   0o145; "/eacute", 0o351; "/ecircumflex", 0o352; "/edieresis", 0o353;
-  "/egrave", 0o350; "/eight", 0o070; "/ellipsis", 0o205; "/emdash", 0o204;
+  "/egrave", 0o350; "/eight", 0o070; "/ellipsis", 0o205; "/emdash", 0o227;
   "/endash", 0o226; "/equal", 0o075; "/eth", 0o360; "/exclam", 0o041;
   "/exclamdown", 0o241; "/f", 0o146; "/five", 0o065; "/florin", 0o203; "/four",
   0o064; "/g", 0o147; "/germandbls", 0o337; "/grave", 0o140; "/greater", 0o076;
@@ -632,7 +665,8 @@ let name_to_win =
   "/twosuperior", 0o262; "/u", 0o165; "/uacute", 0o372; "/ucircumflex", 0o373;
   "/udieresis", 0o374; "/ugrave", 0o371; "/underscore", 0o137; "/v", 0o166;
   "/w", 0o167; "/x", 0o170; "/y", 0o171; "/yacute", 0o375; "/ydieresis", 0o377;
-  "/yen", 0o245; "/z", 0o172; "/zcaron", 0o236; "/zero", 0o060]
+  "/yen", 0o245; "/z", 0o172; "/zcaron", 0o236; "/zero", 0o060; "/space", 0o240;
+  "/hyphen", 0o255]
 
 (* Mac Expert Encoding *)
 let name_to_macexpert =
@@ -820,7 +854,7 @@ let rec read_number = function
       int_of_string (implode ['0'; 'x'; a; b; c]), more
   | '<'::a::b::c::d::'>'::more ->
       int_of_string (implode ['0'; 'x'; a; b; c; d]), more
-  | [] -> raise Not_found
+  | [] -> dpr "Z"; raise Not_found
   | _ -> raise (Pdf.PDFError "Unknown number in /ToUnicode")
 
 (* Read the bytes of the UTF-16BE unicode sequence as a string. *)
@@ -842,8 +876,8 @@ let rec read_unicode = function
             let bytes =
               map
                 (function
-                  | [x; y] -> char_of_int & int_of_string & implode ['0'; 'x'; x; y]
-                  | _ -> assert false)
+                  | [x; y] -> char_of_int (int_of_string (implode ['0'; 'x'; x; y]))
+                  | _ -> raise (Assert_failure ("", 0, 0)))
                 (splitinto 2 chars)
             in
               let rest' =
@@ -859,7 +893,7 @@ let rec read_unicode = function
 let print_bytestream s =
   flprint "\n";
   for x = 0 to stream_size s - 1 do
-    print_char (char_of_int s.{x})
+    print_char (char_of_int (sget s x))
   done;
   flprint "\n"
 
@@ -878,11 +912,11 @@ let pairs_of_section = function
             let number, rest = read_number !numbers in
               let str, rest = read_unicode rest in
                 numbers := rest;
-                results =:: (number, str)
+                results =| (number, str)
           done;
           []
         with
-          Not_found -> rev !results
+          Not_found -> dpr "3J"; rev !results
         end
   | BfRange numbers ->
       let results = ref []
@@ -899,10 +933,12 @@ let pairs_of_section = function
                         match code with
                         | "" -> ""
                         | s ->
-                            let s = String.copy s in
+                            let chars = rev (explode s) in
+                              implode ((rev (tl chars)) @ [char_of_int (int_of_char (hd chars) + d)])
+                            (*i let s = String.copy s in
                               s.[String.length s - 1] <-
                                 char_of_int (int_of_char s.[String.length s - 1] + d);
-                              s
+                              s i*)
                       in
                         let code, rest = read_unicode rest in
                           results =@
@@ -929,7 +965,7 @@ let pairs_of_section = function
           done;
           []
         with
-          Not_found -> rev !results
+          Not_found -> dpr "3I";rev !results
         end
 
 let rec parse_tounicode pdf tounicode =
@@ -939,15 +975,14 @@ let rec parse_tounicode pdf tounicode =
       begin match tounicode with
       | Pdf.Stream {contents = (dict, Pdf.Got data)} ->
           begin try
-            flatten &
-            map pairs_of_section &
-            get_sections &
-            lose Pdf.is_whitespace &
-            charlist_of_bytestream data
+            flatten
+              (map pairs_of_section
+                (get_sections
+                   (lose Pdf.is_whitespace (charlist_of_bytestream data))))
           with
             e -> Printf.eprintf "/ToUnicode Parse Error : %s\n" (Printexc.to_string e); []
           end
-      | _ -> assert false
+      | _ -> raise (Assert_failure ("", 0, 0))
       end
   | Pdf.Stream {contents = (_, Pdf.ToGet (_, _, _))} ->
       Pdf.getstream tounicode;
@@ -976,22 +1011,22 @@ let utf16be_of_codepoint u =
           [w1; w2]
 
 let utf16be_of_codepoints l =
-  implode & map char_of_int & flatten & map utf16be_of_codepoint l
+  implode (map char_of_int (flatten (map utf16be_of_codepoint l)))
 
 (* Return a list of codepoints from a UTF-16BE string. See RFC2871 *)
-let fail () =
+let fail2 () =
   raise (Invalid_argument "codepoints_of_utf16be")
 
 let rec codepoints_of_utf16be_inner prev = function
   | [] -> rev prev
-  | [w1] -> fail ()
+  | [w1] -> fail2 ()
   | [w1a; w1b] ->
       let w1 = (w1a lsl 8) lor w1b in
         if w1 < 0xD800 || w1 > 0xDFFF then
           codepoints_of_utf16be_inner (w1::prev) []
         else
-          fail ()
-  | [_; _; _] -> fail ()
+          fail2 ()
+  | [_; _; _] -> fail2 ()
   | w1a::w1b::w2a::w2b::more ->
       let w1 = (w1a lsl 8) lor w1b in
         if w1 < 0xD800 || w1 > 0xDFFF then
@@ -1005,39 +1040,54 @@ let rec codepoints_of_utf16be_inner prev = function
                   codepoints_of_utf16be_inner
                     ((((ho lsl 10) lor lo) + 0x10000)::prev) more
               else
-                fail ()
+                fail2 ()
           else
-            fail ()
+            fail2 ()
 
 let codepoints_of_utf16be str =
   codepoints_of_utf16be_inner [] (map int_of_char (explode str))
 
 let glyph_hashes =
-  hashtable_of_dictionary (Glyphlist.glyphmap @ Glyphlist.dingbatmap)
+  hashtable_of_dictionary
+    (Glyphlist.glyphmap @ Glyphlist.dingbatmap @ Glyphlist.truetypemap)
 
 (* Build a hashtable for lookups based on an encoding *)
-let table_of_encoding encoding =
+let table_of_encoding pdf font encoding =
+  (*i flprint (Pdfwrite.string_of_pdf font); i*)
   let table = Hashtbl.create 203
   and swp = map (fun (k, v) -> (v, k)) in
     let addvals = iter (fun (k, v) -> Hashtbl.add table k v) in
       let rec add_encoding = function
-        | ImplicitInFontFile -> ()
+        | ImplicitInFontFile ->
+            (*i flprint "FONTA"; i*)
+            ()
         | StandardEncoding ->
+            (*i flprint "FONTB"; i*)
             addvals (swp name_to_standard)
         | MacRomanEncoding ->
+            (*i flprint "FONTC"; i*)
             addvals (swp name_to_macroman)
         | WinAnsiEncoding ->
+            (*i flprint "FONTD"; i*)
             addvals (swp name_to_win)
         | MacExpertEncoding ->
+            (*i flprint "FONTE"; i*)
             addvals (swp name_to_macexpert)
         | CustomEncoding (e, ds) ->
+            (*i flprint "FONTF"; i*)
             add_encoding e;
             addvals (swp ds)
         | FillUndefinedWithStandard e ->
+            (*i flprint "FONTG"; i*)
             addvals (swp name_to_standard);
             add_encoding e
       in
         add_encoding encoding;
+        (* Print the table out *)
+        (*i flprint "Here's the table...\n";
+        Hashtbl.iter
+          (fun k v -> Printf.printf "(%i %s) " k v)
+          table; i*)
         table
 
 (* Method:
@@ -1053,7 +1103,7 @@ let text_extractor_of_font pdf font =
       let convert =
         begin try
           let table =
-            hashtable_of_dictionary & parse_tounicode pdf tounicode
+            hashtable_of_dictionary <| parse_tounicode pdf tounicode
           in
             begin function i ->
               try
@@ -1062,7 +1112,7 @@ let text_extractor_of_font pdf font =
               Not_found -> [i]
             end
         with
-          Pdf.PDFError ("Bad /ToUnicode") -> (function i -> [i])
+          Pdf.PDFError ("Bad /ToUnicode") -> dpr "3K"; (function i -> [i])
         end
       in
         {convert = convert; font = read_font pdf font}
@@ -1073,19 +1123,20 @@ let text_extractor_of_font pdf font =
             match read_font pdf font with
             | StandardFont (_, e) -> e
             | SimpleFont {encoding = e} -> e
-            | _ -> assert false
+            | _ -> raise (Assert_failure ("", 0, 0))
           in
-           let table = table_of_encoding encoding in
+           let table = table_of_encoding pdf font encoding in
           begin function i ->
             try
               (*i Printf.printf "Character input: %i\n" i; i*)
               let decoded = Hashtbl.find table i in
                 (*i Printf.printf "Decoded character: %s\n" decoded; i*)
                 let r = Hashtbl.find glyph_hashes decoded in
-                  (*i flprint "Unicode result:"; iter (Printf.printf "%i") r; flprint "\n"; i*)
+                  (*i flprint "Unicode result:"; iter (Printf.printf "%i") r;
+                   * flprint "\n"; i*)
                   r
             with
-              Not_found -> [i]
+              Not_found -> dpr "3L"; [i]
           end
         else
           (function i -> [i])
@@ -1100,7 +1151,7 @@ let is_identity_h = function
 let codepoints_of_text extractor text =
   if text = "" then [] else
     if is_identity_h extractor.font then
-      let chars = map int_of_char & explode text in
+      let chars = map int_of_char <| explode text in
         if odd (length chars)
           then raise (Pdf.PDFError "Bad Text")
           else
@@ -1129,35 +1180,57 @@ let utf16be_of_text extractor text =
 (* Convert UTF16BE to Latin1. Chars > U+0255 are dropped silently. *)
 let latin1_of_utf16be str =
   implode
-  & map char_of_int
-  & option_map
+  (map char_of_int
+  (option_map
       (fun x ->
-         if x < 0 then assert false else
+         if x < 0 then raise (Assert_failure ("", 0, 0)) else
          if x < 256 then Some x else None)
-  & codepoints_of_utf16be str
+  (codepoints_of_utf16be str)))
 
 (* Lossily convert to Latin1 string *)
 let latin1_string_of_text extractor text =
-  latin1_of_utf16be & utf16be_of_text extractor text
+  latin1_of_utf16be <| utf16be_of_text extractor text
 
 (* Decode a character according to an encoding *)
 let decode_char encoding chr =
   try 
-    let table = table_of_encoding encoding in
+    (* FIXME: Addded Pdf.empty and Pdf.Null here - do it properly? *)
+    let table = table_of_encoding (Pdf.empty ()) Pdf.Null encoding in
       let name = Hashtbl.find table (int_of_char chr) in
         let number = Hashtbl.find glyph_hashes name in
           match number with
           | [number] ->
-            if number < 0 then assert false else
+            if number < 0 then raise (Assert_failure ("", 0, 0)) else
             if number > 255 then chr else
             char_of_int number
-          | _ -> raise Not_found
+          | _ -> dpr "a"; raise Not_found
   with
     Not_found -> chr
 
 (* Return the glyph name from a char in a type3 font. Raises [Not_found] if
 not found. *)
 let decode_type3_char encoding chr =
-  let table = table_of_encoding encoding in
-    Hashtbl.find table (int_of_char chr)
+  let table = table_of_encoding (Pdf.empty ()) Pdf.Null encoding in
+   (*i Printf.printf "decode_type3_char: made table, char = %i\n" (int_of_char chr);
+   Hashtbl.iter
+   (fun i i' -> Printf.printf "%i %s\n" i i')
+     table; i*)
+    let r = Hashtbl.find table (int_of_char chr) in
+      r
+
+(* Is a PDF string unicode (does it have a byte order marker at the beginning). *)
+let is_unicode s =
+  (String.length s >= 2) && s.[0] = '\254' && s.[1] = '\255'
+
+(* Convert a codepoint in PDFDocEncoding to a Unicode Codepoint - just drop > 127, < 32. *)
+let codepoint_of_pdfdocencoding_character i =
+  if i > 127 || i < 32 then None else Some i
+
+(* Look at a PDF string, determine if it's PDFDocEncoding or UTF16BE, and return the
+unicode codepoints or the byte values as a list of integers in either case. *)
+let codepoints_of_textstring s =
+  if is_unicode s then
+    codepoints_of_utf16be (String.sub s 2 (String.length s - 2))
+  else
+    option_map codepoint_of_pdfdocencoding_character (map int_of_char (explode s))
 
