@@ -1,6 +1,6 @@
 (* \chaptertitle{PDFWrite}{Flattening PDF} *)
 open Utility
-open Io
+open Pdfio
 
 let print_ints is =
   iter (fun x -> print_int x; print_string " ") is;
@@ -43,7 +43,8 @@ let pad_to_ten ch s =
       (fold_left ( ^ ) "" (many ch (10 - l))) ^ s
 
 let string_of_xref n =
-  pad_to_ten "0" (Int64.to_string n) ^ " 00000 n \n" 
+  pad_to_ten "0" ((*IF-OCAML*)Int64.to_string(*ENDIF-OCAML*)(*i*)(*F#
+  string_of_int F#*)(*i*) n) ^ " 00000 n \n" 
 
 (* Write the cross-reference table to a channel. xrefs is a list of positions,
 ~-1L meaning a free entry. *)
@@ -69,7 +70,7 @@ let make_pdf_string s =
     | '\012'::cs -> '\\'::'f'::escape cs
     | c::cs -> c::escape cs
   and enclose s = "(" ^ s ^ ")" in
-    enclose & implode & escape & explode s
+    enclose (implode (escape (explode s)))
 
 (* \section{Flattening PDF to strings} *)
 
@@ -82,11 +83,11 @@ type writeout =
 
 (* We want real numbers with no exponents (format compliance), and no trailing
 zeroes (compactness). *)
-let format_real r =
-  let chars = rev (explode (Printf.sprintf "%f" r)) in
+let format_real = Printf.sprintf "%f"
+  (*i let chars = rev (explode (Printf.sprintf "%f" r)) in
     let chars' = dropwhile (eq '0') chars in
       let chars'' = dropwhile (eq '.') chars' in
-        implode (rev chars'')
+        implode (rev chars'') i*)
 
 (* Character codes in a name $<$ 33 or $>$ 126 are replaced with hashed combinations
 (e.g \#20 for space). If the name contains the null character, an exception is
@@ -104,10 +105,24 @@ let rec make_pdf_name_inner prev = function
         make_pdf_name_inner (rev chars @ prev) t
   | h::t -> make_pdf_name_inner (h::prev) t
 
+(* See if a name needs altering by [make_pdf_name_inner]. *)
+let needs_processing s =
+  let result = ref false in
+    String.iter
+      (function
+       | '\000' -> raise (Pdf.PDFError "Name cannot contain the null character")
+       | x when x < '\033' || x > '\126' || Pdf.is_delimiter x || x = '#' -> set result
+       | _ -> ())
+      s;
+    !result
+
 let make_pdf_name n =
-  match explode n with
-  | '/'::more -> "/" ^ (implode & make_pdf_name_inner [] more)
-  | _ -> raise (Pdf.PDFError "bad name")
+  if needs_processing n then
+    match explode n with
+    | '/'::more -> "/" ^ (implode <| make_pdf_name_inner [] more)
+    | _ -> raise (Pdf.PDFError "bad name")
+  else
+    n
 
 (* Calculate a strings and streams representing the given pdf datatype instance,
 assuming it has no unresolved indirect references. *)
@@ -170,7 +185,7 @@ let output_stream o s =
   | Pdf.Stream {contents = _, Pdf.Got arr} ->
       if stream_size arr > 0 then
         for i = 0 to stream_size arr - 1 do
-          o.output_byte arr.{i}
+          o.output_byte (sget arr i)
         done
   | _ -> raise (Pdf.PDFError "output_stream")
 
@@ -218,9 +233,7 @@ let part6_parts_of_pdf pdf =
       in
         let referenced_from_page =
           Pdf.objects_referenced
-            ~don't_follow_dict_entries:["/Thumb"]
-            ~don't_follow_if_dict_contains:
-               [("/Type", Pdf.Name "/Page"); ("/Type", Pdf.Name "/Pages")]
+            ["/Thumb"] [("/Type", Pdf.Name "/Page"); ("/Type", Pdf.Name "/Pages")]
             pdf (Pdf.lookup_obj pdf first_page_objnum)
         in
           setify_preserving_order
@@ -238,17 +251,12 @@ let part4_parts_of_pdf pdf =
     | _ -> raise (Pdf.PDFError "Bad catalog")
   in     
     let catalog = Pdf.catalog_of_pdf pdf in
-      let indirects_from_catalog
-        ?(don't_follow_dict_entries = [])
-        ?(don't_follow_if_dict_contains = [])
-        entry
-      =
+      let indirects_from_catalog no_follow_entries no_follow_contains entry =
         match catalog with
         | Pdf.Dictionary d ->
             begin match lookup entry d with
             | Some v ->
-                  Pdf.objects_referenced
-                    ~don't_follow_dict_entries ~don't_follow_if_dict_contains pdf v
+                Pdf.objects_referenced no_follow_entries no_follow_contains pdf v
             | _ -> []
             end
         | _ -> raise (Pdf.PDFError "bad catalog")
@@ -270,9 +278,8 @@ let part4_parts_of_pdf pdf =
               (catalog_num::
                 flatten
                   (map (indirects_from_catalog
-                  ~don't_follow_dict_entries:["/Parent"]
-                  ~don't_follow_if_dict_contains:["/Type", Pdf.Name "/Page";
-                  "/Type", Pdf.Name "/Pages"]) sources_follow) @
+                  ["/Parent"]
+                  ["/Type", Pdf.Name "/Page"; "/Type", Pdf.Name "/Pages"]) sources_follow) @
                   objnum_of_acroform)
 
 (* Part 7: For each page, objects reachable from this page which are reachable
@@ -294,10 +301,8 @@ let get_main_parts p3nums pdf =
       let objects_from_each_page =
         map
           (Pdf.objects_referenced
-             ~don't_follow_dict_entries:
-                ["/Thumb"; "/Parent"]
-             ~don't_follow_if_dict_contains:
-                [("/Type", Pdf.Name "/Page"); ("/Type", Pdf.Name "/Pages")]
+             ["/Thumb"; "/Parent"]
+             [("/Type", Pdf.Name "/Page"); ("/Type", Pdf.Name "/Pages")]
              pdf)
           pages
       in
@@ -306,19 +311,20 @@ let get_main_parts p3nums pdf =
         (fun (n, objs) -> Printf.printf "PAGE %i" n; print_nums objs)
           (combine (indx objects_from_each_page) objects_from_each_page); i*)
         let histogram =
-          collate compare & sort compare & flatten objects_from_each_page
+          collate compare (sort compare (flatten objects_from_each_page))
         in
           (*i Printf.printf "get main parts: histogram:\n";
           iter print_nums histogram; i*)
           let shared_objects =
-            flatten & map (function x -> [hd x]) &
-            keep (function [] | [_] -> false | _ -> true) histogram
+            flatten
+              (map (function x -> [hd x])
+                (keep (function [] | [_] -> false | _ -> true) histogram))
           in
             let shared_objects = setminus shared_objects p3nums in
               (*i Printf.printf "shared objects...\n";
               print_nums shared_objects; i*)
               let unshared_lists =
-                map (lose (member' shared_objects)) objects_from_each_page
+                map (lose (mem' shared_objects)) objects_from_each_page
               in
                 (* Put them in order (page object first) and flatten *)
                 let part7_pages =
@@ -343,7 +349,7 @@ let rest_of_trailerdict_entries pdf =
     string_of_pdf
       (fold_left Pdf.remove_dict_entry pdf.Pdf.trailerdict ["/Prev"; "/Size"; "/Root"])
   in
-    implode & rev & tl & tl & rev & tl & tl & explode str
+    implode (rev (tl (tl (rev (tl (tl (explode str)))))))
 
 let flatten_W o = function
  | WString s -> output_string o s
@@ -371,7 +377,7 @@ let lin_renumber old_nums new_nums pdf =
 (* Rember the items in [l] according to the (parital) changes given. *)
 let list_renumber old_nums new_nums pdf l =
   let changes = lin_changes old_nums new_nums pdf in
-    map (fun x -> try Hashtbl.find changes x with Not_found -> x) l
+    map (fun x -> match tryfind changes x with Some y -> y | None -> x) l
 
 (* List of (object number, final position of object in file) pairs *)
 type xrefblank =
@@ -385,15 +391,6 @@ type xrefblank =
   | MainXRefTableFirstEntry
   | Prev
 
-(* List of (object number, position of first X in file) pairs. *)
-(*i let x_positions = ref []
-
-(* List of (object number, position of object in file) pairs. *)
-let object_positions = ref []
-
-(* List of (special, position in file) pairs. *)
-let specials = ref [] i*)
-
 (* Replace the markers with the (now calculated) contents *)
 let replace_xs o object_positions x_positions specials =
   iter
@@ -402,7 +399,8 @@ let replace_xs o object_positions x_positions specials =
          begin match lookup i !object_positions with
          | Some pos ->
              o.seek_out xpos;
-             output_string o (pad_to_ten "0" (Int64.to_string pos))
+             output_string o (pad_to_ten "0"
+             ((*IF-OCAML*)Int64.to_string(*ENDIF-OCAML*)(*i*)(*F# string_of_int F#*)(*i*) pos))
          | None -> raise (Pdf.PDFError "Linearization inconsistency")
          end
      | other, xpos ->
@@ -415,20 +413,22 @@ let replace_xs o object_positions x_positions specials =
            match lookup other !specials with
            | Some pos ->
                o.seek_out xpos;
-               output_string o (pad_to_ten pad (Int64.to_string pos))
+               output_string o (pad_to_ten pad
+               ((*IF-OCAML*)Int64.to_string(*ENDIF-OCAML*)(*i*)(*F#
+               string_of_int F#*)(*i*) pos))
            | _ -> ())
     !x_positions 
 
 (* Outputting specials markers *)
 let output_special_xref_line o xrefblank x_positions =
-  x_positions =:: (xrefblank, o.pos_out ());
+  x_positions =| (xrefblank, o.pos_out ());
   output_string o "XXXXXXXXXX 00000 n \n"
 
 let output_xref_line o x_positions objnum =
   output_special_xref_line o (PDFObj objnum) x_positions
 
 let output_special o xrefblank x_positions =
-  x_positions =:: (xrefblank, o.pos_out ());
+  x_positions =| (xrefblank, o.pos_out ());
   output_string o "XXXXXXXXXX"
 
 (* The minimum number of bits needed to represent the number given. *)
@@ -455,24 +455,23 @@ let page_offset_hint_table pdf pages first_page_objects shared_objects object_po
   let objects_reachable_from_each_page =
     let referenced page_objnum =
       Pdf.objects_referenced
-        ~don't_follow_if_dict_contains:
-           [("/Type", Pdf.Name "/Page"); ("/Type", Pdf.Name "/Pages")]
-           pdf (Pdf.lookup_obj pdf page_objnum)
+        [] [("/Type", Pdf.Name "/Page"); ("/Type", Pdf.Name "/Pages")]
+        pdf (Pdf.lookup_obj pdf page_objnum)
     in
       map
-        (function p -> keep (member' shared_objects) & setify & referenced & hd p)
+        (function p -> keep (mem' shared_objects) (setify (referenced <| hd p)))
         pages
   in
   let page_lengths = map length pages
   and page_byte_lengths = map (objects_bytes pdf) pages in
-  let least_in_page = hd & sort compare & page_lengths
-  and most_in_page = hd & sort rev_compare & page_lengths
-  and least_bytes_in_page = hd & sort compare & page_byte_lengths
-  and most_bytes_in_page = hd & sort rev_compare & page_byte_lengths in
+  let least_in_page = hd (sort compare page_lengths)
+  and most_in_page = hd (sort rev_compare page_lengths)
+  and least_bytes_in_page = hd (sort compare page_byte_lengths)
+  and most_bytes_in_page = hd (sort rev_compare page_byte_lengths) in
   (* Least number of objects in a page *)
   let item1 = least_in_page
   (* Location of first page's page object *)
-  and item2 = i64toi (lookup_failnull (hd (hd pages)) !object_positions)
+  and item2 = (*IF-OCAML*)i64toi(*ENDIF-OCAML*) (lookup_failnull (hd (hd pages)) !object_positions)
   (* Number of bits needed to represent the difference between the greatest and
   least number of objects in a page *)
   and item3 = bits_needed (most_in_page - least_in_page) 
@@ -484,8 +483,8 @@ let page_offset_hint_table pdf pages first_page_objects shared_objects object_po
   (* Number of bits needed to represent the greatest number of shared object
   references. (in other words, in part 8) *)
   and item10 =
-    bits_needed (hd & sort rev_compare &
-    (length (hd pages)::map length objects_reachable_from_each_page))
+    bits_needed (hd (sort rev_compare
+    (length (hd pages)::map length objects_reachable_from_each_page)))
   (* Number of bits needed to represent the numerically greatest shared object
   identifier used by the pages *)
   and item11 = bits_needed (max 0 (length shared_objects + length first_page_objects - 1))
@@ -495,40 +494,40 @@ let page_offset_hint_table pdf pages first_page_objects shared_objects object_po
   (* The denominator of the fractional position for each shared object
   reference. *)
   and item13 = 1
-  and b = Io.make_write_bitstream () in
+  and b = make_write_bitstream () in
     (*i Printf.printf "%i %i %i %i %i %i %i %i %i %i %i %i %i\n"
     item1 item2 item3 item4 item5 0 0 0 item5 item10 item11 item12 item13; i*)
     (* Write the header *)
-    Io.putval b 32 (i32ofi item1);
-    Io.putval b 32 (i32ofi item2);
-    Io.putval b 16 (i32ofi item3);
-    Io.putval b 32 (i32ofi item4);
-    Io.putval b 16 (i32ofi item5);
-    Io.putval b 32 0l;
-    Io.putval b 16 0l;
-    Io.putval b 32 0l;
-    Io.putval b 16 (i32ofi item5);
-    Io.putval b 16 (i32ofi item10);
-    Io.putval b 16 (i32ofi item11);
-    Io.putval b 16 (i32ofi item12);
-    Io.putval b 16 (i32ofi item13);
+    putval b 32 (i32ofi item1);
+    putval b 32 (i32ofi item2);
+    putval b 16 (i32ofi item3);
+    putval b 32 (i32ofi item4);
+    putval b 16 (i32ofi item5);
+    putval b 32 0l;
+    putval b 16 0l;
+    putval b 32 0l;
+    putval b 16 (i32ofi item5);
+    putval b 16 (i32ofi item10);
+    putval b 16 (i32ofi item11);
+    putval b 16 (i32ofi item12);
+    putval b 16 (i32ofi item13);
     (* Now the per-page entries *)
     (* Items 1 *)
     for x = 1 to length pages do
-      Io.putval b item3 (i32ofi (length (select x pages) - item1))
+      putval b item3 (i32ofi (length (select x pages) - item1))
     done;
     (* Item 2 *)
     for x = 1 to length pages do
-      Io.putval b item5 (i32ofi (select x page_byte_lengths - item4))
+      putval b item5 (i32ofi (select x page_byte_lengths - item4))
     done;
     (* Item 3 *)
     for x = 1 to length pages do
       if x = 1 then
         if length pages > 1
-          then Io.putval b item10 0l
-          else Io.putval b item10 (i32ofi (length (hd pages)))
+          then putval b item10 0l
+          else putval b item10 (i32ofi (length (hd pages)))
       else
-        Io.putval b item10 (i32ofi (length (select x objects_reachable_from_each_page)))
+        putval b item10 (i32ofi (length (select x objects_reachable_from_each_page)))
     done;
     (* Item 4 *)
     for x = 1 to length pages do
@@ -542,7 +541,7 @@ let page_offset_hint_table pdf pages first_page_objects shared_objects object_po
           in
             iter
               (fun s ->
-                Io.putval b item11 (i32ofi (Hashtbl.find table s)))
+                putval b item11 (i32ofi (Hashtbl.find table s)))
               shared_objects_reachable
     done;
     (* Item 5 *)
@@ -552,12 +551,12 @@ let page_offset_hint_table pdf pages first_page_objects shared_objects object_po
           select x objects_reachable_from_each_page
         in
           for y = 1 to length shared_objects_reachable do
-            Io.putval b item12 0l (* Always use 0 / 1 fraction *)
+            putval b item12 0l (* Always use 0 / 1 fraction *)
           done
     done;
     (* Item 7 (No item 6) *)
     for x = 1 to length pages do
-      Io.putval b item5 0l (* Ignored *)
+      putval b item5 0l (* Ignored *)
     done;
     b
 
@@ -581,7 +580,7 @@ let shared_object_hint_table
       | [] -> 0
       | h::_ -> h
     in
-      let b = Io.make_write_bitstream () in
+      let b = make_write_bitstream () in
         (* Object number of first object in shared objects section *)
         let item1 = match shared_objects with [] -> 0 | h::_ -> h
         (* Location of the first object in the shared objects section *)
@@ -597,13 +596,13 @@ let shared_object_hint_table
         smallest length of an shared object group (=object) in bytes *)
         and item7 = bits_needed (greatest - least)
         in
-          Io.putval b 32 (i32ofi item1);
-          Io.putval b 32 (i32ofi item2);
-          Io.putval b 32 (i32ofi item3);
-          Io.putval b 32 (i32ofi item4);
-          Io.putval b 16 0l;
-          Io.putval b 32 (i32ofi item6);
-          Io.putval b 16 (i32ofi item7);
+          putval b 32 (i32ofi item1);
+          putval b 32 (i32ofi item2);
+          putval b 32 (i32ofi item3);
+          putval b 32 (i32ofi item4);
+          putval b 16 0l;
+          putval b 32 (i32ofi item6);
+          putval b 16 (i32ofi item7);
           (*i Printf.printf "shared object table header: %i %i %i %i %i %i %i\n"
           item1 item2 item3 item4 0 item6 item7; i*)
           (* Main Section, Sequence One (First Page Objects) *)
@@ -611,23 +610,23 @@ let shared_object_hint_table
           iter
             (fun x ->
               let len = object_bytes pdf x - item6 in 
-                Io.putval b item7 (i32ofi len))
+                putval b item7 (i32ofi len))
             first_page_objects;
           (* Item 2s *)
-          iter (function _ -> Io.putval b 1 0l) first_page_objects;
+          iter (function _ -> putval b 1 0l) first_page_objects;
           (* Item 4s *)
-          iter (function _ -> Io.putval b 0 0l) first_page_objects;
+          iter (function _ -> putval b 0 0l) first_page_objects;
           (* Main Section, Sequence Two (Shared Objects (Part 8)) *)
           (* Item 1s *)
           iter
             (fun x ->
               let len = object_bytes pdf x - item6 in
-                Io.putval b item7 (i32ofi len))
+                putval b item7 (i32ofi len))
             shared_objects;
           (* Item 2s *)
-          iter (function _ -> Io.putval b 1 0l) shared_objects;
+          iter (function _ -> putval b 1 0l) shared_objects;
           (* Item 4s *)
-          iter (function _ -> Io.putval b 0 0l) shared_objects;
+          iter (function _ -> putval b 0 0l) shared_objects;
           b
           
 (* This is filled in by the Pdfdoc module at code-loading time. It remains
@@ -646,13 +645,13 @@ let pdf_to_output_linearized encrypt pdf o =
   and object_positions = ref []
   and x_positions = ref [] in
   let pdf = !pagetree_make_explicit pdf in
-  let pdf = Pdf.remove_unreferenced pdf in 
+  Pdf.remove_unreferenced pdf;
   let writeobj pdf p =
     let obj = 
       try Pdf.lookup_obj pdf p with
-        | Not_found -> Pdf.Null
+        | Not_found -> dpr "3N"; Pdf.Null
     in
-      object_positions =:: (p, o.pos_out ()); 
+      object_positions =| (p, o.pos_out ()); 
       iter
         (flatten_W o)
         (strings_of_object (p, obj))
@@ -669,7 +668,7 @@ let pdf_to_output_linearized encrypt pdf o =
   output_string o (header pdf);
   (* Part 2: Linearization parameter dictionary *)
   let lin_dict_obj_number = objects_in_rest_of_file + 1 in
-  specials =:: (LinearizationDictionaryPosition, o.pos_out ());
+  specials =| (LinearizationDictionaryPosition, o.pos_out ());
   output_string o
 (string_of_int lin_dict_obj_number ^ " 0 obj\n<< /Linearized 1.0\n/L ");
   output_special o FileLength x_positions;
@@ -682,7 +681,7 @@ let pdf_to_output_linearized encrypt pdf o =
   output_string o "/E ";
   output_special o EndOfFirstPage x_positions;
   output_string o
-("\n/N " ^ (string_of_int & length & Pdf.page_reference_numbers pdf) ^ "\n/T");
+("\n/N " ^ (string_of_int (length (Pdf.page_reference_numbers pdf))) ^ "\n/T");
   output_special o MainXRefTableFirstEntry x_positions;
   output_string o "\n>>\nendobj\n";
   (* Part 3: First page cross-reference table and trailer *)
@@ -724,15 +723,15 @@ let pdf_to_output_linearized encrypt pdf o =
 " 0 R " ^ rest_of_trailerdict_entries pdf ^ ">>\n" ^ "startxref\n0\n%%EOF\n");
   (* Part 4 and Part 6: Document-level and first page  *)
   iter (writeobj pdf) p3nums;
-  specials =:: (EndOfFirstPage, o.pos_out ());
+  specials =| (EndOfFirstPage, o.pos_out ());
   (* Part 5: Primary hint stream *)
   let all_pages = tl p3nums::p7_pages in
-  let p8positions = cumulative_sum (p7length + i64toi (o.pos_out ())) p8lengths in
+  let p8positions = cumulative_sum (p7length + (*IF-OCAML*)i64toi(*ENDIF-OCAML*) (o.pos_out ())) p8lengths in
   let offset_table = page_offset_hint_table pdf all_pages new_p6objs new_p8nums object_positions in
   let shared_table = shared_object_hint_table pdf new_p6objs new_p8nums p8positions in
   let stream_content =
-    Io.bytestream_of_write_bitstream &
-    Io.write_bitstream_append_aligned offset_table shared_table
+    bytestream_of_write_bitstream <|
+    write_bitstream_append_aligned offset_table shared_table
   in
   let hintstream_dict =
     Pdf.Dictionary
@@ -744,39 +743,33 @@ let pdf_to_output_linearized encrypt pdf o =
       (Pdf.Stream (ref (hintstream_dict, Pdf.Got (stream_content))))
   and hint_num = Pdf.objcard pdf + 2 in
   let hs_offset = o.pos_out () in
-  specials =:: (PrimaryHintStreamPosition, hs_offset);
-  specials =:: (HintOffset, hs_offset);
+  specials =| (PrimaryHintStreamPosition, hs_offset);
+  specials =| (HintOffset, hs_offset);
   output_string o ((string_of_int hint_num) ^ " 0 obj\n");
   iter (flatten_W o) stream_wstrings;
   output_string o "\nendobj\n";
-  let hs_length = i64sub (o.pos_out ()) hs_offset in
-  specials =:: (HintLength, hs_length);
+  let hs_length = (*IF-OCAML*)i64sub(*ENDIF-OCAML*)(*i*)(*F# ( - ) F#*)(*i*) (o.pos_out ()) hs_offset in
+  specials =| (HintLength, hs_length);
   (* Parts 7, 8 and 9: Remaining pages and other objects. *)
   iter (writeobj pdf) new_main_nums;
   (* Part 11: Main cross-reference table and trailer *)
-  specials =:: (Prev, o.pos_out ());
+  specials =| (Prev, o.pos_out ());
   let main_size = length p7nums + length p8nums + length p9nums + 1 in
   output_string o ("xref\n0 " ^ string_of_int main_size ^ "\n");
-  specials =:: (MainXRefTableFirstEntry, o.pos_out ());
+  specials =| (MainXRefTableFirstEntry, o.pos_out ());
   output_string o ("0000000000 65536 f \n");
   iter (output_xref_line o x_positions) new_main_nums;
   output_string o ("trailer\n<< /Size " ^ string_of_int main_size ^ " >>\nstartxref\n");
-  output_string o (Int64.to_string position_of_first_page_xref_table);
+  output_string o ((*IF-OCAML*)Int64.to_string(*ENDIF-OCAML*)(*i*)(*F#
+  string_of_int F#*)(*i*) position_of_first_page_xref_table);
   output_string o "\n%%EOF\n";
-  specials =:: (FileLength, o.pos_out ());
+  specials =| (FileLength, o.pos_out ());
   replace_xs o object_positions x_positions specials
 
 (* \section{Main functions} *)
 
-(*i let print_pdf_objs pdf =
-  Pdf.objiter
-    (fun n obj ->
-       Printf.printf "%i 0 obj:\n\n" n;
-       Printf.printf "%s\n" (string_of_pdf obj))
-    pdf i*)
-
 (* \intf Flatten a PDF document to an [Io.output]. *)
-let pdf_to_output ?(linearize = false) ?encrypt pdf o =
+let pdf_to_output linearize encrypt pdf o =
   if linearize then pdf_to_output_linearized encrypt pdf o else
     let pdf = Pdf.renumber (changes pdf) pdf in
       let pdf = crypt_if_necessary pdf encrypt in
@@ -785,7 +778,7 @@ let pdf_to_output ?(linearize = false) ?encrypt pdf o =
           Pdf.objiter
             (fun ob p ->
                let strings = strings_of_object (ob, p) in
-                 xrefs =:: o.pos_out ();
+                 xrefs =| o.pos_out ();
                  iter (flatten_W o) strings)
             pdf;
           let xrefstart = o.pos_out () in
@@ -803,7 +796,9 @@ let pdf_to_output ?(linearize = false) ?encrypt pdf o =
             in
               iter (flatten_W o) (strings_of_pdf trailerdict');
               output_string o
-                ("\nstartxref\n" ^ Int64.to_string xrefstart ^ "\n%%EOF\n")
+                ("\nstartxref\n" ^
+                (*IF-OCAML*)Int64.to_string(*ENDIF-OCAML*)(*i*)(*F#
+                string_of_int F#*)(*i*) xrefstart ^ "\n%%EOF\n")
 
 let change_id pdf f =
   match pdf.Pdf.trailerdict with
@@ -813,20 +808,24 @@ let change_id pdf f =
   | _ -> raise (Pdf.PDFError "Bad trailer dictionary")
 
 (* \intf Write a PDF to a channel. Don't use [mk_id] when the file is encrypted.*)
-let pdf_to_channel ?(linearize = false) ?encrypt ?(mk_id = false) pdf ch =
+let pdf_to_channel linearize encrypt mk_id pdf ch =
   let pdf =
     if mk_id then change_id pdf "" else pdf
   in
-    pdf_to_output ~linearize ?encrypt pdf (output_of_channel ch)
+    pdf_to_output linearize encrypt pdf (output_of_channel ch)
 
 (* \intf Similarly to a named file. If [mk_id] is set, the /ID entry in the
 document's trailer dictionary is updated using the current date and time and the
 filename. Don't use [mk_id] when the file is encrypted. *)
-let pdf_to_file ?(linearize = false) ?encrypt ?(mk_id = true) pdf f =
+let pdf_to_file_options linearize encrypt mk_id pdf f =
   let pdf' =
     if mk_id then change_id pdf f else pdf
   in
     let ch = open_out_bin f in
-      pdf_to_channel ~linearize ?encrypt ~mk_id:false pdf' ch;
+      pdf_to_channel linearize encrypt false pdf' ch;
       close_out ch
 
+let pdf_to_file pdf f =
+  pdf_to_file_options false None true pdf f
+
+
