@@ -7,6 +7,15 @@ top level, so their names are considered reserved words in other modules.
 All functions in this module are tail-recursive. All are exposed in the
 interface. *)
 
+(* Print something and then flush standard output. *)
+let flprint s =
+  print_string s; flush stdout
+
+(* Debug printing *)
+let dp_print = ref false
+
+let dpr s = if !dp_print then flprint s
+
 (* \section{Functions on lists} *)
 
 (* [xxx] is a tail-recursive version of [List.xxx]. See [List] module for
@@ -88,8 +97,8 @@ let combine3 a b c =
       rev !pairs
     with
       Invalid_argument _ -> raise (Invalid_argument "Utility.combine3")
-
-let fold_left = List.fold_left
+ 
+let fold_left f b l = List.fold_left f b l
 
 let fold_right f l e =
   List.fold_left (fun x y -> f y x) e (rev l)
@@ -137,12 +146,23 @@ let map5 f a b c d e =
 
 (* Calculate the cumulative sum of a list given a base e.g [cumulative_sum 5
 [1;2;3] = [6; 8; 11]] *)
-let cumulative_sum = 
-  let rec cumulative_sum prev base = function
+let cumulative_sum b l = 
+  let rec cumulative_sum prev bse = function
     | [] -> rev prev
-    | h::t -> cumulative_sum ((base + h)::prev) (base + h) t
-in
-  cumulative_sum []
+    | h::t -> cumulative_sum ((bse + h)::prev) (bse + h) t
+  in
+    cumulative_sum [] b l
+
+(* Split a list into a list of lists at every point where [p] is true *)
+let rec split_around_inner p prev curr = function
+  | [] -> if curr = [] then (rev prev) else (rev (rev curr::prev))
+  | h::t ->
+      if p h
+        then split_around_inner p (rev curr::prev) [] t
+        else split_around_inner p prev (h::curr) t
+
+let split_around p l =
+  split_around_inner p [] [] l
 
 (* Count the number of elements matching a predicate. *)
 let rec lcount_inner p c = function
@@ -156,30 +176,24 @@ let lcount p l =
   lcount_inner p 0 l
 
 (* Find the position of the first element matching a predicate. The first
-element is number one. Fails with [Not_found]. *)
+element is number one. *)
 let rec index_inner n p = function
-  | [] -> raise Not_found
+  | [] -> dpr "b"; raise Not_found
   | h::t when p h -> n
   | _::t -> index_inner (n + 1) p t
 
 let index n p = index_inner 1 n p
 
 (* \section{Functions on Strings} *)
-let firstchar s =
-  try Some s.[0] with Invalid_argument _ -> None
+let firstchar (s : string) =
+  try Some s.[0] with Invalid_argument _ -> dpr "3R"; None
 
-let lastchar s =
-  try Some s.[String.length s - 1] with Invalid_argument _ -> None
-
-(* Use a char to char function to alter the elements of a string. *)
-let string_selfmap f s =
-  for c = 0 to String.length s - 1 do
-    s.[c] <- f s.[c]
-  done
+let lastchar (s : string) =
+  try Some s.[String.length s - 1] with Invalid_argument _ -> dpr "3S"; None
 
 (* Make a list of characters from a string, preserving order. *)
 let explode s =
-  let l = ref [] in
+  let l : char list ref = ref [] in
     if String.length s = 0 then !l else
       begin
         for p = 1 to String.length s do
@@ -190,17 +204,13 @@ let explode s =
 
 (* Make a string from a list of characters, preserving order. *)
 let implode l =
-  let result = String.create (length l) in
-    let rec imp i = function
-     | [] -> result
-     | c :: l -> result.[i] <- c; imp (i + 1) l
-    in
-      imp 0 l
+  let b = Buffer.create (List.length l) in
+    List.iter (Buffer.add_char b) l;
+    Buffer.contents b
 
 (* String of character. *)
 let string_of_char c =
-  let s = String.create 1 in
-    s.[0] <- c; s
+  implode [c]
 
 (* \section{Long-integer function abbreviations} *)
 let i32ofi = Int32.of_int
@@ -260,18 +270,15 @@ let i64pred = Int64.pred
 let i64max = Pervasives.max
 
 let i64min = Pervasives.min
-
+ 
 (* \section{Byte streams} *)
 
+(*IF-OCAML*)
+
 (* Type abbreviation for byte-addressable arrays. *)
 type bytestream =
   (int, Bigarray.int8_unsigned_elt, Bigarray.c_layout) Bigarray.Array1.t
 
-(* For lexing / parsing byte streams, keep the position. Starts at zero. *)
-type stream =
-  {mutable pos : int;
-   mutable data : bytestream}
-
 (* Make a stream of a given size. *)
 let mkstream =
   Bigarray.Array1.create Bigarray.int8_unsigned Bigarray.c_layout
@@ -279,14 +286,43 @@ let mkstream =
 (* Find the size of a stream. *)
 let stream_size = Bigarray.Array1.dim
 
+let sset s n v =
+  s.{n} <- v
+
+let sget s n =
+  s.{n}
+
+(*ENDIF-OCAML*)
+
+(*i*)
+(*F#
+
+type bytestream = Bytearray.bytearray
+
+let mkstream = Bytearray.create
+
+let stream_size (x : bytestream) = Bytearray.length x
+
+let sset s n v = Bytearray.set s n (Byte.of_int v)
+
+let sget s n = Byte.to_int (Bytearray.get s n)
+
+F#*)
+(*i*)
+
+(* For lexing / parsing byte streams, keep the position. Starts at zero. *)
+type stream =
+  {mutable pos : int;
+   mutable data : bytestream}
+
 (* Fill a stream with a value. *)
 let fillstream v s =
-  for x = 0 to stream_size s - 1 do s.{x} <- v done
+  for x = 0 to stream_size s - 1 do sset s x v done
 
 let print_stream s =
   if stream_size s > 0 then
     for x = 0 to stream_size s - 1 do
-      Printf.printf "%i " s.{x}
+      Printf.printf "%i " (sget s x)
     done
 
 (* Make a bytestream from a string, with no terminator. *)
@@ -295,7 +331,7 @@ let bytestream_of_string s =
     let stream = mkstream l in
       if l > 0 then
         for k = 0 to l - 1 do
-          stream.{k} <- int_of_char s.[k]
+          sset stream k (int_of_char s.[k])
         done;
       stream
 
@@ -305,7 +341,7 @@ let bytestream_of_list l =
     if length = 0 then mkstream 0 else
       let s = mkstream length and l = ref l in
         for pos = 0 to length - 1 do
-          s.{pos} <- hd !l;
+          sset s pos (hd !l);
           l := tl !l
         done;
         s
@@ -316,7 +352,7 @@ let bytestream_of_charlist cs =
     if length = 0 then mkstream 0 else
       let s = mkstream length and cs = ref cs in
         for pos = 0 to length - 1 do
-          s.{pos} <- int_of_char (hd !cs);
+          sset s pos (int_of_char (hd !cs));
           cs := tl !cs
         done;
         s
@@ -328,64 +364,55 @@ let bytestream_of_arraylist l =
       iter
         (fun a ->
            for x = 0 to Array.length a - 1 do
-             output.{!pos} <- a.(x); incr pos
+             sset output !pos a.(x); incr pos
            done)
         l;
       output
 
 let string_of_bytestream s =
-  if stream_size s > Sys.max_string_length then
-    raise (Invalid_argument "string_of_bytestream")
-  else
-    let l = stream_size s in
-      let str = String.create l in
-        for x = 0 to String.length str - 1 do
-          str.[x] <- char_of_int s.{x}
-        done;
-        str
+  let l = stream_size s in
+    let buf = Buffer.create l in
+      for x = 0 to l - 1 do
+        Buffer.add_char buf (char_of_int (sget s x))
+      done;
+      Buffer.contents buf
 
 let stream_of_int_array a =
   let s = mkstream (Array.length a) in
     for i = 0 to stream_size s - 1 do
-      s.{i} <- a.(i)
+      sset s i a.(i)
     done;
     s
 
 let int_array_of_stream s =
   let a = Array.make (stream_size s) 0 in
     for i = 0 to Array.length a - 1 do
-      a.(i) <- s.{i}
+      a.(i) <- sget s i
     done;
     a
 
+(* Copy a stream. *)
+let copystream s =
+  let l = stream_size s in
+    let s' = mkstream l in
+      if l > 0 then
+        for k = 0 to l - 1 do
+          sset s' k (sget s k)
+        done;
+      s'
+
 let int_array_of_string s =
-   Array.init (String.length s) (fun i -> int_of_char s.[i])
+  Array.init (String.length s) (fun i -> int_of_char s.[i])
 
 let string_of_int_arrays arrays =
   let len = fold_left ( + ) 0 (map Array.length arrays) in
-    let out = String.create len
-    and i = ref 0 in
-      iter (Array.iter (fun v -> out.[!i] <- char_of_int v; incr i)) arrays;
-      out
+    let buf = Buffer.create len in
+      iter (Array.iter (fun v -> Buffer.add_char buf (char_of_int v))) arrays;
+      Buffer.contents buf
 
 let string_of_int_array a =
   string_of_int_arrays [a]
 
-let string_of_stream stream =
-  let string = String.make (stream_size stream) '\000' in
-    for i = 0 to String.length string - 1 do
-      string.[i] <- char_of_int stream.{i}
-    done;
-    string
-
-(* Copy a stream. *)
-let copystream s =
-  let l = Bigarray.Array1.dim s in
-    let s' = mkstream l in
-      if l > 0 then
-        for k = 0 to l - 1 do s'.{k} <- s.{k} done;
-      s'
-
 (* Perform computation [c] until an exception is raised, with the dummy return
 value [r], of the type of the expression evaluted when the exception is caught.
 *)
@@ -396,8 +423,6 @@ let until_exception r c =
 let set_array a v =
   Array.fill a 0 (Array.length a) v
 
-
-
 (* Evaluate [v ()], evaluate and ignore [f ()], return [v ()], in that order. *)
 let do_return v f =
   let r = v () in ignore (f ()); r
@@ -429,7 +454,7 @@ let interleave_lists a b =
     interleave_lists_inner [] a b
 
 (* Cons on list references *)
-let ( =:: ) r e =
+let ( =| ) r e =
   r := e::!r
 
 (* Append on list references *)
@@ -443,7 +468,7 @@ let isdigit = function
 
 (* Abbreviation. \NOTE{Be sure to open [Utility] after [Bigarray] if using both
 libraries since this name clashes with [Bigarray.int].} *)
-let int = int_of_float 
+let toint x = int_of_float x
 
 (* Invert a predicate. *)
 let notpred f =
@@ -458,16 +483,13 @@ let neq = ( <> )
 let map_lol f =
   map (map f)
 
-(* Print something and then flush standard output. *)
-let flprint s =
-  print_string s; flush stdout
 
 (* Raise [x] to the power [i]. *)
-let pow i x =
-  let rec pow_inner r i x =
-    if i = 0 then r else pow_inner (r * x) (i - 1) x
-  in
-    pow_inner 1 i x
+let rec pow i x =
+  match i with
+  | 0 -> 1
+  | 1 -> x
+  | i -> pow (i / 2) (x * x) * (if i mod 2 = 0 then 1 else x)
 
 (* \section{Queues} *)
 
@@ -476,7 +498,7 @@ let pow i x =
 type 'a queue = 'a list * 'a list
 
 (* Make an empty queue *)
-let q_mk = [], []
+let q_mk = (([], []) : 'a queue)
 
 let list_of_q (a, b) =
   a @ rev b
@@ -495,7 +517,7 @@ let q_of_list l =
 
 (* Null predicate *)
 let q_null = function
-  | [], [] -> true
+  | (([], []) : 'a queue) -> true
   | _ -> false
 
 (* Raised when an attempt is made to peek or dequeue on an empty queue. *)
@@ -503,15 +525,15 @@ exception EmptyQueue
 
 (* Peek at the head *)
 let q_hd = function
-  | h::_, _ -> h
-  | _ -> raise EmptyQueue
+  | ((h::_, _) : 'a queue) -> h
+  | _ -> dpr "c"; raise EmptyQueue
 
 (* Dequeue *)  
 let q_deq = function
   | _::t, r -> q_norm (t, r)
-  | _ -> raise EmptyQueue
+  | _ -> dpr "d"; raise EmptyQueue
 
-let q_len (a, b) =
+let q_len ((a, b) : 'a queue) =
   length a + length b
 
 (* \section{Dictionaries implemented as association lists} *)
@@ -523,7 +545,7 @@ let rec lookup k' = function
 
 (* Same, but no [option] type. *)
 let rec lookup_failnull k' = function
-  | [] -> raise Not_found
+  | [] -> dpr "e"; raise Not_found
   | (k, v)::t -> if k = k' then v else lookup_failnull k' t
 
 (* Add something to a dictionary, replacing it if it's already there. *)
@@ -538,7 +560,7 @@ let add k' v d =
 (* Replace something in a dictionary, failing if it doesn't exist. *)
 let replace k' v l =
   let rec replace_inner r k' v = function
-    | [] -> raise Not_found
+    | [] -> dpr "f"; raise Not_found
     | (k, _)::t when k = k' -> r @ ((k', v)::t)
     | h::t -> replace_inner (h::r) k' v t
   in
@@ -560,7 +582,7 @@ let rec mergedict d = function
   | (k, v)::es -> mergedict (add k v d) es
 
 (* An infix operator for the composition of functions. *)
-let ( & ) a b = a b
+let ( <| ) a b = a b
 
 (* Opposite version of [@] *)
 let ( @@ ) a b = b @ a
@@ -586,16 +608,10 @@ let pairs_of_list l =
   in
     pairs_of_list_inner [] l
 
-(* Make a list from a stream. *)
-let list_of_stream s =
-  let l = ref [] in
-    Stream.iter (( =:: ) l) s;
-    rev !l
-
 let charlist_of_bytestream s =
   let l = ref [] in
     for x = stream_size s - 1 downto 0 do
-      l =:: char_of_int s.{x}
+      l =| char_of_int (sget s x)
     done;
     !l
 
@@ -621,17 +637,17 @@ let pairs l =
     pairs_inner [] l
 
 (* Predicate to test if [x] is a member of a list. *)
-let rec member = List.mem
+let mem = List.mem
 
 (* The same, with reversed arguments. *)
-let rec member' l x = member x l
+let rec mem' l x = mem x l
 
 (* Setify. Does not preserve order. *)
 let setify l =
   let rec setify_inner r = function
     | [] -> r
     | h::t ->
-        if member h t
+        if mem h t
           then setify_inner r t
           else setify_inner (h::r) t
   in
@@ -639,7 +655,7 @@ let setify l =
 
 (* The first instance of an element survives *)
 let setify_preserving_order l =
-  (setify (rev l))
+  setify (rev l)
 
 (* Remove all elts of l' from l if l, l' sets. *)
 let setminus l l' =
@@ -647,12 +663,15 @@ let setminus l l' =
     match l with
     | [] -> r
     | h::t ->
-        if member h l'
+        if mem h l'
           then setminus_inner r t l'
           else setminus_inner (h::r) t l'
   in
     setminus_inner [] l l' 
 
+let setminus_preserving_order l l' =
+  rev (setminus l l')
+
 (* Return a list of the heads of a list of lists. *)
 let heads l =
   let rec heads_inner r = function
@@ -669,7 +688,7 @@ let tails l =
     | h::t -> tails_inner (tl h::r) t
   in
     tails_inner [] l
-
+ 
 (* Take a list of lists of equal length, and turn into a list of lists, the
 first containing all the first elements of the original lists, the second the
 second, and so on. *)
@@ -723,7 +742,7 @@ let couple f l =
     | _ -> rev r
   in
     couple_inner [] f l
-
+ 
 (* As above, but an extra function [g] is applied to any last (odd) element. *)
 let couple_ext f g l =
   let rec couple_ext_inner r f g = function
@@ -769,16 +788,18 @@ let rec pair_reduce f = function
 error. *)
 let keep = List.filter
 
-let lose p = List.filter (notpred p)
+let rec lose_inner prev p = function
+  | [] -> rev prev
+  | h::t ->
+    if p h
+      then lose_inner prev p t
+      else lose_inner (h::prev) p t
+
+let lose p = lose_inner [] p
 
 (* Make a list of length [n] with each element equal to [x]. *)
 let many x n =
-  let rec many_inner x l n =
-    if n = 0 then l else many_inner x (x::l) (n - 1)
-  in
-    if n < 0
-      then raise (Invalid_argument "Utility.many")
-      else many_inner x [] n
+  Array.to_list (Array.make n x)
 
 (* A version where we need to apply unit each time, for instance when producing
 a list of random numbers. Result is ordered. *)
@@ -814,12 +835,14 @@ let takewhile p l =
 
 (* Drop [n] elements from the front of a list, returning the remainder in
 order. *)
-let rec drop l n =
+let rec drop_inner n = function
+  | [] -> raise (Invalid_argument "drop")
+  | _::t -> if n = 1 then t else drop_inner (n - 1) t
+
+let drop l n =
   if n < 0 then raise (Invalid_argument "drop") else
-    if n = 0 then l else
-      match l with
-      | [] -> raise (Invalid_argument "drop")
-      | _::t -> drop t (n - 1)
+  if n = 0 then l else
+    drop_inner n l
 
 let drop' n l = drop l n
 
@@ -853,6 +876,17 @@ let cleavewhile p l =
   in
     cleavewhile_inner p l []
 
+let cleavewhile_unordered p l =
+  let rec cleavewhile_unordered_inner p l elts =
+    match l with
+    | [] -> elts, []
+    | e::es ->
+        if p e
+          then cleavewhile_unordered_inner p es (e::elts)
+          else elts, l
+  in
+    cleavewhile_unordered_inner p l []
+
 (* Isolate a central section of a list, from the first element after the element
 for which predicate [p] is true, to the element before [p'] is first true. *)
 let isolate p p' l =
@@ -877,10 +911,10 @@ let collate cmp l =
 length $< n$). *)
 let splitinto n l =
   let rec splitinto_inner a n l =
-    try
-      let h, t = cleave l n in splitinto_inner (h::a) n t
-    with
-      Failure _ | Invalid_argument _ -> rev (if l <> [] then (l::a) else a)
+    if l = [] then rev a else
+      if length l < n then rev (l::a) else
+        let h, t = cleave l n in
+          splitinto_inner (h::a) n t
   in
     splitinto_inner [] n l
 
@@ -900,8 +934,9 @@ let select n l =
     Invalid_argument "drop" | Failure "hd"-> raise (Invalid_argument "select")
 
 (* Simple list utilities. *)
-let null = function [] -> true | _ -> false
-and notnull = function [] -> false | _ -> true
+let isnull = function [] -> true | _ -> false
+
+let notnull = function [] -> false | _ -> true
 
 (* Find the last element of a list. *)
 let rec last = function
@@ -929,7 +964,7 @@ let extremes_and_middle = function
   | h::t ->
       let m, l = cleave t (length t - 1) in
          h, m, hd l
-
+ 
 (* \section{References} *)
 
 (* Set a boolean reference. *)
@@ -1013,7 +1048,7 @@ let distance_between (px, py) (px', py') =
   sqrt (sqr (px -. px') +. sqr (py' -. py))
 
 (* \section{Functions on the integers} *)
-
+ 
 (* The largest power of two by which [n] is exactly divisible. *)
 let largest_pow2_divisible n =
   let rec s test n =
@@ -1052,15 +1087,15 @@ let compare_i (a : int) b =
 (* Reverse comparison *)
 let rev_compare a b =
   ~-(compare a b)
-
+ 
 (* The integer range between $[s..e]$ inclusive. *)
 let ilist s e =
   if e < s then raise (Invalid_argument "Utility.ilist") else
     let nums = ref [] in
       let rec ilist s e =
         if s = e
-          then nums =:: e
-          else (nums =:: s; ilist (s + 1) e)
+          then nums =| e
+          else (nums =| s; ilist (s + 1) e)
       in
         ilist s e;
         rev !nums
@@ -1082,7 +1117,7 @@ let indx0 l =
 
 let indxn n l =
   ilist n (n + length l - 1)
-
+ 
 (* \section{Miscellaneous functions} *)
 
 (* Even/odd predicates. Work for positive, negative and zero values. *)
@@ -1112,7 +1147,7 @@ let array_map2 f a b =
     Array.init (Array.length a) (function i -> f a.(i) b.(i))
   else
     raise (Invalid_argument "Utility.array_map2")
-
+ 
 (* Find the number of bytes in $n$ megabytes. *)
 let megabytes n = n * 1024 * 1024
 
@@ -1126,7 +1161,7 @@ let unopt = function
   | None -> failwith "unopt"
 
 let option_map f l =
-  map unopt & lose (eq None) & map f l
+  map unopt (lose (eq None) (map f l))
 
 (* Integer-specialised minimum and maximum functions for speed, overriding
 \emph{Pervasives.min} and \emph{Pervasives.max}. *)
@@ -1137,13 +1172,18 @@ and max (a : int) b = if a > b then a else b
 let fmin (a : float) b = if a < b then a else b
 and fmax (a : float) b = if a > b then a else b
 
-let fabs = abs_float
+let fabs x = abs_float x
 
 (* The union of two rectangles, each defined by its minimum and maximum
 coordinates *)
 let box_union (xmin0, xmax0, ymin0, ymax0) (xmin1, xmax1, ymin1, ymax1) =
   min xmin0 xmin1, max xmax0 xmax1, min ymin0 ymin1, max ymax0 ymax1
 
+(* The union of two rectangles, each defined by its minimum and maximum
+coordinates *)
+let box_union_float (xmin0, xmax0, ymin0, ymax0) (xmin1, xmax1, ymin1, ymax1) =
+  fmin xmin0 xmin1, fmax xmax0 xmax1, fmin ymin0 ymin1, fmax ymax0 ymax1
+
 (* The intersection rectangle of two rectangles defined by integers. [x0, y0]
 etc refer to the top left, [x1, y1] etc. to the bottom right. *)
 let box_overlap ax0 ay0 ax1 ay1 bx0 by0 bx1 by1 =
@@ -1184,11 +1224,23 @@ and never _ = false
 let null_hash () =
   Hashtbl.create 0
 
+(*IF-OCAML*)
+let tryfind table k =
+  try
+    Some (Hashtbl.find table k)
+  with
+    Not_found -> None
+(*ENDIF-OCAML*)
+
+(*i*)(*F#
+let tryfind = Hashtbl.tryfind
+F#*)(*i*)
+
 (* Extract all (key, value) pairs from a hash table. *)
 let list_of_hashtbl t =
   let contents = ref [] in
     Hashtbl.iter
-      (fun k v -> contents =:: (k, v))
+      (fun k v -> contents =| (k, v))
       t;
   !contents
 
@@ -1206,7 +1258,7 @@ let round x =
 (* Render a float normal by replacing anything abnormal by 0. *)
 let safe_float f =
   match classify_float f with
-  | FP_nan | FP_infinite | FP_zero | FP_subnormal -> 0.
+  | FP_nan | FP_infinite | FP_zero (*IF-OCAML*)| FP_subnormal(*ENDIF-OCAML*) -> 0.
   | _ -> f
 
 (* Build a tuple *)
@@ -1221,12 +1273,63 @@ let swap a i j =
     a.(i) <- a.(j);
     a.(j) <- t
 
-(* Print a Genlex.lexeme. *)
-let print_lexeme = function
-  | Genlex.Kwd s -> Printf.printf "KEYWORD: %s\n" s
-  | Genlex.Ident s -> Printf.printf "IDENT: %s\n" s
-  | Genlex.Int i -> Printf.printf "INT: %i\n" i
-  | Genlex.Float f -> Printf.printf "FLOAT: %f\n" f
-  | Genlex.String s -> Printf.printf "STRING: %s\n" s
-  | Genlex.Char c -> Printf.printf "CHAR: %c\n" c
+(* Print floats, integers or int32 values with spaces between them. *)
+let print_floats fs =
+  iter (fun x -> print_float x; print_string " ") fs;
+  print_newline ()
+
+let print_ints is =
+  iter (fun x -> print_int x; print_string " ") is;
+  print_newline ()
+
+let print_int32s is =
+  iter (fun x -> Printf.printf "%li " x) is;
+  print_newline ()
+
+(*i*)(*F#
+let bytearray_of_string s =
+  let b = Array.make (String.length s) (Byte.of_int 0) in
+    for x = 0 to String.length s - 1 do
+      b.[x] <- Byte.of_int (int_of_char (String.get s x))
+    done;
+    b
+
+let intarray_of_bytearray b =
+  let i = Array.make (Array.length b) 0 in
+    for x = 0 to Array.length b - 1 do
+      i.[x] <- Byte.to_int b.[x]
+    done;
+    i
+
+F#*)(*i*)
+
+let digest =
+  (*IF-OCAML*)
+  Digest.string
+  (*ENDIF-OCAML*)
+  (*i*)(*F#
+  function s ->
+    let hasher = System.Security.Cryptography.MD5.Create () in
+      string_of_int_array (intarray_of_bytearray (hasher.ComputeHash (bytearray_of_string s)))
+  F#*)(*i*)
+ 
+let slash =
+  (*IF-OCAML*)
+  match Sys.os_type with
+  | "Win32" -> "\\"
+  | _ -> "/"
+  (*ENDIF-OCAML*)
+  (*i*)(*F#
+  "\\"
+  F#*)(*i*)
+
+let leafnames_of_dir d =
+  (*IF-OCAML*)
+  Array.to_list (Sys.readdir d)
+  (*ENDIF-OCAML*)
+  (*i*)(*F#
+  map
+    (function (x : System.IO.FileInfo) -> x.Name)
+    (Array.to_list ((System.IO.DirectoryInfo d).GetFiles "*"))
+  F#*)(*i*)
 
