@@ -1,5 +1,11 @@
 (** General functions *)
 
+(** Set debug printing on or off. Off by default *)
+val dp_print : bool ref
+
+(** Debug print a string if [dp_print] is set. *)
+val dpr : string -> unit
+
 (** {2 Functions on Strings} *)
 
 (** Return the first character of a string, should it have one. Otherwise return
@@ -10,10 +16,6 @@ val firstchar : string -> char option
 [None]. *)
 val lastchar : string -> char option
 
-(** Map a string onto itself, character-by-character using the given function.
-*)
-val string_selfmap : (char -> char) -> string -> unit
-
 (** List of characters representing a string. *)
 val explode : string -> char list
 
@@ -26,16 +28,16 @@ val string_of_char : char -> string
 
 (** {2 Functions on lists} *)
 
-val hd : 'a list -> 'a
-val tl : 'a list -> 'a list
-val rev : 'a list -> 'a list
-val iter : ('a -> unit) -> 'a list -> unit
-val iter2 : ('a -> 'b -> unit) -> 'a list -> 'b list -> unit
+val hd : ('a list -> 'a)
+val tl : ('a list -> 'a list)
+val rev : ('a list -> 'a list)
+val iter : (('a -> unit) -> 'a list -> unit)
+val iter2 : (('a -> 'b -> unit) -> 'a list -> 'b list -> unit)
 val iter3 : ('a -> 'b -> 'c -> unit) -> 'a list -> 'b list -> 'c list -> unit
 val append : 'a list -> 'a list -> 'a list
-val ( @ ) : 'a list -> 'a list -> 'a list
+val ( @ ) : ('a list -> 'a list -> 'a list)
 val flatten : 'a list list -> 'a list
-val rev_map : ('a -> 'b) -> 'a list -> 'b list
+val rev_map : (('a -> 'b) -> 'a list -> 'b list)
 val map : ('a -> 'b) -> 'a list -> 'b list
 val map2 : ('a -> 'b -> 'c) -> 'a list -> 'b list -> 'c list
 val split : ('a * 'b) list -> 'a list * 'b list
@@ -46,18 +48,21 @@ val combine : 'a list -> 'b list -> ('a * 'b) list
 val combine3 : 'a list -> 'b list -> 'c list -> ('a * 'b * 'c) list
 val fold_left : ('a -> 'b -> 'a) -> 'a -> 'b list -> 'a
 val fold_right : ('a -> 'b -> 'b) -> 'a list -> 'b -> 'b
-val length : 'a list -> int
-val sort : ('a -> 'a -> int) -> 'a list -> 'a list
+val length : ('a list -> int)
+val sort : (('a -> 'a -> int) -> 'a list -> 'a list)
 (** Tail-recursive versions of list functions. See [Pervasives] for documentation. *)
 
+(** Cumulative sum of a list given an initial value. *)
 val cumulative_sum : int -> int list -> int list
 
+(** Count the number of elements matchin a predicate. *)
 val lcount : ('a -> bool) -> 'a list -> int
 
+(** Map with a function returning an option, dropping all None results and extracting all Some ones. *)
 val option_map : ('a -> 'b option) -> 'a list -> 'b list
 
 (** Synonym for [List.mem]. *)
-val member : 'a -> 'a list -> bool
+val mem : ('a -> 'a list -> bool)
 
 (** Similar to [rev_map], but 3 arguments. *)
 val rev_map3 :
@@ -113,6 +118,9 @@ val cleave : 'a list -> int -> 'a list * 'a list
 in the first returned list. e.g [cleavewhile even [2;4;5;6] ==> ([2;4], [5;6])] *)
 val cleavewhile : ('a -> bool) -> 'a list -> 'a list * 'a list
 
+(** The same, but the output lists are each unordered. *)
+val cleavewhile_unordered : ('a -> bool) -> 'a list -> 'a list * 'a list
+
 (** Isolate a central section of a list, from the first element after the element
 for which predicate [p] is true, to the element before [p'] is first true. *)
 val isolate : ('a -> bool) -> ('a -> bool) -> 'a list -> 'a list
@@ -130,7 +138,7 @@ it has already been sorted. e.g [collate [1; 2; 2; 3; 3]] calculates
 val collate : ('a -> 'a -> int) -> 'a list -> 'a list list
 
 (** Map on lists of lists. So [map_lol f] is [map (map f)]. *)
-val map_lol : ('a -> 'b) -> 'a list list -> 'b list list
+val map_lol : ('a -> 'b) -> ('a list list -> 'b list list)
 
 (** Produce a list of overlapping pairs of elements in a list in order, producing
 the empty list if on singleton input. e.g [pairs [1; 2; 3]] is [[(1, 2); (2,
@@ -138,7 +146,7 @@ the empty list if on singleton input. e.g [pairs [1; 2; 3]] is [[(1, 2); (2,
 val pairs : 'a list -> ('a * 'a) list
 
 (** [List.mem] with arguments reversed. *)
-val member' : 'a list -> 'a -> bool
+val mem' : 'a list -> 'a -> bool
 
 (** Remove duplicates from a list. *)
 val setify : 'a list -> 'a list
@@ -150,6 +158,9 @@ val setify_preserving_order : 'a list -> 'a list
 do not appear in [b]. *)
 val setminus : 'a list -> 'a list -> 'a list
 
+(** The same, but preserving order *)
+val setminus_preserving_order : 'a list -> 'a list -> 'a list
+
 (** Return a list of the heads of a list of lists, each of which has at least
 one element, preserving order. *)
 val heads : 'a list list -> 'a list
@@ -184,11 +195,11 @@ val pair_ext : ('a -> 'a -> 'b) -> ('a -> 'b) -> 'a list -> 'b list
 (** As [couple_reduce] is to [couple], so [pair_reduce] is to [pair]. *)
 val pair_reduce : ('a -> 'a -> 'a) -> 'a list -> 'a
 
-val keep : ('a -> bool) -> 'a list -> 'a list
+val keep : (('a -> bool) -> 'a list -> 'a list)
 
 (** [List.filter] has a confusing name, so we define [keep] and [lose] to avoid
 error. *)
-val lose : ('a -> bool) -> 'a list -> 'a list
+val lose : ('a -> bool) -> ('a list -> 'a list)
 
 (** [many x n] makes a list of length [n] with each element equal to [x]. *)
 val many : 'a -> int -> 'a list
@@ -201,13 +212,14 @@ val manyunique : (unit -> 'a) -> int -> 'a list
 length [< n]), preserving order. *)
 val splitinto : int -> 'a list -> 'a list list
 
+(** Split a list at the given positions. Point 1 means after the first element *)
 val splitat : int list -> 'a list -> 'a list list
 
 (** Select the nth element in a list (first is element 1). Raises
 [Invalid_argument] if the number is out-of-range. *)
 val select : int -> 'a list -> 'a
 
-val null : 'a list -> bool
+val isnull : 'a list -> bool
 
 (** Predicates on the nullness of a list. *)
 val notnull : 'a list -> bool
@@ -230,8 +242,10 @@ val extremes_and_middle : 'a list -> 'a * 'a list * 'a
 (** [ilist 2 5] returns [[2; 3; 4; 5]]. However, [ilist 5 2] raises [Invalid_argument]. *)  
 val ilist : int -> int -> int list
 
+(** Same, but return null list for ilist x x rather than [x] *)
 val ilist_null : int -> int -> int list
 
+(** Same as ilist_null, but return empty list if start > end, instead of failing *)
 val ilist_fail_null : int -> int -> int list
 
 (** [indx l] returns [1; 2; 3] if [l] has length 3, for example. *)
@@ -247,6 +261,7 @@ val indxn : int -> 'a list -> int list
 (if of even length) e.g [drop_evens [1; 2; 3; 4; 5; 6]] is [[1; 3; 5; 6]]. *) 
 val drop_evens : 'a list -> 'a list
 
+(** Same, but don't save the last element. *)
 val really_drop_evens : 'a list -> 'a list
 
 (** Remove the first, third etc. The last odd element is not saved. e.g [drop_odds
@@ -269,42 +284,42 @@ val index : ('a -> bool) -> 'a list -> int
 
 (** {2 32 and 64 bit integers} *)
 
-val i32ofi : int -> int32
-val i32toi : int32 -> int
-val i32tof : int32 -> float
-val i32add : int32 -> int32 -> int32
-val i32sub : int32 -> int32 -> int32
-val i32div : int32 -> int32 -> int32
-val i32mul : int32 -> int32 -> int32
-val lsr32 : int32 -> int -> int32
-val lsl32 : int32 -> int -> int32
-val lor32 : int32 -> int32 -> int32
-val land32 : int32 -> int32 -> int32
-val lxor32 : int32 -> int32 -> int32
-val i32succ : int32 -> int32
-val i32pred : int32 -> int32
-val i32max : 'a -> 'a -> 'a
-val i32min : 'a -> 'a -> 'a
+val i32ofi : (int -> int32)
+val i32toi : (int32 -> int)
+val i32tof : (int32 -> float)
+val i32add : (int32 -> int32 -> int32)
+val i32sub : (int32 -> int32 -> int32)
+val i32div : (int32 -> int32 -> int32)
+val i32mul : (int32 -> int32 -> int32)
+val lsr32 : (int32 -> int -> int32)
+val lsl32 : (int32 -> int -> int32)
+val lor32 : (int32 -> int32 -> int32)
+val land32 : (int32 -> int32 -> int32)
+val lxor32 : (int32 -> int32 -> int32)
+val i32succ : (int32 -> int32)
+val i32pred : (int32 -> int32)
+val i32max : ('a -> 'a -> 'a)
+val i32min : ('a -> 'a -> 'a)
 
-val i64ofi : int -> int64
-val i64toi : int64 -> int
-val i64add : int64 -> int64 -> int64
-val i64sub : int64 -> int64 -> int64
-val i64mul : int64 -> int64 -> int64
-val i64div : int64 -> int64 -> int64
-val lsr64 : int64 -> int -> int64
-val lsl64 : int64 -> int -> int64
-val lor64 : int64 -> int64 -> int64
-val i64succ : int64 -> int64
-val i64pred : int64 -> int64
-val i64max : 'a -> 'a -> 'a
-val i64min : 'a -> 'a -> 'a
+val i64ofi : (int -> int64)
+val i64toi : (int64 -> int)
+val i64add : (int64 -> int64 -> int64)
+val i64sub : (int64 -> int64 -> int64)
+val i64mul : (int64 -> int64 -> int64)
+val i64div : (int64 -> int64 -> int64)
+val lsr64 : (int64 -> int -> int64)
+val lsl64 : (int64 -> int -> int64)
+val lor64 : (int64 -> int64 -> int64)
+val i64succ : (int64 -> int64)
+val i64pred : (int64 -> int64)
+val i64max : ('a -> 'a -> 'a)
+val i64min : ('a -> 'a -> 'a)
 
 (** Shortened names for the obvious functions from [Int32] and [Int64]. *)
 
 (** {2 Efficient Queues} *)
 
-type 'a queue
+type 'a queue = 'a list * 'a list
 
 (** The empty queue *)
 val q_mk : 'a queue
@@ -324,20 +339,22 @@ val q_hd : 'a queue -> 'a
 (** Remove an element from the queue *)
 val q_deq : 'a queue -> 'a queue
 
+(** Length of a queue *)
 val q_len : 'a queue -> int
 
+(** All the elements of a queue *)
 val list_of_q : 'a queue -> 'a list
 
+(** Insert elements of list into queue. *)
 val q_of_list : 'a list -> 'a queue
 
 (** {2 Bytestreams and Streams } *)
 
 (** Big arrays of bytes. *)
-type bytestream =
-  (int, Bigarray.int8_unsigned_elt, Bigarray.c_layout) Bigarray.Array1.t
+type bytestream (*F# = Bytearray.bytearray F#*)
 
 (** Make from a given size. *)
-val mkstream : int -> bytestream
+val mkstream : (int -> bytestream)
 
 (** Size of a stream in bytes. *)
 val stream_size : bytestream -> int
@@ -348,6 +365,12 @@ val fillstream : int -> bytestream -> unit
 (** Print a bytestream. Format undefined: for debug only. *)
 val print_stream : bytestream -> unit
 
+(** Get the value at a position in a bytestream *)
+val sget : bytestream -> int -> int
+
+(** [sset s n v] sets the value n at position v in a bytestream s*)
+val sset : bytestream -> int -> int -> unit
+
 (** Make a bytestream from a string, no terminator. *)
 val bytestream_of_string : string -> bytestream
 
@@ -375,9 +398,6 @@ val string_of_int_arrays : int array list -> string
 (** A string from a single int array *)
 val string_of_int_array : int array -> string
 
-(** A string from a bytestream *)
-val string_of_stream : bytestream -> string
-
 (** Make a string from a bytestream. Fails if array is longer than
 [String.max_length]. *)
 val string_of_bytestream : bytestream -> string
@@ -434,7 +454,7 @@ val ( *.= ) : float ref -> float -> unit
 (** Operations on floating point references *)
 
 (** Cons something onto the contents of a list reference. *)
-val ( =:: ) : 'a list ref -> 'a -> unit
+val ( =| ) : 'a list ref -> 'a -> unit
 
 (** Append something to the front of the contents of a list reference. *)
 val ( =@ ) : 'a list ref -> 'a list -> unit
@@ -504,7 +524,7 @@ val fabs : float -> float
 (** Monomorphic floating-point versions of [Pervasives] functions *)
 
 val even : int -> bool
-val odd : int -> bool
+val odd : (int -> bool)
 (** Even and odd predicates on integers. Work for negative numbers. *)
 
 (** Pi. *)
@@ -541,7 +561,7 @@ val ( |&| ) : bool -> bool -> bool
 val ident : 'a -> 'a
 
 (** [iter2] on arrays. *)
-val array_iter2 : ('a -> 'b -> 'c) -> 'a array -> 'b array -> unit
+val array_iter2 : ('a -> 'b -> unit) -> 'a array -> 'b array -> unit
 
 (** [map2] on arrays. *)
 val array_map2 : ('a -> 'b -> 'c) -> 'a array -> 'b array -> 'c array
@@ -558,6 +578,9 @@ type 'a tree = Lf | Br of 'a * 'a tree * 'a tree
 (** The empty zero-sized hash table. *) 
 val null_hash : unit -> ('a, 'b) Hashtbl.t
 
+(** Option lookup on hashtables *)
+val tryfind : (('a, 'b) Hashtbl.t -> 'a -> 'b option)
+
 (** Make a list of key-value pairs reflecting the contents of a hash table. *)
 val list_of_hashtbl : ('a, 'b) Hashtbl.t -> ('a * 'b) list
 
@@ -571,8 +594,8 @@ val tuple : 'a -> 'b -> 'a * 'b
 (** [mkunit f x] gives [fun () -> f x] *)
 val mkunit : ('a -> 'b) -> 'a -> unit -> 'b
 
-(** [let ( & ) a b = a b] *)
-val ( & ) : ('a -> 'b) -> 'a -> 'b
+(** [let ( <| ) a b = a b] *)
+val ( <| ) : ('a -> 'b) -> 'a -> 'b
 
 (** Consing to each of a pair of lists at the same time. *)
 val conspair : ('a * 'b) * ('a list * 'b list) -> 'a list * 'b list
@@ -585,13 +608,12 @@ val conspairopt :
 val pairs_of_list : 'a list -> ('a * 'a) list
 
 (** Make a list of the elements of a [Stream.t]. *)
-val list_of_stream : 'a Stream.t -> 'a list
-
+(*i val list_of_stream : 'a Stream.t -> 'a list i*)
 
 (** [until_exception r c] Perform computation [c] until an exception is raised,
 with the dummy return value [r], of the type of the expression evaluted when the
 exception is caught. *)
-val until_exception : 'a -> (unit -> 'b) -> 'a
+val until_exception : 'a -> (unit -> unit) -> 'a
 
 (** Set all the values of an array. *)
 val set_array : 'a array -> 'a -> unit
@@ -600,22 +622,22 @@ val set_array : 'a array -> 'a -> unit
 val do_return : (unit -> 'a) -> (unit -> 'b) -> 'a
 
 (** [do_many f n] calls [f ()] n times. *)
-val do_many : (unit -> 'a) -> int -> unit
+val do_many : (unit -> unit) -> int -> unit
 
 (** A character is a decimal digit. *)
 val isdigit : char -> bool
 
 (** Same as [Pervasives.int_of_float] *)
-val int : float -> int
+val toint : float -> int
 
 (** Invert a predicate. *)
 val notpred : ('a -> bool) -> 'a -> bool
 
 (** Equality. *)
-val eq : 'a -> 'a -> bool
+val eq : ('a -> 'a -> bool)
 
 (** Inequality. *)
-val neq : 'a -> 'a -> bool
+val neq : ('a -> 'a -> bool)
 
 (** Like [Pervasives.compare], but the other way around. *)
 val rev_compare : 'a -> 'a -> int
@@ -626,9 +648,6 @@ val flprint : string -> unit
 (** Swaps the data at the given indexes in an array. *)
 val swap : 'a array -> int -> int -> unit
 
-(** Print a [Genlex.token] *)
-val print_lexeme : Genlex.token -> unit
-
 (**/**)
 
 type stream = { mutable pos : int; mutable data : bytestream; }
@@ -639,12 +658,29 @@ val safe_float : float -> float
 
 val box_union :
   int * int * int * int -> int * int * int * int -> int * int * int * int
- 
+
+val box_union_float :
+  float * float * float * float -> float * float * float * float -> float * float * float * float
+
 val box_overlap :
   int -> int -> int -> int -> int -> int -> int -> int ->
     (int * int * int * int) option
+
 val box_overlap_float :
   float -> float -> float -> float -> float -> float -> float -> float ->
   (float * float * float * float) option
 
+val print_floats : float list -> unit
+
+val print_ints : int list -> unit
+
+val print_int32s : int32 list -> unit
+
+val digest : string -> string
+
+val slash : string
+
+val leafnames_of_dir : string -> string list
+
+val split_around : ('a -> bool) -> 'a list -> 'a list list
 
